(
	-- Corona Batch Material Editor
	-- Author: Nick Gushchin
	-- Email: nik.gushchin@gmail.com
	-- Website: hqdetails.com
	
	/*
	##########################################################################################################################
	VARIABLES
	##########################################################################################################################
	*/
	scriptVersion = "1.7.2"
	helpLink = "https://corona-renderer.com/forum/index.php/topic,12857.0.html"

	global nsFloaterBatEd
	global nsBatEdSavedFilter
	global nsBatEdSavedMode
	global nsBatEdSavedPosition
	if nsBatEdSavedFilter == undefined then nsBatEdSavedFilter = ""
	if nsBatEdSavedMode == undefined then nsBatEdSavedMode = 1
	if nsBatEdSavedPosition == undefined then nsBatEdSavedPosition = [1500,120]
	
	rolProperty
	rolExcludeMtls
	rolRandomize
	rolTypeMap
	rolTypeFloat
	rolTypeBoolean
	rolTypeOptions
	rolTypeInteger
	rolTypeColor
	
	MtlClasses = #(CoronaMtl,CoronaLightMtl,CoronaShadowCatcherMtl,CoronaVolumeMtl,VRayMtl)
	MtlClasses = for cls in MtlClasses where findItem material.classes cls != 0 collect cls()
	MapClasses = #(BitmapTex,CoronaBitmap)
	MapClasses = for cls in MapClasses where findItem texturemap.classes cls != 0 collect classof (cls())
	selectedMtlClass = classof MtlClasses[1]
	mtlPropsListArr = #()
	
	targetMtlsArr=#()
	targetMapsArr=#()
	idsArr=#()
	outputMtlValue
	outputMapValues=#()
	valueControlSettings
	endProcessMessage = ""

	/*
	##########################################################################################################################
	PROPERTIES FUNCTIONS
	##########################################################################################################################
	*/
		
	-- GET SETTINGS FOR SOME PROPERTIES
	fn GetPropertySettings propStr = (
		maxValue = 999999
		settingsArr = case selectedMtlClass of (
			CoronaMtl: (
				case of (
					(propStr == "alphaMode"): #(#(0,2,1),#("Default","Always black","Always white"))
					(propStr == "anisotropyOrientationMode"): #(#(0,1,2),#("Local Axis","UVW"))
					(propStr == "bsdf"): #(#(0,1,2,3),#("Phong","Ward","Ashikhmin","GGX"))
					(propStr == "anisotropy"): [-1,1,0]
					(propStr == "anisotropyOrientationUvwChannel"): [0,99,1]
					(propStr == "anisotropyRotation"): [0,360,0]
					(propStr == "fresnelior"): [0,999,1.52]
					(propStr == "gBufferOverride"): [-1,99,-1]
					(propStr == "fresnelIor"): [0,999,1.6]
					(propStr == "ior"): [0,10,1.52]
					(propStr == "scatterDirectionality"): [-0.99,0.99,0]
					(propStr == "reflectGlossiness"): [0,1,1]
					(propStr == "refractGlossiness"): [0,1,1]
					(matchpattern propStr pattern:"mapamount*"): [0,100,100]
					(matchpattern propStr pattern:"level*"): [0,1,1]
				)
			)
			CoronaLightMtl: (
				case of (
					(propStr == "directionality"): [0,1,1.0]
				)
			)
			CoronaShadowCatcherMtl: (
				case of (
					(propStr == "alphaMode"): #(#(0,1,2),#("Always solid","For compositing","Always transparent"))
					(propStr == "projectionMode"): #(#(0,1,2),#("Screen projection onto geometry","Enviro projection onto geometry","Do not alter projection"))
					(propStr == "bumpAmount"): [0,999,1]
					(propStr == "reflectLevel"): [0,1,1]
					(propStr == "reflectGlossiness"): [0,1,1]
					(propStr == "reflectIor"): [0,999,1.52]
					(propStr == "shadowAmount"): [0,10,1]
				)
			)
			CoronaVolumeMtl: (
				case of (
					(propStr == "scatteringDirectionality"): [-0.99,0.99,0]
				)
			)
			VRayMtl: (
				case of (
					(propStr == #anisotropy): [-1,1,0]
					(propStr == #anisotropy_axis): #(#(0,1,2),#("X","Y","Z"))
					(propStr == #anisotropy_rotation): [-360,360,0]
					(propStr == #anisotropy_channel): [1,100,1]
					(propStr == #anisotropy_derivation): #(#(0,1),#("Local axis","Map channel"))
					(propStr == #brdf_type): #(#(0,1,2,4),#("Phong","Blinn","Ward","Microfacet GTR (GGX)"))
					(propStr == #diffuse_roughness): [0,1,0]
					(propStr == #option_cutOff): [0,1,0]
					(propStr == #option_environment_priority): [0,1000,0]
					(propStr == #option_opacityMode): #(#(0,1,2),#("Normal","Clip","Stochastic"))
					(propStr == #option_traceDiffuseAndGlossy): #(#(0,1,2),#("Never","Only for GI rays","Always"))
					(propStr == #preservationMode): #(#(0,1),#("RGB","Monochrome"))
					(propStr == #reflection_dimDistance_falloff): [0,1,0]
					(propStr == #refraction_dispersion): [1,150,50]
					(propStr == #Soften): [-1,1,0]
					(propStr == #translucency_fbCoeff): [0,1,1]
					(propStr == #translucency_scatterCoeff): [0,1,0]
					(propStr == #translucency_on): #(#(0,1,2,3),#("None","Hard (wax) model","Soft (water) model","Hybrid mode"))					
					(matchpattern propStr pattern:"*_glossiness"): [0,1,1]
					(matchpattern propStr pattern:"*_maxDepth"): [1,1000,5]
					(matchpattern propStr pattern:"*_affectAlpha"): #(#(0,1,2),#("Color only","Color+alpha","All channels"))
					(matchpattern propStr pattern:"*_subdivs"): [1,1000,8]
					(matchpattern propStr pattern:"*_ior"): [0,100,1.6]
					(matchpattern propStr pattern:"texmap_*"): [0,100,100]
				)
			)
		)
		if settingsArr == undefined then settingsArr = [maxValue*-1,maxValue,0]
		settingsArr
	)
	
	-- GET LIST OF PROPERTIES FOR ALL CLASSES
	fn GetAllProperties = (
		propClsRenameFrom = #("BooleanClass","UndefinedClass")
		propClsRenameTo = #("Bool","Map")
		excludeClasses = #(ArrayParameter)
		-- fill materials classes
		for i=1 to MtlClasses.count do (
			cls = MtlClasses[i]
			props = getPropNames cls
			pNames = #()
			for p in props do (
				if findItem excludeClasses (classof (getproperty cls p)) == 0 then (
					pclass = classof (getproperty cls p) as string
					renameItem = findItem propClsRenameFrom pclass
					if renameItem != 0 then pclass = propClsRenameTo[renameItem]				
					append pNames (pclass + " - " + (p as string))
				)
			)
			sort pNames
			mtlPropsListArr[i] = pNames
		)
	)
	
	-- GET SELECTED PROPERTIES ARRAY
	fn GetSelProps trueName:false = (
		lst = rolProperty.lstProps
		items = for i in lst.selection collect lst.items[i]
		case trueName of (
			false: items
			true: for i in items collect (filterstring i " - ")[2]
		)
	)
	
	/*
	##########################################################################################################################
	MATERIALS FUNCTIONS
	##########################################################################################################################
	*/
	
	-- GET RANDOM VALUE FROM GIVEN VALUE
	fn GetRandom val = (
		rol = rolRandomize
		case (classof val) of (
			float: (
				validMin = valueControlSettings[1]
				validMax = valueControlSettings[2]
				-- get +- value from rollout
				rndMax = rol.spnRndMax.value
				-- get and validate random value
				newVal = random (val-rndMax) (val+rndMax)
				if newVal > validMax then newVal = validMax
				else if newVal < validMin then newVal = validMin
				-- return new value
				newVal
			)
			color: (
				newColor = copy val
				-- get +- values from rollout
				hueMax = rol.spnRndColHue.value
				satMax = rol.spnRndColSat.value
				valMax = rol.spnRndColVal.value
				
				if hueMax + satMax + valMax != 0 then (
					-- get new random values
					newHue = newColor.h + (random -hueMax hueMax)
					newSat = newColor.s + (random -satMax satMax)
					newVal = newColor.v + (random -valMax valMax)
					-- validate color
					if newHue > 255 then newHue -= 255
					else if newHue < 0 then newHue += 255
					if newSat > 255 then newSat = 255
					else if newSat < 0 then newSat = 0
					if newVal > 255 then newVal = 255
					else if newVal < 0 then newVal = 0
					-- set new values
					newColor.h = newHue
					newColor.s = newSat
					newColor.v = newVal
				)
				-- return new color
				newColor
			)
			default:val
		)
	)
	
	-- SEARCH MATERIALS RECURSIVELY
	fn GetMtlsRecursive matsSearchArr = (
		for mat in matsSearchArr do (
			-- APPEND FOUNDED TARGET MATERIAL
			if classof mat == selectedMtlClass then append targetMtlsArr mat
			-- GET ALL MTLS IN MULTIMATERIAL
			else if classof mat == MultiMaterial then (
				case idsArr.count of (
					0: for submat in mat.materialList do GetMtlsRecursive #(submat)
					default: for i in idsArr where (findItem mat.materialIDList i != 0) do GetMtlsRecursive #(mat[i])
				)
			)
			-- GET ALL MTLS IN NESTED MATERAILS LIKE BLEND
			else if classof mat != UndefinedClass then (
				propNames = getpropnames mat
				propsArr = for prop in propNames collect getproperty mat prop
				matProps = for prop in propsArr where superclassof prop == material collect prop
				for submat in matProps do GetMtlsRecursive #(submat)
			)
		)
	)
	
	-- SEARCH MAPS RECURSIVELY
	fn GetMapsRecursive root mapCls = (
		subslots = getNumSubTexmaps root
		for i=1 to subslots do (
			submap = getSubTexmap root i
			if submap != undefined then (
				if classof submap == mapCls then appendIfUnique targetMapsArr submap
				else GetMapsRecursive submap mapCls
			)
		)
	)
	
	-- EXCLUDE MATERIALS BY PROPERTIES
	fn ExcludeMtls = (
		rol = rolExcludeMtls 
		chkRol = rolExcludeMtls.isDisplayed  
		chkDif = rol.chkDiffuse.checked
		chkRefl = rol.chkReflect.checked
		chkReflIor = rol.chkReflIor.checked
		chkRefr = rol.chkRefract.checked
		excludeEnabled = chkDif or chkRefl or chkReflIor or chkRefr and chkRol
		
		-- if some exclude option is enabled
		if excludeEnabled then (
			validMtls=#()
			for mat in targetMtlsArr do (
				chkValDif = true
				chkValRefl = true
				chkValReflIor = true
				chkValRefr = true
				-- diffuse avarage color
				if chkDif then (
					clr
					lvl
					case selectedMtlClass of (
						CoronaMtl: (
							clr = GetProperty mat "colorDiffuse" 
							lvl = GetProperty mat "levelDiffuse"
						)
						VrayMtl: (
							clr = GetProperty mat "Diffuse" 
							lvl = 1
						)
					)					
					avgcol = ((clr.r + clr.g + clr.b)/3*lvl) as integer
					case rol.ddlDiffuse.selection of (
						1: if avgcol > rol.spnDiffuse.value then chkValDif = false
						2: if avgcol < rol.spnDiffuse.value then chkValDif = false
					)
				)
				-- reflect level
				if chkRefl then (
					val = case selectedMtlClass of (
						CoronaMtl: GetProperty mat "levelReflect"
						VrayMtl: (
							col = GetProperty mat "Reflection"
							(col.r+col.g+col.b)/3/255
						)
					)
					case rol.ddlReflect.selection of (
						1: if val > rol.spnReflect.value then chkValRefl = false
						2: if val < rol.spnReflect.value then chkValRefl = false
					)
				)
				-- reflect ior
				if chkReflIor then (
					val = case selectedMtlClass of (
						CoronaMtl: GetProperty mat "fresnelIor"
						VrayMtl: (
							case GetProperty mat "reflection_lockIOR" of (
								true: GetProperty mat "refraction_ior"
								false: GetProperty mat "reflection_ior"
							)
						)
					)
					case rol.ddlReflIor.selection of (
						1: if val > rol.spnReflIor.value then chkValReflIor = false
						2: if val < rol.spnReflIor.value then chkValReflIor = false
					)
				)
				-- refract level
				if chkRefr then (
					val = case selectedMtlClass of (
						CoronaMtl: GetProperty mat "levelRefract"
						VrayMtl: (
							col = GetProperty mat "Refraction"
							(col.r+col.g+col.b)/3/255
						)
					)
					case rol.ddlRefract.selection of (
						1: if val > rol.spnRefract.value then chkValRefr = false
						2: if val < rol.spnRefract.value then chkValRefr = false
					)					
				)
				chkVals = chkValDif and chkValRefl and chkValReflIor and chkValRefr
				
				-- if check is passed then add material to array
				if chkVals then append validMtls mat				
			)
			targetMtlsArr = validMtls
		)
	)

	-- GET TARGET MATERIALS
	fn GetTargets getMaps:false = (
		endProcessMessage = ""
		targetMtlsArr=#()
		targetMapsArr=#()
		-- get material ids array
		idsBox = rolProperty.edtSubIds
		if idsBox.enabled then (
			integersArr = for i in filterstring idsBox.text "," collect i as integer
			idsArr = for i in integersArr where i != 0 and i != undefined collect i 
			-- fix incorrect input
			str = ""
			for i in idsArr do append str (i as string + ",")
			idsBox.text = str
		)
		-- SELECTION MODES
		selMode = rolProperty.radSelMode.state
		case selMode of (
			-- selected objects
			1: (
				if selection.count > 0 then (
					matFromSel = #()
					for obj in selection where isproperty obj "material" and obj.material != undefined do appendIfUnique matFromSel obj.material
					GetMtlsRecursive matFromSel
				)
				else endProcessMessage = "Select some objects"
			)
			-- all scene materials
			2: (
				targetMtlsArr = getClassInstances selectedMtlClass
			)
			-- medit selected materials
			3: (
				if MatEditor.isOpen() then (
					case MatEditor.mode of (
						#basic: (
							GetMtlsRecursive #(medit.GetCurMtl())
						)
						#advanced: (
							allTargetMtls = getclassinstances selectedMtlClass
							smeView = sme.GetView sme.activeView
							targetMtlsArr = for mtl in allTargetMtls where (smeView.GetNodeByRef mtl != undefined) and (smeView.GetNodeByRef mtl).selected collect mtl
						)
					)
				)
				else endProcessMessage = "Open material editor and select materials"
			)
			-- medit selected maps
			4: (
				if MatEditor.isOpen() then (
					case MatEditor.mode of (
						#basic: (
							for mapCls in MapClasses do GetMapsRecursive (medit.GetCurMtl()) mapCls
						)
						#advanced: (
							for mapCls in MapClasses do (
								allTargetMaps = getclassinstances mapCls
								smeView = sme.GetView sme.activeView
								join targetMapsArr (for map in allTargetMaps where (smeView.GetNodeByRef map != undefined) and (smeView.GetNodeByRef map).selected collect map)
							)
						)
					)
				)
				else endProcessMessage = "Open material editor and select maps"
			)
		)
		ExcludeMtls()
		
		-- if mtl mode but not medit selected maps
		if getMaps and not selMode == 4 then (
			targetMapsArr=#()
			mtlPropsArr = GetSelProps trueName:true
			for mapCls in MapClasses do (
				for mtlProp in mtlPropsArr do (
					for mtl in targetMtlsArr where GetProperty mtl mtlProp != undefined do (
						rootmap = GetProperty mtl mtlProp
						if classof rootmap == mapCls then append targetMapsArr rootmap
						else if getNumSubTexmaps rootmap > 0 then GetMapsRecursive rootmap mapCls
					)
				)
			)
		)
		-- get mtls and maps summary for message if no messages
		if endProcessMessage == "" then (
			endProcessMessage = "Materials: " + targetMtlsArr.count as string + "   Maps: " + targetMapsArr.count as string
		)
	)

	-- START BATCH EDIT PROPERTIES
	fn StartBatchEdit = undo "Batch MEdit" on (
		mapMode = rolTypeMap.isDisplayed and not rolTypeMap.chkRemoveMap.state
		case mapMode of (
			false: (
				GetTargets()
				propsArr = GetSelProps trueName:true
				-- change material properties
				for mtl in targetMtlsArr do (
					for prop in propsArr do (
						propValue = (GetRandom outputMtlValue)
						if rolTypeMap.chkRemoveMap.state then propValue = undefined
						setProperty mtl prop propValue
					)
				)
			)
			true: (
				-- get enabled map properties controls
				mapControls = for chk in rolTypeMap.controls where matchpattern (chk as string) pattern:"*:check_*" == true AND chk.state == true collect chk
				if mapControls.count > 0 then (
					-- get all maps of given class
					GetTargets getMaps:mapMode
					-- set maps parameters
					for map in targetMapsArr do (
						-- parse all enabled parameters
						for chk in mapControls do (
							-- get parameter name
							chkName = filterstring (chk as string) ":"
							case (classof map) of (
								CoronaBitmap: (
									case chkName[2] of (
										"check_RealWorld": map.realWorldScale = rolTypeMap.value_RealWorld.state
										"check_MapCh": map.uvwChannel = rolTypeMap.value_MapCh.value
										
										"check_OffsetW": map.uvwOffset.x = GetRandom rolTypeMap.value_OffsetW.value
										"check_OffsetH": map.uvwOffset.y = GetRandom rolTypeMap.value_OffsetH.value
										"check_SizeW": map.uvwScale.x = GetRandom rolTypeMap.value_SizeW.value
										"check_SizeH": map.uvwScale.y = GetRandom rolTypeMap.value_SizeH.value
										"check_Angle": map.wAngle = GetRandom rolTypeMap.value_Angle.value
										
										"check_Fblur": map.filteringBlur = rolTypeMap.value_Fblur.value
										"check_Ftyp": map.interpolation = rolTypeMap.value_Ftyp.value
										
										"check_Inv": map.output.invert = rolTypeMap.value_Inv.state
										"check_Clmp": map.output.clamp = rolTypeMap.value_Clmp.state
										"check_Argb": map.output.alphaFromRGB = rolTypeMap.value_Argb.state
										
										"check_Outam": map.output.output_amount = GetRandom rolTypeMap.value_Outam.value
										"check_Rgboff": map.output.rgb_offset = GetRandom rolTypeMap.value_Rgboff.value
										"check_Rgblvl": map.output.rgb_level = GetRandom rolTypeMap.value_Rgblvl.value
										"check_Bumpam": map.output.bump_amount = GetRandom rolTypeMap.value_Bumpam.value
									)
								)
								BitmapTex: (
									case chkName[2] of (
										"check_RealWorld": map.coords.realWorldScale = rolTypeMap.value_RealWorld.state
										"check_MapCh": map.coords.mapChannel = rolTypeMap.value_MapCh.value
										
										"check_OffsetW": map.coords.U_Offset = GetRandom rolTypeMap.value_OffsetW.value
										"check_OffsetH": map.coords.V_Offset = GetRandom rolTypeMap.value_OffsetH.value
										"check_SizeW": (
											if map.coords.realWorldScale then map.coords.realWorldWidth = GetRandom rolTypeMap.value_SizeW.value
											else map.coords.U_Tiling = GetRandom rolTypeMap.value_SizeW.value
										)
										"check_SizeH": (
											if map.coords.realWorldScale then map.coords.realWorldHeight = GetRandom rolTypeMap.value_SizeH.value
											else map.coords.V_Tiling = GetRandom rolTypeMap.value_SizeH.value
										)
										"check_Angle": map.coords.W_angle = GetRandom rolTypeMap.value_Angle.value
										
										"check_Fblur": map.coords.blur = rolTypeMap.value_Fblur.value
										"check_Ftyp": map.filtering = rolTypeMap.value_Ftyp.value
										
										"check_Inv": map.output.invert = rolTypeMap.value_Inv.state
										"check_Clmp": map.output.clamp = rolTypeMap.value_Clmp.state
										"check_Argb": map.output.alphaFromRGB = rolTypeMap.value_Argb.state
										
										"check_Outam": map.output.output_amount = GetRandom rolTypeMap.value_Outam.value
										"check_Rgboff": map.output.rgb_offset = GetRandom rolTypeMap.value_Rgboff.value
										"check_Rgblvl": map.output.rgb_level = GetRandom rolTypeMap.value_Rgblvl.value
										"check_Bumpam": map.output.bump_amount = GetRandom rolTypeMap.value_Bumpam.value
									)
								)
							)
						)
					)
				)
				else endProcessMessage = "No enabled map properties"
			)				
		)
	)
	
	/*
	##########################################################################################################################
	ROLLOUTS
	##########################################################################################################################
	*/
	
	-- UPDATE DIALOG HEIGHT
	fn UpdateDialogSize = (
		rolH = nsFloaterBatEd.rollouts.count * 25
		for r in nsFloaterBatEd.rollouts where r.open do rolH += r.height
		nsFloaterBatEd.size.y = rolH
	)

	Rollout rolProperty "Material Properties" category:1 (
		local previousTypeSettings
		local previousTypeClass
		local widFilterBtn = 50
		local tipClass = "Material class"
		local tipFilterClean = "Clear filter string"
		local tipFilterBool = "Filter Boolean properties"
		local tipFilterColor = "Filter Color properties"
		local tipFilterFloat =	"Filter Float properties"
		local tipFilterInt = "Filter Integer properties"
		local tipFilterMap = "Filter Map properties"
		local tipGo = "Start changing selected properties"
		local tipSelMode = "Where to get materials or maps"
		local tipHelp = "Open webpage with help and latest version"
		local selModeLabels = #("Selected objects","All scene materials","Medit selection - Material mode","Medit selection - Map mode")
		
		dropdownlist ddlMtlClass "" items:(for cl in MtlClasses collect cl.name) width:135 selection:1 height:10 align:#right tooltip:tipClass
		edittext edtFilter "" text:nsBatEdSavedFilter fieldWidth:90 align:#left offset:[-4,-22]
		button btnFilterClean "clear" height:16 width:30 offset:[-25,-21] tooltip:tipFilterClean
		button btnFilterBool "Bool" width:widFilterBtn across:5 align:#center tooltip:tipFilterBool
		button btnFilterColor "Color" width:widFilterBtn align:#center tooltip:tipFilterColor
		button btnFilterFloat "Float" width:widFilterBtn align:#center tooltip:tipFilterFloat
		button btnFilterInt "Int" width:widFilterBtn align:#center tooltip:tipFilterInt
		button btnFilterMap "Map" width:widFilterBtn align:#center tooltip:tipFilterMap
		multilistbox lstProps width:(nsFloaterBatEd.size.x-40) height:20
		radiobuttons radSelMode "" default:nsBatEdSavedMode columns:1 align:#left across:2 offset:[0,0] labels:selModeLabels tooltip:tipSelMode
		edittext edtSubIds "mat IDs:" fieldWidth:70 align:#right offset:[0,0] enabled:(rolProperty.radSelMode.state==1)
		button btnGo "GO" width:70 height:40 align:#right offset:[-1,-44] tooltip:tipGo
		button btnHelp "Help" height:16 align:#left across:2 tooltip:tipHelp
		label lblEndMessage "" align:#right style_sunkenedge:true width:220 height:16 offset:[0,0]

		-- UPDATE ROLLOUTS AND UI
		fn UpdateUI = (
			propsArr = GetSelProps()
			checkSelClasses = true
			checkSelSettings = true
			checkSel = true
			checkPrevCls = true
			checkPrevSets = true
			checkPrev = true

			-- get new prop classes and settings
			selectedPropClasses = for p in propsArr collect (filterstring p " - ")[1]
			selectedPropNames = for p in propsArr collect (filterstring p " - ")[2]
			selectedPropSettings = for p in selectedPropNames collect GetPropertySettings p
			-- get first class and settings to compare next to it
			newCls = selectedPropClasses[1]
			newSets = selectedPropSettings[1]
			-- check selected properties
			for s in selectedPropSettings where s != newSets do checkSelSettings = false
			for c in selectedPropClasses where c != newCls do checkSelClasses = false
			checkSel = checkSelSettings and checkSelClasses
			
			-- medit selection - map mode
			mapMode = radSelMode.state == 4
			if mapMode then (
				checkSel = true
				newCls = "Map"
			)
			
			-- check previous properties. compare old class and settings to new
			if previousTypeSettings != undefined then checkPrevSets = (previousTypeSettings == newSets)
			previousTypeSettings = newSets
			if previousTypeClass != undefined then checkPrevCls = (previousTypeClass == newCls)
			previousTypeClass = newCls
			checkPrev = checkPrevSets and checkPrevCls
			-- get settings for rollouts (range, options etc)
			valueControlSettings = newSets

			-- get new rollout based on class name
			rol = case newCls of (
				"Float":rolTypeFloat
				"Integer":(
					if (newSets == undefined) or (classof newSets == Point3) then rolTypeInteger
					else rolTypeOptions
				)
				"Bool":rolTypeBoolean
				"Color":rolTypeColor
				"Map":rolTypeMap
			)
			
			-- remove rollouts if check was failed
			if not checkSel or not checkPrev then (
				dontRemove = #(rolProperty,rolExcludeMtls,rolRandomize)
				for r in nsFloaterBatEd.rollouts do (
					if findItem dontRemove r == 0 then (
						removeRollout r nsFloaterBatEd
					)
				)
			)
			-- add rollout if check was passed
			if checkSel and rol != undefined and not rol.isDisplayed then (
				AddRollout rol nsFloaterBatEd rolledup:false
			)
			
			-- map mode - enable/disable controls
			if mapMode then (
				for ctrl in rolTypeMap.controls do ctrl.enabled = mapMode
				rolTypeMap.chkRemoveMap.checked = false
			)
			rolTypeMap.chkRemoveMap.enabled = not mapMode
			-- map mode - enable all material UI controls
			mapModeExclControls = #(radSelMode,btnGo,lblEndMessage,btnHelp)
			for ctrl in rolProperty.controls where findItem mapModeExclControls ctrl == 0 do ctrl.enabled = not mapMode
			
			-- selected objects mode - enable mat IDs
			edtSubIds.enabled = radSelMode.state == 1

			-- exclude rollout
			case (selectedMtlClass != CoronaMtl and selectedMtlClass != VrayMtl) or mapMode of (
				true: RemoveRollout rolExcludeMtls nsFloaterBatEd
				false: AddRollout rolExcludeMtls nsFloaterBatEd rolledup:true
			)
			
			-- randomize rollout enable controls by class
			rolRandomize.spnRndColHue.enabled = rolTypeColor.isDisplayed
			rolRandomize.spnRndColSat.enabled = rolTypeColor.isDisplayed
			rolRandomize.spnRndColVal.enabled = rolTypeColor.isDisplayed
			rolRandomize.spnRndMax.enabled = rolTypeFloat.isDisplayed or rolTypeMap.isDisplayed
			-- update dialog height
			UpdateDialogSize()
			-- prevent unnecessary go press when properties are not similar
			if lstProps.selection.numberSet == 0 then (
				btnGo.enabled = false
				lblEndMessage.text = "Select some properties"
			)
			else (
				lblEndMessage.text = ""
				btnGo.enabled = checkSel
			)
			-- empty output value and set message if properties are not similar
			if not checkSel then (
				outputMtlValue = undefined
				lblEndMessage.text = "Properties are not similar"
			)
		)
		
		-- UPDATE LIST BY FILTER
		fn UpdateListByFilter = (
			if ddlMtlClass.selection != 0 then (
				allProps = mtlPropsListArr[ddlMtlClass.selection]
				lstProps.items = for n in allProps where matchpattern n pattern:("*"+edtFilter.text+"*") == true collect n
				if lstProps.items.count != 0 then lstProps.selection = 1
			)
			UpdateUI()
		)
		-- FILTER PRESETS BUTTONS
		fn FilterBtn txt = (
			edtFilter.text = txt
			UpdateListByFilter()
			setFocus lstProps
		)
		
		-- EVENTS
		on rolProperty open do GetAllProperties()
		on rolProperty close do (
			-- save settings for max session
			nsBatEdSavedPosition = nsFloaterBatEd.pos
			nsBatEdSavedFilter = edtFilter.text
		)
		on rolProperty rolledUp state do UpdateDialogSize()
		on ddlMtlClass selected i do (
			selectedMtlClass = classof MtlClasses[i]
			UpdateListByFilter()
		)		
		on edtFilter changed theText do UpdateListByFilter()
		on btnFilterBool pressed do FilterBtn "Bool *"
		on btnFilterColor pressed do FilterBtn "Color *"
		on btnFilterFloat pressed do FilterBtn "Float *"
		on btnFilterInt pressed do FilterBtn "Integer *"
		on btnFilterMap pressed do FilterBtn "Map *"
		on btnFilterClean pressed do FilterBtn ""
		
		on lstProps selectionend do UpdateUI()
		on radSelMode changed state do (
			if state != nsBatEdSavedMode then UpdateUI()
			-- save previous selected mode
			nsBatEdSavedMode = state
		)
		on btnHelp pressed do shellLaunch helpLink ""
		on btnGo pressed do (
			StartBatchEdit()
			lblEndMessage.text = endProcessMessage
		)
	)
	
	Rollout rolExcludeMtls "Exclude" category:2 (
		local radOffset = [40,0]
		local radOffsets = #([0,0],[8,0])
		local ddlItems = #("More","Less ")
		local ddlWidth = 50
		local ddlOffset = [0,-3]
		local tipDiffuse = "Exclude materials with average diffuse color more/less than value"
		local tipReflect = "Exclude materials with reflect level more/less than value"
		local tipReflIor = "Exclude materials with reflect ior more/less than value"
		local tipRefract = "Exclude materials with refract level more/less than value"
		
		group "Exclude materials with:" (
			checkbox chkDiffuse "Diffuse avg color:" across:3 align:#left tooltip:tipDiffuse
			dropdownlist ddlDiffuse "" items:ddlItems width:ddlWidth align:#right offset:ddlOffset
			spinner spnDiffuse "than: " range:[0,255,5] align:#right type:#integer fieldwidth:40

			checkbox chkReflect "Reflect level:" across:3 align:#left tooltip:tipReflect
			dropdownlist ddlReflect "" items:ddlItems width:ddlWidth align:#right offset:ddlOffset
			spinner spnReflect "than: " range:[0,1,0.1] align:#right type:#float fieldwidth:40	
			
			checkbox chkReflIor "Reflect IOR:" across:3 align:#left tooltip:tipReflIor
			dropdownlist ddlReflIor "" items:ddlItems width:ddlWidth align:#right offset:ddlOffset
			spinner spnReflIor "than: "range:[1,999,1.6] align:#right type:#float fieldwidth:40
			
			checkbox chkRefract "Refract level:" across:3 align:#left tooltip:tipRefract
			dropdownlist ddlRefract "" items:ddlItems width:ddlWidth align:#right offset:ddlOffset
			spinner spnRefract "than: "range:[0,1,0.1] align:#right type:#float fieldwidth:40
		)
		-- EVENTS
		on rolExcludeMtls rolledUp state do UpdateDialogSize()
	)
	
	Rollout rolRandomize "Randomize" category:3 (
		spinner spnRndMax "Any value + - :" range:[0,999999,0] type:#float fieldwidth:40 align:#center offset:[-27,0]
		group "Color + - :" (
			spinner spnRndColHue "Hue:"range:[0,255,0] type:#integer fieldwidth:40 across:3 align:#left
			spinner spnRndColSat "Sat:"range:[0,255,0] type:#integer fieldwidth:40 align:#center
			spinner spnRndColVal "Val:"range:[0,255,0] type:#integer fieldwidth:40 align:#right
		)
		-- EVENTS
		on rolRandomize rolledUp state do UpdateDialogSize()
	)

	Rollout rolTypeMap "Bitmap Properties" (
		local spnW1 = 40
		local chkBtnW = 16
		local chkBtnH = 16
		local maxVal = 999999
		local tipFilter = "Bitmap:\n  0-Pyramidal\n  1-Summed Area\n  2-None \n\nCoronaBitmap:\n  0-Bilinear\n  1-Bicubic\n  2-Nearest Neighbour"
		local tipRemove = "Selected properties (maps slots) will be empty. \nAll maps in slot will be removed. \nOperation is undoable."
		local enabledCtrls = #()
		
		group "Mapping :" (
			checkbutton check_RealWorld width:chkBtnW height:chkBtnH align:#left across:4
			checkbox value_RealWorld "Real-World" align:#left offset:[-47,0]
			spinner value_MapCh "Map Channel:" range:[1,99,1] type:#integer fieldwidth:25 align:#right offset:[47,0]
			checkbutton check_MapCh "" width:chkBtnW height:chkBtnH align:#right
			
			label lblOffset "Offset:" offset:[0,0] across:2
			label lblSize "Size:" offset:[-50,0]
			
			label lblWidth "U:" align:#left offset:[21,0] across:5
			spinner value_OffsetW range:[-maxVal,maxVal,0] type:#float fieldwidth:spnW1 offset:[-15,0]
			checkbutton check_OffsetW width:chkBtnW height:chkBtnH offset:[-31,0]
			spinner value_SizeW range:[-maxVal,maxVal,1] type:#float fieldwidth:spnW1 offset:[-35,0]
			checkbutton check_SizeW width:chkBtnW height:chkBtnH offset:[-51,0]
			
			label lblHeight "V:" align:#left offset:[21,0] across:5
			spinner value_OffsetH range:[-maxVal,maxVal,0] type:#float fieldwidth:spnW1 offset:[-15,0]
			checkbutton check_OffsetH width:chkBtnW height:chkBtnH offset:[-31,0]
			spinner value_SizeH range:[-maxVal,maxVal,1] type:#float fieldwidth:spnW1 offset:[-35,0]
			checkbutton check_SizeH width:chkBtnW height:chkBtnH offset:[-51,0]
			
			label lblAngle "Angle:"	align:#left across:3
			spinner value_Angle range:[-maxVal,maxVal,0] type:#float fieldwidth:spnW1 align:#left offset:[-49,0]
			checkbutton check_Angle "" width:chkBtnW height:chkBtnH align:#left offset:[-83,0]
		)
		group "Filtering :" (
			label lblBlur "Blur:" align:#left across:5
			spinner value_Fblur range:[0.01,100,1] type:#float fieldwidth:spnW1 align:#left offset:[-14,0]
			checkbutton check_Fblur "" width:chkBtnW height:chkBtnH align:#left offset:[-13,0]
			spinner value_Ftyp "Type:" range:[0,2,0] type:#integer fieldwidth:spnW1 tooltip:tipFilter align:#right offset:[34,0]
			checkbutton check_Ftyp "" width:chkBtnW height:chkBtnH align:#right
		)
		group "Output :" (
			checkbutton check_Inv width:chkBtnW height:chkBtnH align:#left across:4
			checkbox value_Inv "Invert" align:#left offset:[-47,0]
			spinner value_Outam "Output Amount:" range:[-9999,9999,1] type:#float fieldwidth:spnW1 align:#right offset:[47,0]
			checkbutton check_Outam width:chkBtnW height:chkBtnH align:#right

			checkbutton check_Clmp width:chkBtnW height:chkBtnH align:#left across:4
			checkbox value_Clmp "Clamp" align:#left offset:[-47,0]
			spinner value_Rgboff "RGB Offset:" range:[-9999,9999,0] type:#float fieldwidth:spnW1 align:#right offset:[47,0]
			checkbutton check_Rgboff width:chkBtnW height:chkBtnH align:#right

			checkbutton check_Argb width:chkBtnW height:chkBtnH align:#left across:4
			checkbox value_Argb "Alpha from RGB" align:#left offset:[-47,0]
			spinner value_Rgblvl "RGB Level:" range:[-9999,9999,1] type:#float fieldwidth:spnW1 align:#right offset:[47,0]
			checkbutton check_Rgblvl width:chkBtnW height:chkBtnH align:#right

			spinner value_Bumpam "Bump Amount:" range:[-9999,9999,1] type:#float fieldwidth:spnW1 align:#right offset:[112,0] across:2
			checkbutton check_Bumpam width:chkBtnW height:chkBtnH align:#right
		)
		
		checkbox chkRemoveMap "Clear map slot" align:#left tooltip:tipRemove across:2
		label lblRemoveMap "Caution! Read tooltip." align:#right visible:false
		
		-- EVENTS
		on value_RealWorld changed v do check_RealWorld.state = true
		on value_MapCh changed v do check_MapCh.state = true
		on value_OffsetW changed v do check_OffsetW.state = true
		on value_OffsetH changed v do check_OffsetH.state = true
		on value_SizeW changed v do check_SizeW.state = true
		on value_SizeH changed v do check_SizeH.state = true
		on value_Angle changed v do check_Angle.state = true
		on value_Fblur changed v do check_Fblur.state = true
		on value_Ftyp changed v do check_Ftyp.state = true
		on value_Inv changed v do check_Inv.state = true
		on value_Clmp changed v do check_Clmp.state = true
		on value_Argb changed v do check_Argb.state = true
		on value_Outam changed v do check_Outam.state = true
		on value_Rgboff changed v do check_Rgboff.state = true
		on value_Rgblvl changed v do check_Rgblvl.state = true
		on value_Bumpam changed v do check_Bumpam.state = true
		
		on chkRemoveMap changed state do (
			if state then (
				-- store enabled controls
				enabledCtrls = for ctrl in rolTypeMap.controls collect #(ctrl,ctrl.enabled)
				lblRemoveMap.visible = true
				for ctrl in rolTypeMap.controls where ctrl != chkRemoveMap and ctrl != lblRemoveMap do ctrl.enabled = false
			)
			else (
				lblRemoveMap.visible = false
				for ctrl in enabledCtrls do ctrl[1].enabled = ctrl[2]
			)
		)
	)
	
	Rollout rolTypeFloat "Float" (
		spinner spnFloat "" range:valueControlSettings fieldwidth:50 align:#center
		-- EVENTS
		on rolTypeFloat open do outputMtlValue = spnFloat.value
		on spnFloat changed val do outputMtlValue = val
		on rolTypeFloat rolledUp state do UpdateDialogSize()
	)
	
	Rollout rolTypeBoolean "Boolean" (
		checkbox chkBool "Enable" checked:false align:#center
		-- EVENTS
		on rolTypeBoolean open do outputMtlValue = chkBool.checked
		on chkBool changed state do outputMtlValue = state
		on rolTypeBoolean rolledUp state do UpdateDialogSize()
	)
	
	Rollout rolTypeOptions "Options" (
		radiobuttons radOptions labels:valueControlSettings[2] columns:1 align:#center
		-- EVENTS
		on rolTypeOptions open do outputMtlValue = valueControlSettings[1][radOptions.state]
		on radOptions changed state do outputMtlValue = valueControlSettings[1][state]			
		on rolTypeOptions rolledUp state do UpdateDialogSize()
	)
	
	Rollout rolTypeInteger "Integer" (
		spinner spnInteger "" range:valueControlSettings type:#integer fieldwidth:30 align:#center
		-- EVENTS
		on rolTypeInteger open do outputMtlValue = spnInteger.value
		on spnInteger changed val do outputMtlValue = val			
		on rolTypeInteger rolledUp state do UpdateDialogSize()
	)
	
	Rollout rolTypeColor "Color" (
		colorpicker clrPick "" color:(color 255 255 255) align:#center
		-- EVENTS
		on rolTypeColor open do outputMtlValue = clrPick.color
		on clrPick changed clr do outputMtlValue = clr
		on rolTypeColor rolledUp state do UpdateDialogSize()
	)	
	
	-- CLOSE AND CREATE FLOATER
	try (CloseRolloutFloater nsFloaterBatEd) catch()
	nsFloaterBatEd = NewRolloutFloater ("Corona Batch MEdit " + scriptVersion) 300 500 nsBatEdSavedPosition.x nsBatEdSavedPosition.y
	AddRollout rolProperty nsFloaterBatEd
	AddRollout rolRandomize nsFloaterBatEd rolledup:true
	rolProperty.UpdateListByFilter()
)