-- Corona Scatter Lister v1.00
-- Stepan Hojdar

(
	global returnObject
	global enableAll
	global selectAll
	global updateAll
	global pressUpdateBtn
	global updateAllPress
	
	global userSelection = #() -- list of currently selected CScatters, should be quicker than asking 3DSMax each time selection changes
	global scatterObjects = #() -- an array of references to CScatter objects in the scene

	global countEnabled = 0 -- how many CScatters are enabled, to manage the state of 'Enable All' button (if all are, the button needs to be pressed, if at least one isn't, the button needs to be de-pressed)
	global countAutoUpdate = 0 -- same as countEnabled, keeps track of how many CScatters are set on automatic updates
	
	global ScatterListRollout -- the main Rollout needs to be global for all the callbacks to access it
	global MainListerWindow -- needs to be global for all callbacks to access it
	
	local createHandlers
	local createMainRollout 
	local setGUIValues

	local refreshWindow -- boolean if we should refresh the window after node deletion, see whenDeletedPre()/Post() for more details
	
	-- callbacks need to be global so the callbacks.AddScript() sees them
	global whenAdded
	global whenDeletedPre
	global whenDeletedPost
	global whenReset
	
	global whenClosedCleanup
	local cleanUpBool -- boolean if we should run the cleanUp script, see whenClosedCleanup() for details
	
	local enableColor = [30,131,231] -- the color of Enable and EnableAll checkbuttons
	local selectColor = [195,105,15] -- the color of Select and SelectAll checkbuttons
	
	global previzCount=#(0,0,0,0,0,0) -- array containing how many Scatters of each PrevizType there are (first element represents none, last pointCloud, etc..)
	global scatterPrevizSetting=#() -- an array containing of which previzType each scatter is (first element represents first Scatter, etc..)
	
	global setPrevizAll -- a function for agregated previzType checkbuttons
	global UIAutoUpdate = true -- boolean holding if the program autoUpdates the UI, tied to the UI's checkbox
	global changedNotUpdated = 0 -- a counter of how many CScatter objects changed since the last update of the script (it has no meaning if AutoUpdate is set on TRUE)
	
	global deletedScatters = #() -- an array containing indexes of deleted scatters from the scatterObjects array, resets every UI refresh
	
	fn returnObject ind = ( -- used by handlers of GUI elements (ie. checkboxes for 'enable', edittexts for 'name') to return a reference to their CScatter to change its values
		scatterObjects[ind]
	)

	fn addObject &obj: = ( -- adds the reference to the CScatter to the 'scatterObjects' list
		append scatterObjects obj
	)
	
	fn pressUpdateBtn = ( -- gets called when the 'Update now' is pressed
		max modify mode		
		CommandPanelHWND = for c in (windows.getChildrenHWND (windows.getDesktopHWND())) where c[5] == "Command Panel" do ( exit with c[1] )
		UpdateButton = (windows.getChildHWND CommandPanelHWND "Update now")[1]
		UIAccessor.PressButton UpdateButton
	)

	fn setPrevizAll val = ( -- gets called when one of the aggregated PrevizType buttons are clicked
		for ob in objects where classof ob == CScatter do ( -- set all Scatters to the designated previzType
			if( (findItem scatterObjects ob) != 0) then ( ob.previzType = val )
		)
		
		-- for simplicity - set all PrevizTypes to unchecked
		for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:"CBPreviz*" do (
			if(ctrl.enabled == true) then ( ctrl.checked = false )
		)
		
		-- get all the checkbuttons for the selected previzType and set them on
		for cycleInd=1 to scatterObjects.count do
		(
			for c in ScatterListRollout.controls where matchPattern c.name pattern:("CBPreviz"+(cycleInd as string)+"_"+(val as string)) do (
				if(c.enabled == true) then ( c.checked = true )
			)
			scatterPrevizSetting[cycleInd] = val -- also updating the array that holds a previzType for each scatter
		)
		
		-- check the clicked checkbutton
		for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("CBPrevizAll"+(val as string)) do (
			 ctrl.checked = true
		)
		
		if(val == 5) then ( -- if we set all the scatters to Point Cloud, we enable the points % UI control
			setPtsFractionEnabled = true
		)
		else ( setPtsFractionEnabled = false)
		
		for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("SpinnerPointFr*") do (
			workingControl = true
			for delItems in deletedScatters do
			(
				if(matchPattern ctrl.name pattern:("SpinnerPointFr"+(delItems as string)+"*") ) then
				( 
					workingControl = false
				)
			) 
			if(workingControl) then (
			ctrl.enabled = setPtsFractionEnabled )
		)
		
		-- update the previz Counters
		previzCount=#(0,0,0,0,0,0)
		previzCount[val+1] = (scatterObjects.count - deletedScatters.count)
	)
	
	fn updateAllPress = ( -- called when the 'Update All' is pressed 
		userSelectionBackup = copy userSelection #nomap -- since we need to modify selection for pressUpdateBtn() to work, we backup the user's selection first
		selectedSmt = false
		for o in objects where classof o == CScatter do (
			if( (findItem scatterObjects o) != 0) then (
			select o
			pressUpdateBtn()
			selectedSmt = true
			)
		)
		if (userSelectionBackup.count == 0) then ( if(selectedSmt == true) then ( deselect $ ) )else select userSelectionBackup
	)
	
	fn enableAll val = ( -- gets called when the 'Enable All' button is pressed
		for c in ScatterListRollout.controls where matchPattern c.name pattern:"CBEna*" do (
			if(c.enabled == true) then c.checked = val
		)
	
		if val == true then (
			countEnabled = (scatterObjects.count - deletedScatters.count)
		)
		else (
			countEnabled = 0
		)
		
		for o in objects where classof o == CScatter do (
			if( (findItem scatterObjects o) != 0) then ( o.enable = val )
		)
	)

	fn selectAll val = ( -- gets called when the 'Select All' button is pressed
		for c in ScatterListRollout.controls where matchPattern c.name pattern:"CBSel*" do (
			if(c.enabled == true) then c.checked = val
		)

		if val == false then (
			userSelection = #()
			i = 1
			for obj in scatterObjects do (
				if( (findItem deletedScatters i) == 0) then (
				deselect obj )
				i += 1
			)				
		)
		else (
			userSelection=#()
			for o in objects where classof o == CScatter do (
				if( (findItem scatterObjects o) != 0) then ( append userSelection o )
			)
			select userSelection
		)
	)
	
	fn updateAll val = ( -- gets called when the 'Update All' button is pressed
		for c in ScatterListRollout.controls where matchPattern c.name pattern:"CBUp*" do (
			if(c.enabled == true) then c.checked = val
		)
		
		if val == true then (
			countAutoUpdate = (scatterObjects.count)
		)
		else (
			countAutoUpdate = 0
		)

		for o in objects where classof o == CScatter do (
			if( (findItem scatterObjects o) != 0) then ( o.automaticUpdates = val )
		)
	)

	-- Rollout Creation functions
	-- the main method of creating the main rollout is addListerRollout() which:
	-- 	creates a new dynamic rollout
	--	if we don't have any CScatter, writes the "No scatters" text, otherwise adds the labels, the first line of control elements (using the function addInitLabel() )
	-- 	and then adds a control row for each CScatter (using the addRow() function)

	-- a function that adds a row of Control Elements for each CScatter
	-- The structure is: for each element call rci.addControl(), then create the handler code for the GUI-element, then assign the handler using rci.addHandler()
	fn addRow rci i &ob: = (  
		-- we create offsets for the elemets in the window to manage the position of the elements easier
		offsets=#("[-10,0]","[-27,0]","[-50,0]","[-77,0]","[37,0]","[20,0]","[3,0]","[-14,0]","[-31,0]","[-48,0]","[-63,0]","[-55,0]","[-47,0]","[-40,0]","[-20,0]","[-10,0]","[0,0]","[1,0]")

		-- The ComboButton for Enabling the CScatter -- 
		rci.addControl #checkbutton (("CBEnabled"+(i as string) as string) as name) "e" paramStr:("tooltip:\"Enable or disable the Scatter.\" height:16 highlightColor:"+(enableColor as string)+"  offset:"+offsets[1]+"across:18 align:#left checked:"+(ob.enable as string))
		handler_code="o = (returnObject " + (i as string) +") \n o.enable = val\n\
		if (val == true) then countEnabled = countEnabled + 1\
		else (countEnabled = countEnabled - 1)\
		if(countEnabled < (scatterObjects.count - deletedScatters.count)) then (newDynamicRollout.CBEnableAll.checked = false)\
		else (newDynamicRollout.CBEnableAll.checked = true)"
		rci.addHandler (("CBEnabled"+(i as string) as string) as name) #changed paramStr:"val" codeStr:handler_code

		-- The ComboButton for Selecting the CScatter
		rci.addControl #checkbutton (("CBSelected"+(i as string) as string) as name) "s" paramStr:("tooltip:\"Select or deselect the Scatter.\" height:16 highlightColor:"+(selectColor as string)+" offset:"+offsets[2]+"align:#left checked:true")
		select_handler_code="\
		\
		if val == true then (\
			appendIfUnique userSelection (returnObject " + (i as string) +")\n  \
			select userSelection\n \
			if userSelection.count == (scatterObjects.count - deletedScatters.count) then (newDynamicRollout.CBSelectAll.checked = true)\n\
			else (newDynamicRollout.CBSelectAll.checked = false)\n\
		) else ( \
			deleteItem userSelection (findItem userSelection (returnObject " + (i as string) +"))\n\
			deselect (returnObject " + (i as string) +")\
			if userSelection.count < (scatterObjects.count - deletedScatters.count) then(newDynamicRollout.CBSelectAll.checked = false)\n\
			else (newDynamicRollout.CBSelectAll.checked = true))"

		rci.addHandler (("CBSelected"+(i as string) as string) as name)  #changed paramStr:"val" codeStr:select_handler_code

		-- The color-picker for managing the color of the CScatter
		rci.addControl #colorpicker (("ColorPicker"+(i as string) as string) as name) "" paramStr:("tooltip:\"Change the color of the Scatter.\" height:16 title:\"Wireframe color in the viewport\" width:15 offset:"+offsets[3]+" color:"+((ob.wireColor as Point3) as string))
		colorPickerCode="o = (returnObject "+ (i as string) +") \n o.wireColor = newColor"
		rci.addHandler (("ColorPicker"+(i as string) as string) as name)  #changed paramStr:"newColor" codeStr:colorPickerCode
		
		-- Edittext GUI element for the CScatter's name
		rci.addControl #edittext (("EdittextName"+(i as string) as string) as name) ("" as string) paramStr:("tooltip:\"Name of the Scatter.\" width:150 offset:"+offsets[4]+" align:#left text:\""+ob.name+"\"")
		edittext_code="o = (returnObject "+ (i as string) +") \n o.name = text"
		rci.addHandler (("EdittextName"+(i as string) as string) as name) #entered paramStr:"text" codeStr:edittext_code

		-- A ComboButton for each of the previzType options (None, Point, Full, Box, etc...)
		obj = returnObject i
		for previzInd=0 to 5 do (
			if(previzInd == obj.previzType) then (
				checkedBool = true
			)
			else (
				checkedBool = false
			)

			rci.addControl #checkbutton (("CBPreviz"+(i as string)+"_"+(previzInd as string) as string) as name) " " paramStr:("tooltip:\"The currently selected Previz type is marked in green - click any grey box to change this Scatter to the selected Previz type.\" height:16 highlightColor:[0,170,2] offset:"+offsets[5+previzInd]+"align:#left checked:"+(checkedBool as string))
			previzTypeHandler="\
			if state == off then(\
				  "+("CBPreviz"+(i as string)+"_"+(previzInd as string) as string)+".checked = true)\
				else \
				(  o = (returnObject "+ (i as string) +") \n o.previzType = "+(previzInd as string)+" \
					for ct in newDynamicRollout.controls where matchPattern ct.name pattern:(\"CBPreviz"+ (i as string) +"_*\") do \
					(\
						if (ct != "+("CBPreviz"+(i as string)+"_"+(previzInd as string) as string)+") then ct.checked= false\
					)\
					;preType = scatterPrevizSetting["+(i as string)+"];\
					;previzCount[preType+1] -= 1;\
					;previzCount["+(previzInd as string)+"+1] += 1;\
					;scatterPrevizSetting["+(i as string)+"] = o.previzType;\
					if (previzCount[1] == scatterObjects.count - deletedScatters.count) then (newDynamicRollout.CBPrevizAll0.checked = true ) else (newDynamicRollout.CBPrevizAll0.checked = false) \
					if (previzCount[2] == scatterObjects.count - deletedScatters.count) then (newDynamicRollout.CBPrevizAll1.checked = true ) else (newDynamicRollout.CBPrevizAll1.checked = false)\
					if (previzCount[3] == scatterObjects.count - deletedScatters.count) then (newDynamicRollout.CBPrevizAll2.checked = true ) else (newDynamicRollout.CBPrevizAll2.checked = false)\
					if (previzCount[4] == scatterObjects.count - deletedScatters.count) then (newDynamicRollout.CBPrevizAll3.checked = true ) else (newDynamicRollout.CBPrevizAll3.checked = false)\
					if (previzCount[5] == scatterObjects.count - deletedScatters.count) then (newDynamicRollout.CBPrevizAll4.checked = true ) else (newDynamicRollout.CBPrevizAll4.checked = false)\
					if (previzCount[6] == scatterObjects.count - deletedScatters.count) then (newDynamicRollout.CBPrevizAll5.checked = true ) else (newDynamicRollout.CBPrevizAll5.checked = false);"
				if(previzInd == 5) then (
				previzTypeHandler += "for ct in newDynamicRollout.controls where matchPattern ct.name pattern:(\"SpinnerPointFr"+ (i as string)+ "\") do ( ct.enabled = true );"	
				)
				else ( previzTypeHandler += "for ct in newDynamicRollout.controls where matchPattern ct.name pattern:(\"SpinnerPointFr"+ (i as string) + "\") do ( ct.enabled = false );"	)
				previzTypeHandler += ")"
			rci.addHandler (("CBPreviz"+(i as string)+"_"+(previzInd as string) as string) as name) #changed paramStr:"state" codeStr:previzTypeHandler
		)

		-- Spinner for displayedFraction control
		rci.addControl #spinner (("SpinnerDisplayFr"+(i as string) as string) as name) "" paramStr:("tooltip:\"The fraction of scattered objects displayed in the viewport.\" offset:"+offsets[11]+" width:50 range:[0,100,"+(ob.displayedFraction as string)+"] align:#center type:#float")
		spinnerHandlerCode="o = (returnObject " + (i as string) +") \n o.displayedFraction = (val as float)"
		rci.addHandler (("SpinnerDisplayFr"+(i as string) as string) as name) #changed paramStr:"val" codeStr:spinnerHandlerCode

		-- Spinner for points cloud percentage control
		if(obj.previzType == 5) then (ptsEnabled = true) else (ptsEnabled = false)
		rci.addControl #spinner (("SpinnerPointFr"+(i as string) as string) as name) "" paramStr:("tooltip:\"The fraction of points to display in the point cloud representations of each instance.\" offset:"+offsets[12]+" width:50  enabled:"+(ptsEnabled as string)+" range:[0,100,"+(ob.pointCloudFraction as string)+"] align:#center type:#float")
		spinnerPointHandlerCode="o = (returnObject " + (i as string) +") \n o.pointCloudFraction = (val as float)"
		rci.addHandler (("SpinnerPointFr"+(i as string) as string) as name) #changed paramStr:"val" codeStr:spinnerPointHandlerCode

		-- Count/Density control block
		-- turn on/off density, count is always enabled
		obj = returnObject i
		-- the Count/Density radio and spinners in the 'Scattering' tab
		rci.addControl #checkbutton (("CBinstancesMode"+(i as string) as string) as name) "C/D" paramStr:("tooltip:\"When enabled, number of instances is specified by \\\"Density\\\" per area/volume unit. Maximum number of instances is always limited by \\\"Count\\\".\" height:16 offset:"+offsets[13]+" align:#left checked:"+(obj.instancesUseDensity as string))
		instanceModeController="o = (returnObject " + (i as string) +") \n  if (val == true) then (o.instancesUseDensity = true \n newDynamicRollout.SpinnerDensity"+(i as string)+".enabled = true) else (o.instancesUseDensity = false \n newDynamicRollout.SpinnerDensity"+(i as string)+".enabled = false)"
		rci.addHandler (("CBinstancesMode"+(i as string) as string) as name) #changed paramStr:"val" codeStr:instanceModeController
		
		-- Count controlling spinner
		rci.addControl #spinner (("SpinnerCount"+(i as string) as string) as name) "" paramStr:("tooltip:\"Specifies the maximum number of instances generated.\" offset:"+offsets[14]+" width:70 scale:1 range:[1,1000000000,"+( obj.count as string)+"] align:#center type:#integer enabled:true")
		spinnerCountHandler="o = (returnObject " + (i as string) +") \n o.count = (val as integer)"
		rci.addHandler (("SpinnerCount"+(i as string) as string) as name) #changed paramStr:"val" codeStr:spinnerCountHandler
		
		-- Density controlling spinner
		rci.addControl #spinner (("SpinnerDensity"+(i as string) as string) as name) "" paramStr:("tooltip:\"If enabled, specifies the number of instances generated.\" offset:"+offsets[15]+" width:50 scale:0.005 range:[0,1000000000,"+(obj.density as string)+"] align:#center type:#float enabled:"+(obj.instancesUseDensity as string))
		spinnerDensityHandler="o = (returnObject " + (i as string) +") \n o.density = (val as float)"
		rci.addHandler (("SpinnerDensity"+(i as string) as string) as name) #changed paramStr:"val" codeStr:spinnerDensityHandler

		-- Spinner for random seed
		rci.addControl #spinner (("SpinnerRndSeed"+(i as string) as string) as name) "" paramStr:("tooltip:\"Determines the particular random distribution used. Changing the value makes random permutations of the scattered objects.\" offset:"+offsets[16]+" width:50 range:[1,100000000000,"+(ob.seed as string)+"] align:#center type:#integer")
		rndSeedHandlerCode="o = (returnObject " + (i as string) +") \n o.seed = (val as float)"
		rci.addHandler (("SpinnerRndSeed"+(i as string) as string) as name) #changed paramStr:"val" codeStr:rndSeedHandlerCode

		
		-- A ComboButton for the 'Update automatically' checkbox
		rci.addControl #checkbutton (("CBUpdate"+(i as string) as string) as name) "u" paramStr:("tooltip:\"If enabled, the positions of the instances will change in real-time when a Distributed on object is moved. All parameter values in the Lister and Scatter object UIs continue to update automatically regardless of this setting.\" height:16 offset:"+offsets[17]+" align:#left checked:"+(ob.automaticUpdates as string))
		automatUpdateHandler = "o = (returnObject " + (i as string) +"); o.automaticUpdates = val\
		if (val == true) then (countAutoUpdate = countAutoUpdate + 1) else (countAutoUpdate = countAutoUpdate - 1)\
		if(countAutoUpdate < (scatterObjects.count - deletedScatters.count)) then (newDynamicRollout.CBUpdateAll.checked = false)\
		else (newDynamicRollout.CBUpdateAll.checked = true)"
		rci.addHandler (("CBUpdate"+(i as string) as string) as name) #changed paramStr:"val" codeStr:automatUpdateHandler

		-- A button for 'Update Now'
		rci.addControl #button (("ButtonUpdate"+(i as string) as string) as name) "Update now" paramStr:("tooltip:\"Manually update the positions of the instances after a Distributed on object has been moved. Only required if Auto update is not enabled for this Scatter.\" height:16 offset:"+offsets[18]+" align:#center width:75")
		updateOneHandler = "o = (returnObject " + (i as string) +") \n userSelectionBackup = copy userSelection #nomap \n \
		select o \n pressUpdateBtn() \
		if (userSelectionBackup.count ==0) then deselect $ else select userSelectionBackup"
		rci.addHandler (("ButtonUpdate"+(i as string) as string) as name) #pressed codeStr:updateOneHandler
		
	)

	-- initial row of Labels to indicate the GUI elements' functions
	fn addInitLabel rci = (
		-- we pre-define the names and offsets of the labels to make it easier
		labels=#("Sel","Name","Previz type: None","Dot","Box", "Wire","Full", "Point","Disp [%]","Points [%]","Count","Density","Rnd seed","Auto update")	
		offsets=#("[-51,0]","[-55,0]","[7,0]","[12,0]","[-12,0]","[-35,0]","[-62,0]","[-90,0]","[-97,0]","[-94,0]","[-65,0]","[-40,0]","[-35,0]","[-7,0]")
		
		-- then add the labels
		rci.addControl #label (("InitLabel"+(0 as string) as string) as name) ("Enbl") paramStr:("offset:[-23,0]  height:20 align:#center across:"+((labels.count+1) as string))
		i = 1	
		for lab in labels do (
			rci.addControl #label (("InitLabel"+(i as string) as string) as name) (lab as string) paramStr:("offset:"+offsets[i]+"align:#center")
			i = i+1
		)
		-- and a separating line
		rci.addControl #label "line" "_________________________________________________________________________________________________________________________________" paramStr:"height:16 offset:[0,-12]"
	)
	
	-- initial row of controls for all CScatter objects
	fn addInitControl rci = (		
		rci.addControl #checkbutton "CBEnableAll" "E" paramStr:("tooltip:\"Enable or disable all Corona Scatters at once.\" across:11 highlightColor:"+(enableColor as string)+" checked:false offset:[-33,0]")
		rci.addHandler "CBEnableAll" #changed paramStr:"val" codeStr:"(enableAll val)"
				
		rci.addControl #checkbutton "CBSelectAll" "S" paramStr:("tooltip:\"Select or deselect all Corona Scatters at once.\" highlightColor:"+(selectColor as string)+" offset:[-78,0] checked:true")
		rci.addHandler "CBSelectAll" #changed paramStr:"val" codeStr:"(selectAll val)"

		rci.addControl #label "forAllText" "All CScatter objects" paramStr:"offset:[-70,0]"
		
		previzOffsets=#("[-2,0]","[-46,0]","[-90,0]","[-134,0]","[-178,0]","[-222,0]")
		
		for CBPrevizInd=0 to 5 do (
			rci.addControl #checkbutton ("CBPrevizAll"+(CBPrevizInd as string)) " " paramStr:("tooltip:\"Click any grey box to change all Scatters to the selected Previz type.\" height:16 offset:"+previzOffsets[CBPrevizInd+1]+" highlightColor:[0,170,2] align:#left checked:false")
			rci.addHandler ("CBPrevizAll"+(CBPrevizInd as string)) #changed paramStr:"val" codeStr:("(setPrevizAll "+(CBPrevizInd as string)+")")
		)
	
		rci.addControl #checkbutton "CBUpdateAll" "U" paramStr:"tooltip:\"If enabled, the positions of the instances will change in real-time when a Distributed on object is moved. All parameter values in the Lister and Scatter object UIs continue to update automatically regardless of this setting.\" offset:[27,0] checked:true"
		rci.addHandler "CBUpdateAll" #changed paramStr:"val" codeStr:"(updateAll val)"
		
		rci.addControl #button "UpdateAllButton" "Update all" paramStr:"tooltip:\"Manually update the positions of the instances after a Distributed on object has been moved for all Scatters at once.\" width:75 offset:[11,0]"
		rci.addHandler "UpdateAllButton" #pressed codeStr:"updateAllPress()"
	)
	
	-- cleans up the callbacks made by this script when the rollout closes and it is done by the user
	-- cleanUpBool indicates if the rollout is closed by user or not
	-- when we refresh the rollout, we set the bool to false, therefore not erasing the callbacks
	-- after the refresh it is set back to true, so if the user closes the script, the callbacks are deleted
	fn whenClosedCleanup = (
		if(cleanUpBool == true) then
		(
			callbacks.removeScripts id:#CSLstrAdded
			callbacks.removeScripts id:#CSLstrPreDelete
			callbacks.removeScripts id:#CSLstrPostDelete
			deleteAllChangeHandlers id:#SelectionChange
			deleteAllChangeHandlers id:#ParamChange
			deleteAllChangeHandlers id:#NameChange
		)
	)
	
	-- Creates the whole new dynamic CScatter list rollout and returns it
	fn addListerRollout = (		
		rci = rolloutCreator "newDynamicRollout" "CScatter objects list" -- we create a new dynamic rollout
		rci.begin()
		
		if(scatterObjects.count == 0) then (
			rci.addControl #label "noScatterText" "No Corona Scatter objects in the scene. Inserting one will update this list if 'Automatically update UI' is checked." paramStr:"height:20"
			rci.addControl #label "noScatterText2" "If not, you can either check it or update your UI manually by clicking 'Update UI now'." paramStr:"height:20"
		)
		else (		
			addInitLabel rci -- add row with labels
			addInitControl rci -- add row with controls for all CScatter objects
			
			i = 1
			for o in objects where classof o == CScatter do (
				addRow rci i ob:&o -- add a row for this CScatter			
				i = i +1
			)
		)
		rci.addHandler #newDynamicRollout #close codeStr:"whenClosedCleanup()"
		return (rci.end())		
	)
	
	-- creates the change handlers for all CScatter objects, so if users changes something outside of the script, the script is notified and updates itself
	fn createHandlers = (
		-- a selection handler
		deleteAllChangeHandlers id:#SelectionChange
		when select (for o in objects where classof o == CScatter collect o) changes id:#SelectionChange obj do (
			pos = (findItem userSelection obj )
			-- we deselected a selected object
			if ( pos != 0 and obj.isSelected == false ) then (
					-- find the scatter and turn it's selected ComboButton off
					objID = (findItem scatterObjects obj)
					checkboxes = for c in ScatterListRollout.controls where matchPattern c.name pattern:("CBSelected"+(objID as string)) collect c
					checkboxes[1].checked=false
					deleteItem userSelection pos -- update the userSelection array
					if(userSelection.count != (scatterObjects.count - deletedScatters.count)) then ( ScatterListRollout.CBSelectAll.checked = false  ) --update the SelectedAll CheckButton
			)
			-- we selected an unselected object
			else if(obj.isSelected == true and pos == 0) then (
					append userSelection obj -- update the userSelection array
					-- find the scatter and turn it's selected ComboButton on
					objID = (findItem scatterObjects obj)
					checkboxes = for c in ScatterListRollout.controls where matchPattern c.name pattern:("CBSelected"+(objID as string)) collect c
					checkboxes[1].checked=true
					if(userSelection.count == (scatterObjects.count - deletedScatters.count)) then ( ScatterListRollout.CBSelectAll.checked = true  ) --update the SelectedAll CheckButton
			)
		)
		
		-- a "catch-all" callback for all the CScatter's parameters
		deleteAllChangeHandlers id:#ParamChange
		when parameters (for o in objects where classof o == CScatter collect o) change id:#ParamChange obj do (
			-- find out how many are enabled and autoUpdateEnabled, since we don't know which parameter changed, we have to build these counts from 0
			enabledCount = 0
			autoUpdateCount = 0
			for o in objects where classof o == CScatter do ( -- we count all scatters that are in the scene AND the Lister knows about
				if( (findItem scatterObjects o) != 0) then (
					if (o.enable == true ) then (enabledCount = enabledCount + 1)
					if (o.automaticUpdates == true) then (autoUpdateCount = autoUpdateCount + 1)
				)
			)
			
			-- turn on/off the CheckButtons accordingly
			if(enabledCount == scatterObjects.count - deletedScatters.count) then ScatterListRollout.CBEnableAll.checked = true
			else ScatterListRollout.CBEnableAll.checked = false
			if(autoUpdateCount == scatterObjects.count - deletedScatters.count ) then ScatterListRollout.CBUpdateAll.checked = true
			else ScatterListRollout.CBUpdateAll.checked = false
			
			-- find the CScatter object and update it's values
			pos = (findItem scatterObjects obj)
			for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("*"+(pos as string)) do (
				if ( ctrl.name == "CBEnabled"+(pos as string)) then(ctrl.checked = obj.enable)
				if ( ctrl.name == "SpinnerDisplayFr"+(pos as string)) then(ctrl.value = ( obj.displayedFraction as float ) )
				if ( ctrl.name == "SpinnerPointFr"+(pos as string)) then (
					ctrl.value = ( obj.pointCloudFraction as float )
					if(obj.previzType == 5) then (ctrl.enabled = true) else (ctrl.enabled = false) 
					)
				if ( ctrl.name == "SpinnerRndSeed"+(pos as string)) then(ctrl.value = ( obj.seed as integer ) )
				if ( ctrl.name == "CBUpdate"+(pos as string)) then(ctrl.checked = (obj.automaticUpdates) ) 
				if ( ctrl.name == "ColorPicker"+(pos as string)) then( ctrl.color = (obj.wireColor as color) ) 
				if ( ctrl.name == "CBinstancesMode"+(pos as string)) then( if (obj.instancesUseDensity == false) then ( ctrl.checked = false ) else ( ctrl.checked = true ) ) 					
				if ( ctrl.name == "SpinnerCount"+(pos as string)) then (
					ctrl.value = obj.count
				)
				if ( ctrl.name == "SpinnerDensity"+(pos as string)) then (
					ctrl.value = obj.density
					if (obj.instancesUseDensity == true) then ( ctrl.enabled = true ) else (ctrl.enabled = false )
				)
			)
			
			for c in ScatterListRollout.controls where matchPattern c.name pattern:("CBPreviz"+(pos as string)+"_*") do (
				if ( c.name == "CBPreviz"+(pos as string)+"_"+(obj.previzType as string)) then ( c.checked = true)
				else ( c.checked = false )
			)
			
			-- redo the Previz Type distribution 
			previzCount=#(0,0,0,0,0,0)
			scatterPrevizSetting=#()
			indexPreviz = 1
			for o in objects where classof o == CScatter do (
				--scatterPrevizSetting[indexPreviz] = o.previzType
				previzCount[o.previzType+1] = previzCount[o.previzType+1] + 1
				indexPreviz = indexPreviz + 1
			)
			scatterPrevizSetting[pos] = obj.previzType
			
			-- setup agregated Previz Type buttons
			if (previzCount[1] == scatterObjects.count) then (ScatterListRollout.CBPrevizAll0.checked = true ) else (ScatterListRollout.CBPrevizAll0.checked = false) 
			if (previzCount[2] == scatterObjects.count) then (ScatterListRollout.CBPrevizAll1.checked = true ) else (ScatterListRollout.CBPrevizAll1.checked = false)
			if (previzCount[3] == scatterObjects.count) then (ScatterListRollout.CBPrevizAll2.checked = true ) else (ScatterListRollout.CBPrevizAll2.checked = false)
			if (previzCount[4] == scatterObjects.count) then (ScatterListRollout.CBPrevizAll3.checked = true ) else (ScatterListRollout.CBPrevizAll3.checked = false)
			if (previzCount[5] == scatterObjects.count) then (ScatterListRollout.CBPrevizAll4.checked = true ) else (ScatterListRollout.CBPrevizAll4.checked = false)
			if (previzCount[6] == scatterObjects.count) then (ScatterListRollout.CBPrevizAll5.checked = true ) else (ScatterListRollout.CBPrevizAll5.checked = false)
		)
		
		-- a name change callback
		deleteAllChangeHandlers id:#NameChange
		when name (for o in objects where classof o == CScatter collect o) change id:#NameChange obj do (
			pos = (findItem scatterObjects obj)
			for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("*"+(pos as string)) do (
				if ( ctrl.name == "EdittextName"+(pos as string)) then(ctrl.text = obj.name)
			)
		)
	)

	-- the main function that creates and displays the whole rollout
	-- gets callled if a CScatter enters/leaves the scene to update the list completely
	fn createMainRollout = (
		-- renew the CScatter reference list
		scatterObjects=#()
		deletedScatters = #()
		previzCount=#(0,0,0,0,0,0)
		scatterPrevizSetting=#()
		indexPreviz = 1
		for o in objects where classof o == CScatter do (
			addObject obj:&o --add this scatter to the global list
			scatterPrevizSetting[indexPreviz] = o.previzType
			previzCount[o.previzType+1] = previzCount[o.previzType+1] + 1
			indexPreviz = indexPreviz + 1
		)
		
		-- remove the old one, add a new one
		cleanUpBool = false -- turn off callback cleaning for this rollout exit
		try ( removeRollout ScatterListRollout ) catch() -- stops duplicating of the dialog by trying to remove each time
		ScatterListRollout = addListerRollout()
		cleanUpBool = true -- turn it back on
		
		-- display the rollout
		addrollout ScatterListRollout MainListerWindow
		-- if we have some CScatters, set the correct values to the new rollotu and create the handlers 
		if(scatterObjects.count > 0) then (
			setGUIValues()
			createHandlers()
		)
		
		-- scale the window so all the CScatters are visible just fine
		heightMainWindow = 22 * scatterObjects.count + 100 + 50
		MainListerWindow.size = [800,heightMainWindow]
	)

	-- handler function for when a CScatter enters the scene
	fn whenAdded = (
		if(UIAutoUpdate) then (
			addedNode = callbacks.notificationParam()
			if (classOf addedNode == CScatter) then ( -- if the node is a CScatter object
				createMainRollout()	-- we update the whole list because we can't modify a once created rollout, so we have to redo it from scratch
			)
		) else ( changedNotUpdated += 1 )
	)
	
	fn whenReset = (
		if(UIAutoUpdate) then (
			createMainRollout()	-- we update the whole list because we can't modify a once created rollout, so we have to redo it from scratch
		) else (
			changedNotUpdated += 1 -- always want to update next
			for ctrl in ScatterListRollout.controls do (
				ctrl.enabled = false
			)
			index = 1
			for o in scatterObjects do (
				deletedScatters = append deletedScatters index
				index+=1
			)
				
		)
	)
	
	-- Delete handlers
	-- since Maxscript provides 2 types of delete handlers (pre and post delete) we need both of them because
	-- we want to refresh only if the object leaving the scene is a CScatter
	-- this information is availbable only in pre-Delete state but we want to refresh in the post-Delete state
	
	-- handler function for BEFORE a CScatter leaves the scene
	fn whenDeletedPre = (
		deletedNode = callbacks.notificationParam()
		if (classOf deletedNode == CScatter) then ( -- if the node is a CScatter object
			if (UIAutoUpdate == true) then ( refreshWindow = true ) -- we set the refresh indicator to true
			else (
				changedNotUpdated += 1
				deletedScatterNumber = -1
				for index=1 to scatterObjects.count do (
					obj = returnObject index
					if(obj == deletedNode) then deletedScatterNumber = index
				)
				previzCount[deletedNode.previzType + 1] -= 1
				if(deletedNode.enable == true) then (countEnabled -= 1)
				if(deletedNode.automaticUpdates == true) then (countAutoUpdate -= 1)
				deletedScatters = append deletedScatters deletedScatterNumber
				for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("CBPreviz"+(deletedScatterNumber as string)+"*") do ctrl.enabled = false
				for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("CBEnabled"+(deletedScatterNumber as string)+"*") do ctrl.enabled = false
				for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("CBSelected"+(deletedScatterNumber as string)+"*") do ctrl.enabled = false
				for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("ColorPicker"+(deletedScatterNumber as string)+"*") do ctrl.enabled = false
				for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("EdittextName"+(deletedScatterNumber as string)+"*") do ctrl.enabled = false
				for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("SpinnerDisplayFr"+(deletedScatterNumber as string)+"*") do ctrl.enabled = false
				for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("SpinnerPointFr"+(deletedScatterNumber as string)+"*") do ctrl.enabled = false
				for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("SpinnerRndSeed"+(deletedScatterNumber as string)+"*") do ctrl.enabled = false
				for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("CBinstancesMode"+(deletedScatterNumber as string)+"*") do ctrl.enabled = false
				for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("SpinnerCount"+(deletedScatterNumber as string)+"*") do ctrl.enabled = false
				for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("SpinnerDensity"+(deletedScatterNumber as string)+"*") do ctrl.enabled = false
				for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("CBUpdate"+(deletedScatterNumber as string)+"*") do ctrl.enabled = false
				for ctrl in ScatterListRollout.controls where matchPattern ctrl.name pattern:("ButtonUpdate"+(deletedScatterNumber as string)+"*") do ctrl.enabled = false
			)
		)
	)
	
	-- handler function for AFTER a CScatter leaves the scene
	fn whenDeletedPost = (
		if(refreshWindow == true) then ( -- if the node that left the scene was a CScatter, this will be true and we refresh, otherwise do nothing
			createMainRollout()
			refreshWindow = false
		)		
	)

	-- function for  setting the correct values to some of the GUI controls
	-- many values are set when the rollout enters the view but some of them require more complex computation which we perform here. For example SelectAll, EnableAll, etc.
	fn setGUIValues = (
		-- EnableAll and AutoUpdateAll get counted and set properly here
		countEnabled = 0
		countAutoUpdate = 0
		for o in objects where classof o == CScatter do ( -- count them
			if (o.enable == true ) then (countEnabled = countEnabled + 1)
			if (o.automaticUpdates == true) then (countAutoUpdate = countAutoUpdate + 1)
		)
		
		-- set proper states
		if countEnabled == scatterObjects.count then ScatterListRollout.CBEnableAll.checked=true else ScatterListRollout.CBEnableAll.checked=false
		if countAutoUpdate == scatterObjects.count then ScatterListRollout.CBUpdateAll.checked=true else ScatterListRollout.CBUpdateAll.checked=false
		
		-- SelectAll gets evalueted here
		userSelection = selection as array		
		if (userSelection.count != scatterObjects.count) then ( -- if we don't have all selected
			ScatterListRollout.CBSelectAll.checked=false
		)
		else (
			ScatterListRollout.CBSelectAll.checked=true
		)

		-- setup agregated Previz Type buttons
		if (previzCount[1] == scatterObjects.count) then (ScatterListRollout.CBPrevizAll0.checked = true ) else (ScatterListRollout.CBPrevizAll0.checked = false) 
		if (previzCount[2] == scatterObjects.count) then (ScatterListRollout.CBPrevizAll1.checked = true ) else (ScatterListRollout.CBPrevizAll1.checked = false)
		if (previzCount[3] == scatterObjects.count) then (ScatterListRollout.CBPrevizAll2.checked = true ) else (ScatterListRollout.CBPrevizAll2.checked = false)
		if (previzCount[4] == scatterObjects.count) then (ScatterListRollout.CBPrevizAll3.checked = true ) else (ScatterListRollout.CBPrevizAll3.checked = false)
		if (previzCount[5] == scatterObjects.count) then (ScatterListRollout.CBPrevizAll4.checked = true ) else (ScatterListRollout.CBPrevizAll4.checked = false)
		if (previzCount[6] == scatterObjects.count) then (ScatterListRollout.CBPrevizAll5.checked = true ) else (ScatterListRollout.CBPrevizAll5.checked = false)
	
		-- all of 'selected' CheckButtons are set here
		for o in objects where classof o == CScatter do ( -- for each CScater: find it in the list, get it's 'selected' CheckButton, select it if user has it selected
			pos = findItem scatterObjects o
			selectedBoxes = for c in ScatterListRollout.controls where matchPattern c.name pattern:("CBSelected"+(pos as string)) collect c
			
			if(selectedBoxes.count != 1) then print "Error! Too many buttons for this Scatter." -- there should be only one button for each CScater
			if((findItem userSelection o) == 0) then (
				selectedBoxes[1].checked =false
			)
			else (
				selectedBoxes[1].checked =true
			)
		)
	)	
	
	rollout UIOptionsRollout "UI Autorefresh Options"
	(
		checkbutton CAutoUpdate "Automatically update Lister's UI" offset:[-110,0] across:2 checked:true tooltip:"When enabled, the Lister will update automatically if an existing Scatter is deleted, or if a new one is added. All parameter values in the Lister and Scatter object UIs continue to update automatically regardless of this setting."
		button BUpdateUI "Update Lister's UI now" offset:[-340,0] tooltip:"Manually updates the Lister to delete the UI controls for all deleted Scatters, or add UI controls for any new ones that have been added. Only required if Automatically update Lister's UI is disabled."
		
		on BUpdateUI pressed do
		(
			createMainRollout()
			changedNotUpdated = 0
		) 
		
		on CAutoUpdate changed state do
		(
			UIAutoUpdate = state
			if(state == on and changedNotUpdated > 0) then (
				createMainRollout()
				changedNotUpdated = 0
			)			
		) 
	)
	
	-- Main Script Runtime
	
	try ( -- close if the dialog alredy exists to avoid doubling of the windows when user launches the script twice in a row
		closeRolloutFloater  MainListerWindow
	)	catch()		

	for o in objects where classof o == CScatter do ( -- we add all the CScater objects' references to the list
			addObject obj:&o --add this scatter to the global list
	)
	
	-- remove all previous callbacks made by this script
	callbacks.removeScripts id:#CSLstrAdded
	callbacks.removeScripts id:#CSLstrPreDelete
	callbacks.removeScripts id:#CSLstrPostDelete
	callbacks.removeScripts id:#CSLstrPostReset
	callbacks.removeScripts id:#CSLstrFileOpen
	callbacks.removeScripts id:#CSLstrNewScene
	callbacks.addscript #sceneNodeAdded "whenAdded()" id:#CSLstrAdded
	callbacks.addscript #nodePreDelete "whenDeletedPre()" id:#CSLstrPreDelete
	callbacks.addscript #nodePostDelete "whenDeletedPost()" id:#CSLstrPostDelete
	callbacks.addscript #systemPostReset "whenReset()" id:#CSLstrPostReset
	callbacks.addscript #filePostOpen "whenReset()" id:#CSLstrFileOpen
	callbacks.addscript #systemPostNew "whenReset()" id:#CSLstrNewScene
	
	
	-- create the script's main window
	size = 26 * scatterObjects.count + 100 + 50
	MainListerWindow = newRolloutFloater "Corona ScatterLister" 800 size

	addrollout UIOptionsRollout MainListerWindow
	
	
	-- fill the window with the rollout
	createMainRollout()		
)
