5-- MacroScript File
-- Created:       Jan 15 2002
-- Last Modified: 2015_08_23
-- Light Lister Script 3.0
-- Version: 3ds max 2016
-- Author: Alexander Esppeschit Bicalho [discreet]
--***********************************************************************************************
-- MODIFY THIS AT YOUR OWN RISK

/* History
this script is based on the lightlistner that came with vray 3 which it was based in the original one that came with max, it supports all the max lights, vray and corona

updates to the vray code was to add tooltips to some parameters and support to edit the ies files on both vray and photometric lights

2015_08_27 - v3.02 : Guillermo Leal LL
			fix:	crash when deleteing a corona light
			add:	clicking on the matreial button now shows an option to select or ediit the material
			
2015_08_23 - v3.01 : Guillermo Leal LL
			add: support for coronal materials and corona light materials

2015_08_21 - v3.00 : Guillermo Leal LL
			 add:	support for corona lights
			 add:	tooltips for the light name as sometimes they have a long name, you need to hover the mouse over the on checkbox as the textedit control does not has tooltips.
			 add:	tooltips for some items
			 add:	edit ies files on corona , vray ies lights and max photometric
			 
- added support for mr_Sky, mr_SkyPortal, and mr_Sun (LAM)
- fixed mentalray shadow map list bug by zhangy
- Added product switcher: this macro file can be shared with all Discreet products
- Added Support to Mental Ray Lights and Plugin Script lights - this uses 4 functions to manage Delegate properties
- Added Support to Blur_Adv. Shadows and mental Ray shadows
- Enabled Luminaire support
- Enabled Deletion Callback
- Removed Blur Adv. Shadows from the UI (they're still in the Engine) since PF's spec calls for that
- Fixed divide by 0 on adding mr lights to the list
- Fixed an incorrect try/catch loop that would crash the script when radiosity was present
- Fixed crash on Refresh after deleting light - LLister.UIControlList not reset, contained deleted node (LAM)
- Added support for MultiProduct (IFDEFs for #VIZR) (AB)
- Added support to Global Shadow Generator - when a Global Shadow Generator is changed, the Refresh button is 
  highlighted to warn the user he needs a refresh
- Fixed problem when launching the Light Lister when lights have manipulators enabled
- Added LineOffset and YOffset to help Japanese/Chinese localization
- Added support for V-Ray light and shadow plugins.
- Added selectionsets support

to do:
- add the corona shadow code for regular lights. never use it so i might never add it.....

*/


/*

macros.run "Lights and Cameras" "Light_list"

This Light Lister supports all new lights in 3ds max 5:

- Photometric Lights
- Skylights
- IES Sun

It also supports the new shadows types:

- Area Shadows
- Adv. Raytraced shadows

*/

/* Expanding the Light Lister -- AB Jun 20, 2002

This Light Lister does not automatically support new light or shadow plugins.

For them to be supported, you need to make several changes in the script:

-- Class Definitions

Here the classes for each light types are defined. If you want to add a new light type, add a new class entry and list the
classes in the array
In the end of the script, each class definition is scanned and generates the UI entries.
You'll also need to change the script to parse and collect all instances of your class, as is done with the current code.

-- Properties

The function CreateControls generates the dynamic rollout containing all spinners, properties, etc. 
The controls are grouped by light type and handle special cases like different parameter names for MAX lights and Photometric
lights. The On/Off checkbox is an example of how to handle a control that is tied to a property in a scene light. 
In the example below, ControlName is the control name and Property is the property you want to expose/access.

	LLister.maxLightsRC.addControl #checkbox (("ControlName" + LLister.count as string) as name) "" \
		paramStr:("checked:" + (LLister.LightIndex[LLister.count][1].Property as string) + " offset:[8,-22] width:18")
	LLister.maxLightsRC.addHandler (("ControlName" + LLister.count as string) as name) #'changed state' filter:on \
		codeStr:("LLister.LightIndex[" + LLister.count as string + "][1].Property = state")

Notice the controls are all aligned using Offset. If you add a new control, you need to reorganize the remaining controls.

-- Exposing Shadow Plugins

Shadow plugins are harder to expose because each shadow has a different set of parameters, or even different parameter
names. The framework to expose them is similar to the one to expose the properties, but you need to create special cases
for each shadowplugin or each property.
For instance, if your shadow plugin class is myShadow and it exposes a Bias Property called myShadowBias, you'll need to
change the Bias Control and the Shadow Dropdown. In the Bias Control, you need to read the Bias value, and change the  
event so it checks for the correct class and sets the property.
In the Shadow Dropdown event, you need to set the control state and value acording to the shadow class.

In any case, make sure you keep a copy of the original Light Lister so you can come back to it in case you have problems

*/

-- Backwards compatibility helper functions
if (substituteString==undefined) do (
	global substituteString
	fn substituteString srcstr fromstr tostr =(
		local result=srcstr
		local fromIndex
		
		while ((fromIndex=findString result fromstr)!=undefined) do (
			result=replace result fromIndex fromstr.count tostr
		)
		
		result
	)
)

							
-- macroScript GToolsLightLister
-- toolTip: "G.Tools Light Lister"
-- category:"--G.Tools--" 
-- ButtonText:"G.Tools Light Lister"
-- Icon:#("Lights",7)
(
local ver = "3.02"
	
struct gt_Tools_struct 
	(
		TheLightMaterials = #(),
		tmpMats = #(),
		
		fn getLightMaterialArrayIntoArray =
		(
			tmp = #()
			
			for i = 1 to TheLightMaterials.count do
				(
					a = #()
					append a TheLightMaterials[i]
					append tmp a
				)			
				
			return tmp
		),
		
		fn removeDupliateMaterials =
		(
			
			for i = 1 to tmpMats.count do
				(
				if tmpMats[i] != undefined then
					(
					if (finditem TheLightMaterials tmpMats[i]) == 0 then append TheLightMaterials tmpMats[i]
					)
				)

		),
		
		fn getLightMaterialArray =
		(
			return TheLightMaterials
		),
		
		fn loadIesFile =
			(
			Local TheNewIES = getOpenFileName caption:"Select the new IES File:" \
												types:"HDRI Files(*.ies)"
				if TheNewIES != undefined then
					(
						return TheNewIES
					) else
					return undefined
			),
			
		fn loadBitmapTexture = 
			(
				Local TheNewFile= getOpenFileName caption:"Select the new Texture File:" \
												types:"*.*"
				if TheNewFile != undefined do
					(
						theBMP = Bitmaptexture()
						theBMP.filename = TheNewFile
						return theBMP
					)
			),
			
		fn CheckValidMatType theMat theClass =
		(	
			MatClass = classof theMat
			
			if MatClass == CoronaMtl and  theMat.levelSelfIllum > 0.0 and MatClass == theClass then 
				return true
			if MatClass == CoronaLightMtl and MatClass == theClass  then
				return true
			else
				return false
		),
		
		-- Function to look for a specific classof material
		fn FindClass matt theClass =
			(		
				submatcount = try(getnumsubmtls matt)catch(0) -- get the number of submaterials
	
				if submatcount != 0 then -- if it has more than 1
				 (
					for i in 1 to submatcount do -- loop trough all the submaterials
						( 
							the_submat = getsubmtl matt i -- get the actual material of each sub
								
							  if the_submat != undefined and (CheckValidMatType the_submat theClass) do -- if it has a material and its one of the suported materials add it to the array
							   (		
							--	print 	the_submat					   
							   FindClass the_submat theClass -- send it again to the same function until there are no more subs
							   )	
						)
				 )else
				 
				 if ((CheckValidMatType matt theClass) == true )do 
				 (
					 append tmpMats matt -- append the material to the array if its in the list of supported materials types
				 )
			),
			
					-- Function to select by Class of Material ---		
		fn FindMaterials classType selected:false  =
			(	
				
			TheLightMaterials = #()  -- reset the array
			tmpMats = #()
			-- Loop trough all the scenematerials and look for the wanted material 
			if selected then 
				( -- selection
					tempSel = getCurrentSelection()
					for i = 1 to tempSel.count do
						(
							if tempSel[i].material != undefined then (FindClass tempSel[i].material classType)
						)									
				)
			else
				(-- Scence
				for matt in scenematerials do (FindClass matt  classType)
				)
				
			removeDupliateMaterials()
			),
			
		fn TheMenu  theMat =
			(
				global tmp_current_mat = theMat	-- rcmenu does not allow me to use local variables, not sure why
				
				rcmenu RightClickMulti
					(
					menuItem SelectObjs "Select Objects" 		
					menuItem editMat "Edit Material" 
						
					on SelectObjs picked do 
						(
							Local TheSelection = #()

							-- find the dependencis of that material												
							TempMap = refs.dependents tmp_current_mat
							-- add the geometry that have that material to the array
							for x in TempMap where superclassof x == GeometryClass do append TheSelection x	

							-- select the complete array
							try(select TheSelection)catch()
						)
						
					on editMat picked do 
						(
							setMeditMaterial  1 tmp_current_mat
						)
					) -- end menu
			)
	)

struct GtoolsLightListerStruct (GlobalLightParameters, LightInspectorSetup, LightInspectorFloater, LightInspectorListRollout, ShadowPlugins, \
							ShadowPluginsName, maxLightsList, LSLightsList, SkyLightsList, SunLightsList, enableUIElements, \
							LuminairesList, maxLightsRC, CreateLightRollout, UIControlList, DeleteCallback, disableUIElements, \
							LLUndoStr, count, lbcount, lightIndex, decayStrings, totalLightCount, \
							miLightsList, getLightProp, setLightProp, setShdProp, getShdProp, fnShadowClass, enableRefreshBtn, \
							mrSkyLightsList, mrSunLightsList, mrSkyPortalLightsList, MRSkyPortal_ShadowSamples, MRSkyPortal_Modes, \
							VRayLightsList, VRayIESLightsList, VRayAmbientLightsList, VRaySunSkyLightsList, yOffset, LineOffset, \
							CoronaLightsList, CoronaSunSkyLightsList, CoronaMatList, CoronaLightMatList, totalMatCount
		)


global gtoolsLLister, gtoolsLListerYOffset, gt_myTools
if gtoolsLLister == undefined or debug == true do gtoolsLLister = GtoolsLightListerStruct()

gt_myTools =	gt_Tools_struct()
	
-- Strings for Localization

gtoolsLLister.decayStrings = #("None","Inverse","Inv. Square")
gtoolsLLister.LLUndoStr = "LightLister"

local dialogUp = false

-- End Strings

-- Positioning to help localization

gtoolsLListerYOffset = 0
gtoolsLLister.yOffset = gtoolsLListerYOffset
gtoolsLLister.LineOffset = 0

							

-- Useful Functions

fn subtractFromArray myArray mySub =
(
	tmpArray = #()
	for i in myArray do append tmpArray i
	for i in mySub do


	(
		itemNo = finditem tmpArray i
		local newArray = #()
		if itemNo != 0 do
		(
			for j in 1 to (itemNo-1) do append newArray tmpArray[j]
			for j in (itemNo+1) to tmpArray.count do append newArray tmpArray[j]
			tmpArray = newArray
		)
	)
	tmpArray
)

fn compareFN v1 v2 =
(
	if (v1.name as name) < (v2.name as name) then -1
	else if (v1.name as name) > (v2.name as name) then 1
	else 0
)

fn SortNodeArrayByNameAscendingOrder myArray =
(
qsort myArray compareFN
myArray
)

fn SortNodeArrayByName myArray =
(
qsort myArray (fn myname v1 v2 = (if v1.name < v2.name then 0 else 1))
myArray
)


fn copyArray array1 = for i in array1 collect i

fn wrapString inString =
(
	local string1In = "\\"
	local string1Out = "\\\\"
	local string2In = "\""
	local string2Out = "\\\""
	local temp_text_string = substituteString inString string1In string1Out
	temp_text_string = substituteString temp_text_string string2In string2Out
	temp_text_string = string2In + temp_text_string + string2In
	temp_text_string -- return value
)

fn disableUIElements array1 = for i in array1 do execute ("maxLightsRollout." + i as string + ".enabled = false")
gtoolsLLister.disableUIElements = disableUIElements

fn enableRefreshBtn lightobj =
(
	if (gtoolsLLister.GetLightProp lightObj #useGlobalShadowSettings) == true do
	(
		gtoolsLLister.LightInspectorSetup.BtnReload.Checked = true
	)
)
gtoolsLLister.enableRefreshBtn = enableRefreshBtn

fn getLightProp obj prop =
(
	if (isProperty obj prop) and not (isProperty obj #delegate) then
		getProperty obj prop
	else 
		if isProperty obj #delegate then 
			if isProperty obj.delegate prop then
				getProperty obj.delegate prop
			else undefined
		else undefined
)
gtoolsLLister.getLightProp = getLightProp

fn setLightProp obj prop val =
(
	if (isProperty obj prop) and not (isProperty obj #delegate) then
		setProperty obj prop val
	else
		if isProperty obj #delegate then 
			if isProperty obj.delegate prop then
				setProperty obj.delegate prop val
			else undefined
		else undefined
)
gtoolsLLister.setLightProp = setLightProp

fn getShdProp obj prop =
(
	if (isProperty obj #shadowGenerator) and not (isProperty obj #delegate) then
		if (isProperty obj.ShadowGenerator prop) do getProperty obj.ShadowGenerator prop
	else 
		if isProperty obj #delegate then 
			if isProperty obj.delegate #ShadowGenerator then
				if (isProperty obj.delegate.ShadowGenerator prop) do getProperty obj.delegate.ShadowGenerator prop
			else undefined
		else undefined
)
gtoolsLLister.getShdProp = getShdProp

fn setShdProp obj prop val =
(
	if (isProperty obj #shadowGenerator) and not (isProperty obj #delegate) then
		if (isProperty obj.ShadowGenerator prop) do
		(
			setProperty obj.ShadowGenerator prop val
			gtoolsLLister.enableRefreshBtn obj
		)
	else 
		if isProperty obj #delegate then 
			if isProperty obj.delegate #ShadowGenerator then
				if (isProperty obj.delegate.ShadowGenerator prop) do
				(
					setProperty obj.delegate.ShadowGenerator prop val
					gtoolsLLister.enableRefreshBtn obj
				)
			else undefined
		else undefined
)
gtoolsLLister.setShdProp = setShdProp

fn fnShadowClass obj = classof (gtoolsLLister.getLightProp obj #shadowGenerator)
gtoolsLLister.fnShadowClass = fnShadowClass

-- Hardcoded shadow plugins to the ones available

	gtoolsLLister.ShadowPlugins = #(Adv__Ray_Traced, mental_ray_Shadow_Map, Area_Shadows, shadowMap, raytraceShadow, VRayShadow, VRayShadowMap, CoronaShadows)
	gtoolsLLister.ShadowPluginsName = #("Adv. Ray Traced", "mental_ray_Shadow_Map", "Area Shadows", "Shadow Map", "Raytrace Shadow", "VRayShadow", "VRayShadowMap", "CoronaShadows")

/* -- uncomment if you want the Blur Shadows
gtoolsLLister.ShadowPlugins = #(Adv__Ray_Traced, mental_ray_Shadow_Map, Area_Shadows, Blur_Adv__Ray_Traced, shadowMap, raytraceShadow, VRayShadow, VRayShadowMap)
gtoolsLLister.ShadowPluginsName = #("Adv. Ray Traced", "mental_ray_Shadow_Map", "Area Shadows", "Blur Adv. Ray Traced","Shadow Map", "Raytrace Shadow", "VRayShadow", "VRayShadowMap")
*/
	gtoolsLLister.MRSkyPortal_ShadowSamples = for i = 1 to 10 collect (2^i) as string
	gtoolsLLister.MRSkyPortal_Modes = #("Existing", "Envir.", "Custom") -- correspond to mode values of 2, 0, 1

fn isCoronaCurrent = matchPattern (renderers.current as string) pattern:"*Corona*"	

-- Main Function

local CreateLightRollout

fn createLightRollout myCollection selectionOnly:false =
(
	gtoolsLLister.LightInspectorSetup.pbar.visible = true

	-- Class Definitions
	
	maxLights = #(TargetDirectionallight, targetSpot, Directionallight, Omnilight, freeSpot)
	SkyLights = #(IES_Sky, Texture_Sky, Skylight)
	SunLights = #(IES_Sun) -- AB: Jun 20, 2002
	LSLights = #(Free_Area, Target_Area, Free_Linear, Target_Linear, Free_Point, Target_Point, 
					   Free_Sphere, Target_Sphere, Free_Disc, Target_Disc, Free_Cylinder, Target_Cylinder)
	Luminaires = #(Luminaire)
	mrLights = #(miAreaLight, miAreaLightomni)
	mrSkyLight = #(mr_sky)
	mrSunLight = #(mr_sun)
	mrSkyPortalLight = #(mr_sky_portal)
	VRayLights = #(VRayLight)
	VRayIESLights = #(VRayIES)
	VRayAmbientLights = #(VRayAmbientLight)
	VRaySunSkyLights = #(VRaySun)
					   
	CoronaLights = #(CoronaLight)
	CoronaSunSkyLights = #(CoronaSun)

	CoronaMaterial = #(CoronaMtl)
	CoronaLightMaterial = #(CoronaLightMtl)
	
	-- Scene parser
	
	try (SceneLights = myCollection as array)
    catch (if (classof myCollection == SelectionSet) do (SceneLights = myCollection))
	sceneMaxLights = #()
	sceneLSLights = #()
	sceneSkyLights = #()
	sceneSunLights = #()
	sceneLuminaires = #()
	scenemiLights = #()
	scenemrSkyLights = #()
	scenemrSunLights = #()
	scenemrSkyPortalLights = #()
	sceneVRayLights = #()
	sceneVRayIESLights = #()
	sceneVRayAmbientLights = #()
	sceneVRaySunSkyLights = #()
		
	sceneCoronaLights = #()
	sceneCoronaSunSkyLights = #()
		
	sceneCoronaMaterial = #()
	sceneCoronaLightMaterial = #()		

			
	for i in SceneLights do
	(
		LightClass = classof i
		if findItem MaxLights LightClass != 0 do append sceneMaxLights i
		if findItem LSLights LightClass != 0 do append sceneLSLights i
		if findItem SkyLights LightClass != 0 do append sceneSkyLights i
		if findItem SunLights LightClass != 0 do append sceneSunLights i
		if findItem Luminaires LightClass != 0 do append sceneLuminaires i
		if findItem mrLights LightClass != 0 do append scenemiLights i
		if findItem mrSkyLight LightClass != 0 do append scenemrSkyLights i
		if findItem mrSunLight LightClass != 0 do append scenemrSunLights i
		if findItem mrSkyPortalLight LightClass != 0 do append scenemrSkyPortalLights i
		if findItem VRayLights LightClass != 0 do append sceneVRayLights i
		if findItem VRayIESLights LightClass != 0 do append sceneVRayIESLights i
		if findItem VRayAmbientLights LightClass != 0 do append sceneVRayAmbientLights i
		if findItem VRaySunSkyLights LightClass != 0 do append sceneVRaySunSkyLights i
			
		if findItem CoronaLights LightClass != 0 do append sceneCoronaLights i
		if findItem CoronaSunSkyLights LightClass != 0 do append sceneCoronaSunSkyLights i	
	)

		-- Materials
	if isCoronaCurrent() then
	(

-- 		for o = 1 to sceneCoronaMaterial.count do
-- 			format "mat:% \n" sceneCoronaMaterial[o]
	)
	
	if sceneMaxLights.count > 0 then SortNodeArrayByNameAscendingOrder sceneMaxLights
	if sceneLSLights.count > 0 then SortNodeArrayByNameAscendingOrder sceneLSLights
	if sceneSkyLights.count > 0 then SortNodeArrayByNameAscendingOrder sceneSkyLights
	if sceneSunLights.count > 0 then SortNodeArrayByNameAscendingOrder sceneSunLights
	if sceneLuminaires.count > 0 then SortNodeArrayByNameAscendingOrder sceneLuminaires
	if scenemiLights.count > 0 then SortNodeArrayByNameAscendingOrder scenemiLights
	if scenemrSkyLights.count > 0 then SortNodeArrayByNameAscendingOrder scenemrSkyLights
	if scenemrSunLights.count > 0 then SortNodeArrayByNameAscendingOrder scenemrSunLights
	if scenemrSkyPortalLights.count > 0 then SortNodeArrayByNameAscendingOrder scenemrSkyPortalLights
	if sceneVRayLights.count > 0 then SortNodeArrayByNameAscendingOrder sceneVRayLights
	if sceneVRayIESLights.count > 0 then SortNodeArrayByNameAscendingOrder sceneVRayIESLights
	if sceneVRayAmbientLights.count > 0 then SortNodeArrayByNameAscendingOrder sceneVRayAmbientLights
	if sceneVRaySunSkyLights.count > 0 then SortNodeArrayByNameAscendingOrder sceneVRaySunSkyLights
	
	if sceneCoronaLights.count > 0 then SortNodeArrayByNameAscendingOrder sceneCoronaLights
	if sceneCoronaSunSkyLights.count > 0 then SortNodeArrayByNameAscendingOrder sceneCoronaSunSkyLights
		

	-- get the corona materials
	gt_myTools.FindMaterials CoronaMtl
	sceneCoronaMaterial = gt_myTools.getLightMaterialArray()
	if sceneCoronaMaterial.count > 0 then SortNodeArrayByNameAscendingOrder sceneCoronaMaterial	
	gtoolsLLister.CoronaMatList = gt_myTools.getLightMaterialArrayIntoArray()	
	
	gt_myTools.FindMaterials CoronaLightMtl
	sceneCoronaLightMaterial = gt_myTools.getLightMaterialArray()
	if sceneCoronaLightMaterial.count > 0 then SortNodeArrayByNameAscendingOrder sceneCoronaLightMaterial	
 	gtoolsLLister.CoronaLightMatList = gt_myTools.getLightMaterialArrayIntoArray()	
	
-- 	format "CoronaMtl: % \n" sceneCoronaMaterial.count 
-- 	format "CoronaLightMtl: % \n" sceneCoronaLightMaterial.count 		
	
	
	
-- 	for o = 1 to sceneCoronaMaterial.count do
-- 			format "mat:% \n" sceneCoronaMaterial[o]
-- 	
-- 	for o = 1 to sceneCoronaLightMaterial.count do
-- 			format "light:% \n" sceneCoronaLightMaterial[o]
	
	-- Collect Light Instances and build array to be displayed
	
	tmpParser = #( \
		tmpsceneMaxLights = copyArray sceneMaxLights, \
		tmpscenemiLights = copyArray scenemiLights, \
		tmpscenemrSkyLights = copyArray scenemrSkyLights, \
		tmpscenemrSunLights = copyArray scenemrSunLights, \
		tmpscenemrSkyPortalLights = copyArray scenemrSkyPortalLights, \
		tmpsceneLSLights = copyArray sceneLSLights, \
		tmpsceneSkyLights = copyArray sceneSkyLights, \
		tmpsceneSunLights = copyArray sceneSunLights, \
		tmpsceneLuminaires = copyArray sceneLuminaires, \
		tmpsceneVRayLights = copyArray sceneVRayLights, \
		tmpsceneVRayIESLights = copyArray sceneVRayIESLights, \
		tmpsceneVRayAmbientLights = copyArray sceneVRayAmbientLights, \
		tmpsceneVRaySunSkyLights = copyArray sceneVRaySunSkyLights, \
				
		tmpsceneCoronaLights = copyArray sceneCoronaLights, \
		tmpsceneCoronaSunSkyLights = copyArray sceneCoronaSunSkyLights
	)
	
	
	ListParser = #( \
		gtoolsLLister.maxLightsList = #(), \
		gtoolsLLister.miLightsList = #(), \
		gtoolsLLister.mrSkyLightsList = #(), \
		gtoolsLLister.mrSunLightsList = #(), \
		gtoolsLLister.mrSkyPortalLightsList = #(), \
		gtoolsLLister.LSLightsList = #(), \
		gtoolsLLister.SkyLightsList = #(), \
		gtoolsLLister.SunLightsList = #(), \
		gtoolsLLister.LuminairesList = #(), \
		gtoolsLLister.VRayLightsList = #(), \
		gtoolsLLister.VRayIESLightsList = #(), \
		gtoolsLLister.VRayAmbientLightsList = #(), \
		gtoolsLLister.VRaySunSkyLightsList = #(), \
		gtoolsLLister.CoronaLightsList = #(), \
		gtoolsLLister.CoronaSunSkyLightsList = #()
	)

	for i in 1 to tmpParser.count do
	(
		while tmpParser[i].count > 0 do
		(
			tmpNode = tmpParser[i][1].baseObject
			depends = refs.dependents tmpNode
			discard = #()
			for k in depends do if classof k != classof tmpNode or (superclassof k != light and superclassof k != helper) do append discard k
			for k in depends do 
				try
				(
					if classof k == DaylightAssemblyHead or classof k == ParamBlock2ParamBlock2 then 
						append discard k 
					else
						if k.AssemblyMember and not k.AssemblyHead and classof k.parent != DaylightAssemblyHead do append discard k
				) 
				catch()
			depends2 = subtractFromArray depends discard
			depends = SortNodeArrayByName depends2
			if depends.count > 0 do append listParser[i] depends
			tmpParser[i] = subtractFromArray tmpParser[i] (discard + depends)
		)
	)
	



	
	gtoolsLLister.totalLightCount = 	gtoolsLLister.maxLightsList.count + \
								gtoolsLLister.LSLightsList.count + \
								gtoolsLLister.SkyLightsList.count + \
								gtoolsLLister.SunLightsList.count + \
								gtoolsLLister.LuminairesList.count + \
								gtoolsLLister.miLightsList.count + \
								gtoolsLLister.mrSkyLightsList.count + \
								gtoolsLLister.mrSkyPortalLightsList.count + \
								gtoolsLLister.mrSunLightsList.count + \
								gtoolsLLister.VRayLightsList.count + \
								gtoolsLLister.VRayIESLightsList.count + \
								gtoolsLLister.VRayAmbientLightsList.count + \
								gtoolsLLister.VRaySunSkyLightsList.count + \
								gtoolsLLister.CoronaLightsList.count + \
								gtoolsLLister.CoronaSunSkyLightsList.count	+ \
								gtoolsLLister.CoronaMatList.count + \
								gtoolsLLister.CoronaLightMatList.count

	
	
	-- build controls and rollouts
	
	-- MAX Lights
	
	gtoolsLLister.maxLightsRC = rolloutCreator "maxLightsRollout" "Lights" -- Localize the 2nd string only
	gtoolsLLister.maxLightsRC.begin()
  	-- print gtoolsLLister.maxLightsRC.str.count
	
	gtoolsLLister.maxLightsRC.addText "fn clearCheckButtons = for i in gtoolsLLister.LightInspectorListRollout.controls do if classof i == checkButtonControl do if i.checked do i.checked = false\n"
	
	gtoolsLLister.count = 1
	gtoolsLLister.lbCount = 1
	gtoolsLLister.LightIndex = #()
	gtoolsLLister.UIControlList = #(#(),#())

	struct td (label, offset)
	struct titleTemplate (maxLights, lsLights, miLights, luminaires, sunLights, skyLights, mrSkyLights, mrSunLights, mrSkyPortalLights, vrayLights, vrayIESLights, vrayAmbientLights, vraySunSkyLights, coronaLights, coronaSunSkyLights, coronaLightMats, coronaMats)
		
	-- Start Localization
	local titleTemplates = titleTemplate \
		coronaLightMats:#(td "On" 8, td "Name" 28, td "Intensity" 147, td "Color" 205, td "Texture" 280, td "Emit Light" 370, td "Emit BS" 430, td "Directionality" 480, td "Occlude Lights" 550, td "S. Catcher" 630, td "V. Directly" 690, td "V. Refle." 750, td "V. Refra." 800) \
		coronaMats:#(td "On" 8, td "Name" 28, td "Multiplier" 147, td "Color" 205, td "Texture" 233) \
		coronaLights:#(td "On" 8, td "Name" 28, td "Intensity" 147, td "Color" 205, td "Kelvin temp." 233, td "Units" 350, td "Shape" 450, td "Width/radius" 525, td "Height" 610, td "Segments" 670, td "Directionality" 730, td "Emit BS" 800, td "S. Catcher" 845, td "V. Directly" 905, td "V. Refle." 960, td "V. Refra." 1010, td "V. Occlude" 1060, td "Gizmo S." 1120, td "Ies On" 1175, td "Ies File" 1240) \
		coronaSunSkyLights:#(td "On" 8, td "Name" 28, td "Intensity" 145, td "Color" 211, td "Size" 263, td "Color Mode" 325, td "Kelvin temp." 400, td "V. Directly" 470, td "V. Reflections" 530, td "V. Refractions" 600) \
		vrayLights:#(td "On" 8, td "Name" 28, td "Multiplier" 147, td "Color" 205, td "Temperature" 233, td "Units" 309, td "Shadows" 407, td "Subdivs" 460, td "Bias" 525, td "Invisible" 580, td "Skylight" 640, td "Diff." 700, td "Spec." 730, td "Reflect." 770, td "Caust. subd." 830, td "Wire Color" 910, td "Text" 970) \
		vrayIESLights:#(td "On" 8, td "Name" 28, td "Power" 147, td "Color" 205, td "Temperature" 233, td "Shadows" 307, td "Subdivs" 365, td "Bias" 440, td "Diff." 490, td "Spec." 530, td "Use shape" 590, td "Area spec." 660, td "Caust. subd." 720, td "Wire Color" 800, td "Text" 860, td "Ies File" 920) \
		vrayAmbientLights:#(td "On" 8, td "Name" 28, td "Intensity" 147, td "Color" 205, td "Caust. subd." 233, td "Wire Color" 299) \
		vraySunSkyLights:#(td "On" 8, td "Name" 28, td "Intens. Mult." 145, td "Size Mult." 211, td "Sh. Subdivs" 263, td "Sh. Bias" 325, td "Invisible" 377, td "Turbidity" 419, td "Ozone" 477, td "Ph. Emit Rad." 535, td "Sky Model "605, td "Horiz. Illum." 697, td "Caust. subd." 757) \
		maxLights:#(td "On" 8, td "Name" 28, td "Multiplier" 147, td "Color" 205, td "Shadows" 235, td "Map Size" 377, td "Bias" 435, td "Sm.Range" 488, td "Transp." 540, td "Int." 580, td "Qual." 615, td "Decay" 657, td "Start" 735, td "Caust. subd." 795) \
		lsLights:#(td "On" 8, td "Name" 28, td "Intensity(cd)" 147, td "Color" 205, td "Shadows" 235, td "Map Size" 377, td "Bias" 435, td "Sm.Range" 488, td "Transp." 540, td "Int." 580, td "Qual." 615, td "Length" 657, td "Width/Radius" 716,  td "Ies File" 820) \
		miLights:#(td "On" 8, td "Name" 28, td "Multiplier" 147, td "Color" 205, td "Shadows" 235, td "Map Size" 377, td "Bias" 435, td "Sm.Range" 488, td "Transp." 540, td "Int." 580, td "Qual." 615, td "Decay" 657, td "Start" 735) \
		luminaires:#(td "Name" 28, td "Dimmer" 147, td "Color" 205) \
		sunLights:#(td "On" 8, td "Name" 28, td "Intensity(lux)" 147, td "Color" 205, td "Shadows" 235, td "Map Size" 377, td "Bias" 435, td "Sm.Range" 488, td "Transp." 540, td "Int." 580, td "Qual." 615) \
		skyLights:#(td "On" 8, td "Name" 28, td "Multiplier" 147, td "Color" 205) \
		mrSkyLights:#(td "On" 8, td "Name" 28, td "Multiplier" 147, td "Haze" 206, td "H Height" 263, td "H Blur" 322, td "Ground" 378, td "Night" 427, td "Redness" 465, td "Saturation" 525, td "UseAerialPersp" 585, td "AerialPersp" 665) \
		mrSunLights:#(td "On" 8, td "Name" 28, td "Multiplier" 147, td "Shadows" 204, td "Softness" 256, td "Samples" 303, td "Targeted" 349, td "Distance" 398, td "Inherit" 471, td "Haze" 508, td "H Height" 558, td "Redness" 606, td "Saturation" 654, td "Use Targ" 708, td "Radius" 757) \
		mrSkyPortalLights:#(td "On" 8, td "Name" 28, td "Multiplier" 147, td "Color" 205, td "Shadows" 235, td "Extend" 285, td "Samples" 325, td "Length" 380, td "Width" 453, td "Flip Flux" 523, td "Visible" 570, td "Transparency" 605, td "Source" 685)
		-- End Localization
	
	fn WriteTitle labels:undefined =
	(
		local lbName
		fn lbName = 
		(
			if gtoolsLLister.lbCount == undefined do gtoolsLLister.lbCount = 1
			gtoolsLLister.lbCount += 1
			("LB" + gtoolsLLister.lbCount as string) as name
		)
		
		if (labels != undefined) do (
			for i = 1 to labels.count do (
				gtoolsLLister.maxLightsRC.addControl #label (lbname()) labels[i].label paramStr:(" align:#left offset:[" + labels[i].offset as string + "," + ((if (i == 1) then -3 else -18) + gtoolsLLister.yOffset + gtoolsLLister.LineOffset) as string + "]")
			)
		)
	)
	
	fn CreateControls hasMultiplier:false hasColor:false hasShadow:false hasDecay:false hasSize:false hasWireColor:false hasIconText:false isLuminaire:false isMRSky:false isMRSun:false isMRSkyPortal:false isVRayLight:false isVRayIESLight:false isVRayAmbientLight:false isVRaySun:false hasCausticsSubdivs:false isCoronaLight:false isCoronaSun:false isCoronaLightMat:false isCoronaMat:false =
	(

		local lightClassName = ((classof gtoolsLLister.LightIndex[gtoolsLLister.count][1]) as string) as name
		
		
		isMaterial = false
		
		if isCoronaLightMat or isCoronaMat do
			isMaterial = true
		
		-- Selection Checkbutton
		local isLightSelected = false
		
		if not isMaterial then
			for i in gtoolsLLister.LightIndex[gtoolsLLister.count] where (not isLightSelected) do isLightSelected = i.isSelected
				
		gtoolsLLister.UIControlList[1][gtoolsLLister.count] = gtoolsLLister.LightIndex[gtoolsLLister.count][1]
		gtoolsLLister.UIControlList[2][gtoolsLLister.Count] = #()
		
-- 								codeStr:(" \n" + \
-- 											"theMat = 
-- 											"ies = gt_myTools.loadIesFile() \n" + \ -- load the new ies file
-- 											"if ies != undefined then ( \n" + \ 
-- 											"gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #webFile ies \n" + \ -- update the light
-- 											"iesName = filenameFromPath ies \n" + \	-- get the filename only fot he button text
-- 											"maxIesFile" + gtoolsLLister.count as string + ".text = iesName " + "\n" + \	-- update the button text
-- 											"maxIesFile" + gtoolsLLister.count as string + ".tooltip = ies " + "\n" + \		-- update the button tooltip
-- 											") \n")	
		
		if isCoronaLightMat or isCoronaMat	then 
			(
				gtoolsLLister.maxLightsRC.addControl #button (("LightSel" + gtoolsLLister.count as string) as name) "" \
					paramStr:(" offset:[-5,"+ (-2+ gtoolsLLister.yOffset + gtoolsLLister.LineOffset) as string + "] align:#left width:10 height:20 ")
					
				gtoolsLLister.maxLightsRC.addHandler (("LightSel" + gtoolsLLister.count as string) as name) #'pressed ' filter:on \
								codeStr:(" \n" + \
											"theMat = gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] \n" + \ 
											"PopupMenu (gt_myTools.TheMenu theMat)  \n" + \
											" \n")	
										
		)else (
			
				gtoolsLLister.maxLightsRC.addControl #checkbutton (("LightSel" + gtoolsLLister.count as string) as name) "" \
			paramStr:("checked:" + (isLightSelected as string) + " offset:[-5,"+ (-2+ gtoolsLLister.yOffset + gtoolsLLister.LineOffset) as string + "] align:#left" +\
					" width:10 height:20 ")
					
		gtoolsLLister.maxLightsRC.addHandler (("LightSel" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
			codeStr: \
			(
			"clearCheckButtons();if state then (max modify mode;select gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "];LightSel" + (gtoolsLLister.count as string) + ".checked = true); else max select none"
			)
		)
		
		append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightSel" + gtoolsLLister.count as string) as name)
	
		
-- 						gtoolsLLister.maxLightsRC.addControl #button (("maxIesFile" + gtoolsLLister.count as string) as name) (iesName) \
-- 					paramStr:(" offset:[800,"+ (-24+ gtoolsLLister.yOffset) as string + "] width:100 border:true align:#left tooltip:\" " + iesCompleteFileName + " \" ")
-- 				
-- 				gtoolsLLister.maxLightsRC.addHandler (("maxIesFile" + gtoolsLLister.count as string) as name) #'pressed ' filter:on \
-- 								codeStr:(" \n" + \
-- 											"ies = gt_myTools.loadIesFile() \n" + \ -- load the new ies file
-- 											"if ies != undefined then ( \n" + \ 
-- 											"gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #webFile ies \n" + \ -- update the light
-- 											"iesName = filenameFromPath ies \n" + \	-- get the filename only fot he button text
-- 											"maxIesFile" + gtoolsLLister.count as string + ".text = iesName " + "\n" + \	-- update the button text
-- 											"maxIesFile" + gtoolsLLister.count as string + ".tooltip = ies " + "\n" + \		-- update the button tooltip
-- 											") \n")	
											
											
											
		-- On/Off
		if not isMaterial then -- materials dont have on/off
		(
			if isLuminaire == false do
			(
				local onProp = case lightClassName of (
					#VRayIES:				#enabled
					#VRayAmbientLight:	#enabled
					#VRaySun:				#enabled
					#CoronaLight:			#on
					#CoronaSun:			#on
					default:					#on
				)
				gtoolsLLister.maxLightsRC.addControl #checkbox (("LightOn" + gtoolsLLister.count as string) as name) "" \
					paramStr:("checked:" + ((gtoolsLLister.GetlightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] onProp) as string) + " offset:[8,"+ (-22+ gtoolsLLister.yOffset) as string + "] width:18 tooltip:\" " + gtoolsLLister.LightIndex[gtoolsLLister.count][1].name + " \" ")
					
				gtoolsLLister.maxLightsRC.addHandler (("LightOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
					codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #" + onProp +" state")
				append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightOn" + gtoolsLLister.count as string) as name)
			)
		)
	
		-- Light Name
		local isUsingEdittextOffset = 0
		
		if gtoolsLLister.LightIndex[gtoolsLLister.count].count == 1 then
		(
			local wrappedName = wrapString gtoolsLLister.LightIndex[gtoolsLLister.count][1].name
			gtoolsLLister.maxLightsRC.addControl #edittext (("LightName" + gtoolsLLister.count as string) as name) "" \
				paramStr:(" text:" + wrappedName + " width:120 height:16 offset:[23,"+ (-21+ gtoolsLLister.yOffset) as string + "] height:21" )
			
			gtoolsLLister.maxLightsRC.addHandler (("LightName" + gtoolsLLister.count as string) as name) #'entered txt' filter:on \
				codeStr:("gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].name = txt")

			isUsingEdittextOffset = 4
		)
		else
		(
			theNames = for j in gtoolsLLister.LightIndex[gtoolsLLister.count] collect j.name
			sort theNames
			namelist = "#("
			for j in 1 to theNames.count do 
				(
				local wrappedName = wrapString theNames[j]
				append namelist wrappedName
				if j != theNames.count do append namelist ","
				)
			append namelist ")"
			gtoolsLLister.maxLightsRC.addControl #dropDownList (("LightName" + gtoolsLLister.count as string) as name) "" filter:on\
				paramStr:(" items:" + NameList + " width:120 offset:[27,"+ (-22+ gtoolsLLister.yOffset) as string + "] ")
		)
		
		append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightName" + gtoolsLLister.count as string) as name)
		
		-- photometric ies files

		if not isMaterial then
		(
			theClass = (classof gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseobject)
			
			if theClass == Free_Area or theClass == Target_Area or theClass == Free_Light or theClass == Target_Light   then
			(
				-- IES File
				iesCompleteFileName =	 (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #webFile)
				if iesCompleteFileName == undefined then -- check if there is an ies file
					iesCompleteFileName = ""
				iesName = filenameFromPath iesCompleteFileName
				
		--		format "ies:% \n" iesCompleteFileName
				
				gtoolsLLister.maxLightsRC.addControl #button (("maxIesFile" + gtoolsLLister.count as string) as name) (iesName) \
					paramStr:(" offset:[800,"+ (-24+ gtoolsLLister.yOffset) as string + "] width:100 border:true align:#left tooltip:\" " + iesCompleteFileName + " \" ")
				
				gtoolsLLister.maxLightsRC.addHandler (("maxIesFile" + gtoolsLLister.count as string) as name) #'pressed ' filter:on \
								codeStr:(" \n" + \
											"ies = gt_myTools.loadIesFile() \n" + \ -- load the new ies file
											"if ies != undefined then ( \n" + \ 
											"gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #webFile ies \n" + \ -- update the light
											"iesName = filenameFromPath ies \n" + \	-- get the filename only fot he button text
											"maxIesFile" + gtoolsLLister.count as string + ".text = iesName " + "\n" + \	-- update the button text
											"maxIesFile" + gtoolsLLister.count as string + ".tooltip = ies " + "\n" + \		-- update the button tooltip
											") \n")							
							
				
				append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("maxIesFile" + gtoolsLLister.count as string) as name)	
			)
		)
		
		-- Light Multiplier
		if hasMultiplier do (
			local multiplierProp = case lightClassName of (
				#Free_Area:			#intensity
				#Target_Area:			#intensity
				#Free_Linear:			#intensity
				#Target_Linear:		#intensity
				#Free_Point:			#intensity
				#Target_Point:		#intensity
				#Free_Sphere:		#intensity
				#Target_Sphere:		#intensity
				#Free_Disc:			#intensity
				#Target_Disc:			#intensity
				#Free_Cylinder:		#intensity
				#Target_Cylinder:	#intensity
				#Luminaire:				#dimmer
				#mr_sun:				#skymult
				#VRayIES:				#power
				#VRayAmbientLight:	#intensity
				#VRaySun:				#intensity_multiplier
				#CoronaLight:			#intensity
				#CoronaSun:			#intensity	
				#CoronaLightMtl:		#intensity
				#CoronaMtl:			#levelSelfIllum
				default:					#multiplier
			)
			local multiplierEn = case lightClassName of (
				#VRayLight:	(not gtoolsLLister.LightIndex[gtoolsLLister.count][1].skylightPortal)
				default:		true
			)
			local multiplierMin = case lightClassName of (
				#mr_sky:					0.0
				#mr_sun:					0.0
				#mr_sky_portal:		0.0
				#VRayLight:				-1000000000.0
				#VRayIES:				0.0
				#VRayAmbientLight:	0.0
				#VRaySun:				0.0
				#CoronaLight:			0.0
				#CoronaSun:			0.0				
				default:					-1000000.0
			)
			local multiplierMax = case lightClassName of (
				#mr_sky:					15.0
				#mr_sun:					10.0
				#VRayLight:				1000000000.0
				#VRayIES:				1000000000.0
				#VRayAmbientLight:	1000000000.0
				#VRaySun:				1000000000.0
				#CoronaLight:			1000000000.0
				#CoronaSun:			9999.0				
				default:					1000000.0
			)
	
			
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightMult" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[" + multiplierMin as string + "," + multiplierMax as string + "," + \
				(gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] multiplierProp) as string + "] type:#float " + \
				"fieldwidth:45 align:#left offset:[145," + (isUsingEdittextOffset-24+gtoolsLLister.yOffset) as string + "] " +\
				"tooltip: \"" + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] multiplierProp) as string ) + "\" " + \ 
				"enabled:" + multiplierEn as string)
				
			gtoolsLLister.maxLightsRC.addHandler (("LightMult" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #"+ multiplierProp + " val" + "\n" +\
					"LightMult" + gtoolsLLister.count as string + ".tooltip = val as string " + "\n" + \		-- update the tooltip
					" \n")	
			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightMult" + gtoolsLLister.count as string) as name)
				
		)			
			
		-- Light Color
		if hasColor do (
			local colorProp = case lightClassName of (
				#Free_Light:			#rgbFilter
				#Target_Light:		#rgbFilter
				#Free_Area:			#rgbFilter
				#Target_Area:		#rgbFilter
				#Free_Linear:			#rgbFilter
				#Target_Linear:		#rgbFilter
				#Free_Point:			#rgbFilter
				#Target_Point:		#rgbFilter
				#Free_Sphere:		#rgbFilter
				#Target_Sphere:		#rgbFilter
				#Free_Disc:			#rgbFilter
				#Target_Disc:			#rgbFilter
				#Free_Cylinder:		#rgbFilter
				#Target_Cylinder:	#rgbFilter
				#Luminaire:				#FilterColor
				#mr_sky_portal:		#rgbFilter	
				#CoronaMtl:			#colorSelfIllum
				#CoronaLightMtl:		#color
				default:					#Color
			)
			
			local colorEn = case lightClassName of 
				(
				#VRayLight:	 ((gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseobject.color_mode == 0) and
										(not gtoolsLLister.LightIndex[gtoolsLLister.count][1].skylightPortal))
				#VRayIES:		(gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseobject.color_mode == 0)
				#CoronaLight:		(gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseobject.colorMode == 0)

			--	#CoronaSun: 	(gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseobject.colorMode == 0)
				default:			true
				)
				
			
			gtoolsLLister.maxLightsRC.addControl #colorpicker (("LightCol" + gtoolsLLister.count as string) as name) "" \
				paramStr:("color:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] colorProp) as string + \
				" offset:[203,"+ (-23+ gtoolsLLister.yOffset) as string + "] width:25 enabled:" + colorEn as string)
			
				gtoolsLLister.maxLightsRC.addHandler (("LightCol" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #" + colorProp + " val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightCol" + gtoolsLLister.count as string) as name)
		)
	
		if hasShadow do (
			-- Shadow On/Off
			
			gtoolsLLister.maxLightsRC.addControl #checkbox (("LightShdOn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #castshadows as string)+ \
				" offset:[235,"+ (-22+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("LightShdOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #castshadows state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightShdOn" + gtoolsLLister.count as string) as name)
			
			-- Shadow Plugin
			local LLshadowClass = gtoolsLLister.fnShadowClass gtoolsLLister.LightIndex[gtoolsLLister.count][1]
			local LLshadowGen = (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #shadowGenerator)
			
			gtoolsLLister.maxLightsRC.addControl #dropDownList (("LightShd" + gtoolsLLister.count as string) as name) "" filter:on\
				paramStr:(" items:" + gtoolsLLister.ShadowPluginsName as string + " width:110 offset:[255,"+ (-24+ gtoolsLLister.yOffset) as string + "]" + \
				"selection:(finditem gtoolsLLister.ShadowPlugins (gtoolsLLister.fnShadowClass gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1]))")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightShd" + gtoolsLLister.count as string) as name)
			
			-- Light Map Size
			local mapSizeTmp = \
				case LLshadowClass of (
					shadowMap:					LLshadowGen.mapSize
					mental_ray_shadow_map:	LLshadowGen.Map_Size
					VRayShadowMap:			LLshadowGen.resolution
					default:							512
				)
				
			local hasShadowMapSize = LLshadowClass == shadowMap or LLShadowClass == mental_ray_shadow_map or
													LLshadowClass == VRayShadowMap
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightMapSiz" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,10000," + (mapSizeTmp as string) + "] type:#integer " + \
				"fieldwidth:45 align:#left offset:[375,"+ (-24+ gtoolsLLister.yOffset) as string + "] enabled:" \
				+ hasShadowMapSize as string)
			gtoolsLLister.maxLightsRC.addHandler (("LightMapSiz" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr: \
				(
				"local propname = case (gtoolsLLister.fnShadowClass gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1]) of\n" + \
				"(shadowMap:#mapSize; mental_ray_shadow_map:#Map_Size; VRayShadowMap:#resolution; default:0)\n" + \
				"if propname != 0 do gtoolsLLister.SetShdProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] propName val"
				)
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightMapSiz" + gtoolsLLister.count as string) as name)
			
			-- Light Bias
			local BiasTmp = \
				case LLshadowClass of (
					shadowMap:				LLShadowGen.mapBias
					raytraceShadow:			LLShadowGen.raytraceBias
					Area_Shadows:			LLShadowGen.ray_Bias
					Adv__Ray_Traced:		LLShadowGen.ray_Bias
					Blur_Adv__Ray_Traced:	LLShadowGen.ray_Bias
					VRayShadow:				LLShadowGen.bias
					VRayShadowMap:		LLShadowGen.bias
					default:						1.0
				)
			
			local hasShadowBias = LLShadowClass == shadowMap or LLShadowClass == raytraceShadow or
												LLShadowClass == Blur_Adv__Ray_Traced or LLShadowClass == Area_Shadows or
												LLShadowClass == Adv__Ray_Traced or LLShadowClass == VRayShadow or
												LLShadowClass == VRayShadowMap
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightBias" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,10000," + (BiasTmp as string) + "] type:#float " + \
				"fieldwidth:45 align:#left offset:[433,"+ (-21+ gtoolsLLister.yOffset) as string + "] enabled:" \
				+ hasShadowBias as string)
			gtoolsLLister.maxLightsRC.addHandler (("LightBias" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr: \
				(
				"local propname = case (gtoolsLLister.fnShadowClass gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1]) of\n" + \
				"(shadowMap:#mapbias; raytraceShadow:#raytraceBias; Area_Shadows:#ray_bias; Adv__Ray_Traced:#ray_bias;" + \
					"Blur_Adv__Ray_Traced:#ray_bias; VRayShadow:#bias; VRayShadowMap:#bias; default:0)\n" + \
				"if propname != 0 do gtoolsLLister.SetShdProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] propName val"
				)
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightBias" + gtoolsLLister.count as string) as name)
			
			-- Light Sample Range
			local smpRangeTmp = 4.0
			local smpRangeStr = "#samplerange"   --fix the mr shadow sample problem here.
			if LLShadowClass == shadowMap  then	smpRangeTmp = LLShadowGen.samplerange
			else if LLShadowClass == mental_ray_Shadow_Map do (
				smpRangeTmp = LLShadowGen.Sample_Range
				smpRangeStr = "#Sample_Range"
			)
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightSmpRange" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,50," + (smpRangeTmp as string) + "] type:#float " + \
				"fieldwidth:45 align:#left offset:[491,"+ (-21+ gtoolsLLister.yOffset) as string + "] enabled:" + (LLShadowClass == shadowMap or LLShadowClass == mental_ray_shadow_map) as string)
			gtoolsLLister.maxLightsRC.addHandler (("LightSmpRange" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.SetShdProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] "+smpRangeStr+" val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightSmpRange" + gtoolsLLister.count as string) as name)
			
			-- Transparency On/Off
			gtoolsLLister.maxLightsRC.addControl #checkbox (("LightTrans" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + \
						((if LLShadowClass == Area_Shadows or LLShadowClass == Adv__Ray_Traced or LLShadowClass == Blur_Adv__Ray_Traced then \
						LLShadowGen.shadow_Transparent else false) as string) + \
						" offset:[553,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15 enabled:" + \
						((LLShadowClass == Area_Shadows or LLShadowClass == Adv__Ray_Traced or LLShadowClass == Blur_Adv__Ray_Traced) as string))
			gtoolsLLister.maxLightsRC.addHandler (("LightTrans" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setShdProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #shadow_Transparent state")
			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightTrans" + gtoolsLLister.count as string) as name)
			
			-- Integrity
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightInteg" + gtoolsLLister.count as string) as name) "" \
				paramStr:("type:#integer fieldwidth:30 align:#left range:[1,15," + \
						((if LLShadowClass == Area_Shadows or LLShadowClass == Blur_Adv__Ray_Traced or\
						LLShadowClass == Adv__Ray_Traced then \
						LLShadowGen.pass1 else 1) as string) + \
						"] offset:[566,"+ (-21+ gtoolsLLister.yOffset) as string + "] width:15 enabled:" + \
						((LLShadowClass == Area_Shadows or LLShadowClass == Blur_Adv__Ray_Traced or\
						LLShadowClass == Adv__Ray_Traced) as string))
			gtoolsLLister.maxLightsRC.addHandler (("LightInteg" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setShdProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #pass1 val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightInteg" + gtoolsLLister.count as string) as name)
			
			-- Quality
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightQual" + gtoolsLLister.count as string) as name) "" \
				paramStr:("type:#integer fieldwidth:30 align:#left range:[1,15," + \
						((if LLShadowClass == Area_Shadows or LLShadowClass == Blur_Adv__Ray_Traced or \
						LLShadowClass == Adv__Ray_Traced then \
						LLShadowGen.pass2 else 2) as string) + \
						"] offset:[610,"+ (-21+ gtoolsLLister.yOffset) as string + "] width:15 enabled:" + \
						((LLShadowClass == Area_Shadows or LLShadowClass == Blur_Adv__Ray_Traced or \
						LLShadowClass == Adv__Ray_Traced) as string))
			gtoolsLLister.maxLightsRC.addHandler (("LightQual" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setShdProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #pass2 val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightQual" + gtoolsLLister.count as string) as name)
			
			-- Shadow Plugin dropdown handler
			gtoolsLLister.maxLightsRC.addHandler (("LightShd" + gtoolsLLister.count as string) as name) #'selected i' filter:on \
				codeStr:(\
					"gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #shadowGenerator (gtoolsLLister.ShadowPlugins[i]());" + \
					"local shdClass = gtoolsLLister.fnShadowClass gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1]\n" + \
					"LightMapSiz" + gtoolsLLister.count as string + ".enabled = LightSmpRange" + gtoolsLLister.count as string + ".enabled = " + \
						"(shdClass == shadowMap or shdClass == mental_ray_shadow_map or shdClass == VRayShadowMap)\n" + \
					"LightTrans" + gtoolsLLister.count as string + ".enabled = LightInteg" + gtoolsLLister.count as string + ".enabled = LightQual" + gtoolsLLister.count as string + ".enabled = " + \
						"shdClass == Adv__Ray_Traced or shdClass == Blur_Adv__Ray_Traced or shdClass == Area_Shadows\n" + \
					"LightBias" + gtoolsLLister.count as string + ".enabled = (shdClass == Area_Shadows or shdClass == shadowMap or " + \
						"shdClass == Blur_Adv__Ray_Traced or shdClass == raytraceShadow or shdClass ==  Adv__Ray_Traced or " + \
						"shdClass == VRayShadow or shdClass == VRayShadowMap)\n" + \
					"if (val = gtoolsLLister.getShdProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #mapSize) != undefined do LightMapSiz" + \
						gtoolsLLister.count as string + ".value = val\n" + \
					"if (val = gtoolsLLister.getShdProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #resolution) != undefined do LightMapSiz" + \
						gtoolsLLister.count as string + ".value = val\n" + \
					"if (val = gtoolsLLister.getShdProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #sampleRange) != undefined do LightSmpRange" + \
						gtoolsLLister.count as string + ".value = val\n" + \
					"if (val = gtoolsLLister.getShdProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #pass1) != undefined do LightInteg" + \
						gtoolsLLister.count as string + ".value = val\n" + \
					"if (val = gtoolsLLister.getShdProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #pass2) != undefined do LightQual" + \
						gtoolsLLister.count as string + ".value = val\n" + \
					"if (val = gtoolsLLister.getShdProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #mapBias) != undefined do LightBias" + \
						gtoolsLLister.count as string + ".value = val\n" + \
					"if (val = gtoolsLLister.getShdProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #ray_Bias) != undefined do LightBias" + \
						gtoolsLLister.count as string + ".value = val\n" + \
					"if (val = gtoolsLLister.getShdProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #raytraceBias) != undefined do LightBias" + \
						gtoolsLLister.count as string + ".value = val\n" + \
					"if (val = gtoolsLLister.getShdProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #bias) != undefined do LightBias" + \
						gtoolsLLister.count as string + ".value = val\n" + \
					"if (val = gtoolsLLister.getShdProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #shadow_Transparent) != undefined do LightTrans" + \
						gtoolsLLister.count as string + ".checked = val\n" + \
					"gtoolsLLister.enableRefreshBtn gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1]"
					)
		) -- end has Shadow
		
		if hasDecay do
		(
			-- Decay selection
			gtoolsLLister.maxLightsRC.addControl #dropDownList (("LightDecay" + gtoolsLLister.count as string) as name) "" filter:on\
				paramStr:(" items:" + gtoolsLLister.decayStrings as string + " width:80 offset:[657,"+ (-24+ gtoolsLLister.yOffset) as string + "]" + \
				"selection:(gtoolsLLister.getLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #attenDecay)")
			gtoolsLLister.maxLightsRC.addHandler (("LightDecay" + gtoolsLLister.count as string) as name) #'selected i' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #attenDecay i")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightDecay" + gtoolsLLister.count as string) as name)
			
			-- Decay Start
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightDecStart" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,10000," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #decayRadius) as string) + "] type:#worldunits " + \
				"fieldwidth:45 align:#left offset:[735,"+ (-24+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightDecStart" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #decayRadius val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightDecStart" + gtoolsLLister.count as string) as name)
		) -- end hasDecay
		
		if hasSize do
		(
			local lightType = gtoolsLLister.LightIndex[gtoolsLLister.count][1].type
	--		format "lightType:% \n" lightType
			-- Light Length
			local hasLength = lightType == #Free_Line or lightType == #Target_Line or 
										lightType == #Free_Rectangle or lightType == #Target_Rectangle or 
										lightType == #Free_Cylinder or lightType == #Target_Cylinder or
										lightType == #Free_Area or lightType == #Target_Area or
										lightType == #Free_Linear or lightType == #Target_Linear
			
			local lengthTmp = case lightType of (
				#Free_Line:			gtoolsLLister.LightIndex[gtoolsLLister.count][1].length
				#Target_Line:			gtoolsLLister.LightIndex[gtoolsLLister.count][1].length
				#Free_Rectangle:		gtoolsLLister.LightIndex[gtoolsLLister.count][1].length
				#Target_Rectangle:	gtoolsLLister.LightIndex[gtoolsLLister.count][1].length
				#Free_Cylinder:		gtoolsLLister.LightIndex[gtoolsLLister.count][1].length
				#Target_Cylinder:	gtoolsLLister.LightIndex[gtoolsLLister.count][1].length
				#Free_Area:			gtoolsLLister.LightIndex[gtoolsLLister.count][1].light_length
				#Target_Area:			gtoolsLLister.LightIndex[gtoolsLLister.count][1].light_length
				#Free_Linear:			gtoolsLLister.LightIndex[gtoolsLLister.count][1].light_length
				#Target_Linear:		gtoolsLLister.LightIndex[gtoolsLLister.count][1].light_length
				default:					48
			)
			local lengthStr = case lightType of (
				#Free_Line:			"length"
				#Target_Line:			"length"
				#Free_Rectangle:		"length"
				#Target_Rectangle:	"length"
				#Free_Cylinder:		"length"
				#Target_Cylinder:	"length"
				#Free_Linear:			"light_length"
				#Target_Area:			"light_length"
				#Free_Area:			"light_length"
				#Target_Linear:		"light_length"
				default:					"length"
			)
			
			gtoolsLLister.maxLightsRC.addControl #spinner (("LSLightLength" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,100000," + (lengthTmp as string) + "] type:#worldunits " + \
				"fieldwidth:45 align:#left offset:[655,"+ (-21+ gtoolsLLister.yOffset) as string + "] enabled:" \
				+ (hasLength as string))
			gtoolsLLister.maxLightsRC.addHandler (("LSLightLength" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1]." + lengthStr + " = val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LSLightLength" + gtoolsLLister.count as string) as name)
			
			-- Light Width / Radius
			local hasWidth = lightType == #Free_Rectangle or lightType == #Target_Rectangle or
										lightType == #Free_Area or lightType == #Target_Area or
										lightType == #Target_Point or lightType == #VRayLight_Plane
			
			local widthTmp = case lightType of (
				#Free_Rectangle:		gtoolsLLister.LightIndex[gtoolsLLister.count][1].width
				#Target_Rectangle:	gtoolsLLister.LightIndex[gtoolsLLister.count][1].width
				#Free_Area:			gtoolsLLister.LightIndex[gtoolsLLister.count][1].light_width
				#Target_Area:			gtoolsLLister.LightIndex[gtoolsLLister.count][1].light_width
				#VRayLight_Plane:	gtoolsLLister.LightIndex[gtoolsLLister.count][1].size1
				default:					24
			)
			
			local widthStr = case lightType of (
				#Free_Rectangle:		"width"
				#Target_Rectangle:	"width"
				#Free_Area:			"light_width"
				#Target_Area:			"light_width"
				#VRayLight_Plane:	"size1"
				default:					"width"
			)
			
			local hasRadius = lightType == #Free_Cylinder or lightType == #Target_Cylinder or 
										lightType == #Free_Disc or lightType == #Target_Disc or 
										lightType == #Free_Sphere or lightType == #Target_Sphere
			
			local radiusTmp = case lightType of (
				#Free_Cylinder:		gtoolsLLister.LightIndex[gtoolsLLister.count][1].radius
				#Target_Cylinder:	gtoolsLLister.LightIndex[gtoolsLLister.count][1].radius
				#Free_Disc:			gtoolsLLister.LightIndex[gtoolsLLister.count][1].radius
				#Target_Disc:			gtoolsLLister.LightIndex[gtoolsLLister.count][1].radius
				#Free_Sphere:		gtoolsLLister.LightIndex[gtoolsLLister.count][1].radius
				#Target_Sphere:		gtoolsLLister.LightIndex[gtoolsLLister.count][1].radius
				default:					5.5
			)
			
			local radiusStr = case lightType of (
				#Free_Cylinder:		"radius"
				#Target_Cylinder:	"radius"
				#Free_Disc:			"radius"
				#Target_Disc:			"radius"
				#Free_Sphere:		"radius"
				#Target_Sphere:		"radius"
				#VRayLight:			"size0"
				default:					"radius"
			)
			
			if (hasWidth) then (
				gtoolsLLister.maxLightsRC.addControl #spinner (("LSLightWidth" + gtoolsLLister.count as string) as name) "" \
					paramStr:("range:[0,100000," + (widthTmp as string) + "] type:#worldunits " + \
					"fieldwidth:45 align:#left offset:[714,"+ (-21+ gtoolsLLister.yOffset) as string + "] enabled:" \
					+ (hasWidth as string))
				gtoolsLLister.maxLightsRC.addHandler (("LSLightWidth" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
					codeStr:("gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1]." + widthStr + " = val")
			)
			else (
				gtoolsLLister.maxLightsRC.addControl #spinner (("LSLightWidth" + gtoolsLLister.count as string) as name) "" \
					paramStr:("range:[0,100000," + (radiusTmp as string) + "] type:#worldunits " + \
					"fieldwidth:45 align:#left offset:[714,"+ (-21+ gtoolsLLister.yOffset) as string + "] enabled:" \
					+ (hasRadius as string))
				
				if (hasRadius) do (
					gtoolsLLister.maxLightsRC.addHandler (("LSLightWidth" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
						codeStr:("gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1]." + radiusStr + " = val")
				)
			)
			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LSLightWidth" + gtoolsLLister.count as string) as name)
		) -- end hasSize
		
		-- Viewport wire color
		if hasWireColor do (
			local wireColorX=case lightClassName of (
				#VRayLight:         916
				#VRayIES:           805
				#VRayAmbientLight:  301
				default:            827
			)

			-- Wire color
			local wireColorOnTmp = (gtoolsLLister.LightIndex[gtoolsLLister.count][1].viewport_wire_color_on)
			gtoolsLLister.maxLightsRC.addControl #checkbox (("WireColorOn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + wireColorOnTmp as string + " offset:[" + wireColorX as string + ","+ (-23+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("WireColorOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #viewport_wire_color_on state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("WireColorOn" + gtoolsLLister.count as string) as name)
			
			wireColorX+=15
			local wireColorTmp = (gtoolsLLister.LightIndex[gtoolsLLister.count][1].viewport_wire_color)
			gtoolsLLister.maxLightsRC.addControl #colorpicker (("WireColor" + gtoolsLLister.count as string) as name) "" \
				paramStr:("color:" + wireColorTmp as string + " offset:[" + wireColorX as string + ","+ (-23+ gtoolsLLister.yOffset) as string + "] width:25")
			gtoolsLLister.maxLightsRC.addHandler (("WireColor" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #viewport_wire_color val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("WireColor" + gtoolsLLister.count as string) as name)
		) -- end hasWireColor

		if hasIconText do (
			local hasTextX=case lightClassName of (
				#VRayLight: 975
				#VRayIES: 868
				default: 885
			)
			local iconTextOnTmp = (gtoolsLLister.LightIndex[gtoolsLLister.count][1].icon_text)
			gtoolsLLister.maxLightsRC.addControl #checkbox (("IconTextOn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + iconTextOnTmp as string + " offset:[" + hasTextX as string + ","+ (-23+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("IconTextOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #icon_text state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("IconTextOn" + gtoolsLLister.count as string) as name)
		) -- end hasIconText

		if isMRSky do
		(
			-- Haze
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightHaze" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,15," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #haze) as string) + "] type:#float " + \
				"fieldwidth:45 align:#left offset:[203,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightHaze" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #haze val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightHaze" + gtoolsLLister.count as string) as name)

			-- HorizonHeight
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightHHeight" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[-10,10," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #HorizonHeight) as string) + "] type:#float " + \
				"fieldwidth:45 align:#left offset:[261,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightHHeight" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #HorizonHeight val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightHHeight" + gtoolsLLister.count as string) as name)

			-- HorizonBlur
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightHBlur" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,10," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #HorizonBlur) as string) + "] type:#float " + \
				"fieldwidth:45 align:#left offset:[319,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightHBlur" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #HorizonBlur val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightHBlur" + gtoolsLLister.count as string) as name)

			-- GroundColor
			gtoolsLLister.maxLightsRC.addControl #colorpicker (("LightGroundCol" + gtoolsLLister.count as string) as name) "" \
				paramStr:("color:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #GroundColor) as string + \
				" offset:[380,"+ (-23+ gtoolsLLister.yOffset) as string + "] width:25")
			gtoolsLLister.maxLightsRC.addHandler (("LightGroundCol" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #GroundColor val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightGroundCol" + gtoolsLLister.count as string) as name)
				
			-- NightColor
			gtoolsLLister.maxLightsRC.addControl #colorpicker (("LightNightCol" + gtoolsLLister.count as string) as name) "" \
				paramStr:("color:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #NightColor) as string + \
				" offset:[425,"+ (-25+ gtoolsLLister.yOffset) as string + "] width:25")
			gtoolsLLister.maxLightsRC.addHandler (("LightNightCol" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #NightColor val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightNightCol" + gtoolsLLister.count as string) as name)
				
			-- Redness
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightRedness" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[-1,1," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #Redness) as string) + "] type:#float " + \
				"fieldwidth:45 align:#left offset:[464,"+ (-23+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightRedness" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #Redness val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightRedness" + gtoolsLLister.count as string) as name)

			-- Saturation
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightSaturation" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,2," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #Saturation) as string) + "] type:#float " + \
				"fieldwidth:45 align:#left offset:[522,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightSaturation" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #Saturation val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightSaturation" + gtoolsLLister.count as string) as name)

			-- UseAerialPerspective
			gtoolsLLister.maxLightsRC.addControl #checkbox (("LightUseAerPersp" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #UseAerialPerspective as string)+ \
				" offset:[615,"+ (-21+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("LightUseAerPersp" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #UseAerialPerspective state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightUseAerPersp" + gtoolsLLister.count as string) as name)

			-- AerialPerspective
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightAerialPersp" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,1e8," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #AerialPerspective) as string) + "] type:#worldunits " + \
				"fieldwidth:60 align:#left offset:[663,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightAerialPersp" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #AerialPerspective val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightAerialPersp" + gtoolsLLister.count as string) as name)
		)
		
		if isMRSun do
		(
			-- Shadow On/Off
			gtoolsLLister.maxLightsRC.addControl #checkbox (("LightShdOn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #castshadows as string)+ \
				" offset:[220,"+ (-22+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("LightShdOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #castshadows state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightShdOn" + gtoolsLLister.count as string) as name)

			-- shadowSoftness
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightShdSoft" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,50," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #shadowSoftness) as string) + "] type:#float " + \
				"fieldwidth:35 align:#left offset:[255,"+ (-20+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightShdSoft" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #shadowSoftness val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightShdSoft" + gtoolsLLister.count as string) as name)

			-- shadowSamples
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightShdSamples" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,1000," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #shadowSamples) as string) + "] type:#integer " + \
				"fieldwidth:35 align:#left offset:[304,"+ (-20+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightShdSamples" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #shadowSamples val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightShdSamples" + gtoolsLLister.count as string) as name)

			-- hasTarget
			gtoolsLLister.maxLightsRC.addControl #checkbox (("LightHasTarg" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #hasTarget as string)+ \
				" offset:[365,"+ (-21+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("LightHasTarg" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #hasTarget state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightHasTarg" + gtoolsLLister.count as string) as name)

			-- targetDistance
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightTargDist" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,1e8," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseobject #targetDistance) as string) + "] type:#worldunits " + \
				"fieldwidth:60 align:#left offset:[397,"+ (-20+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightTargDist" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #targetDistance val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightTargDist" + gtoolsLLister.count as string) as name)

			-- inherit
			gtoolsLLister.maxLightsRC.addControl #checkbox (("LightInherit" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #inherit as string)+ \
				" offset:[480,"+ (-21+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("LightInherit" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #inherit state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightInherit" + gtoolsLLister.count as string) as name)

			-- haze
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightHaze" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,15," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #haze) as string) + "] type:#float " + \
				"fieldwidth:35 align:#left offset:[508,"+ (-20+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightHaze" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #haze val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightHaze" + gtoolsLLister.count as string) as name)

			-- horizonheight
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightHHeight" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[-10,10," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #horizonheight) as string) + "] type:#float " + \
				"fieldwidth:35 align:#left offset:[558,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightHHeight" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #horizonheight val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightHHeight" + gtoolsLLister.count as string) as name)

			-- redblueshift
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightRedness" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[-1,1," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #redblueshift) as string) + "] type:#float " + \
				"fieldwidth:35 align:#left offset:[608,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightRedness" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #redblueshift val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightRedness" + gtoolsLLister.count as string) as name)

			-- saturation
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightSaturation" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,2," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #saturation) as string) + "] type:#float " + \
				"fieldwidth:35 align:#left offset:[658,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightSaturation" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #saturation val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightSaturation" + gtoolsLLister.count as string) as name)

			-- usephotontarget
			gtoolsLLister.maxLightsRC.addControl #checkbox (("LightUsePTarg" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #usephotontarget as string)+ \
				" offset:[723,"+ (-21+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("LightUsePTarg" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #usephotontarget state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightUsePTarg" + gtoolsLLister.count as string) as name)

			-- photontarget
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightPhotonTarg" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,1e8," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #photontarget) as string) + "] type:#worldunits " + \
				"fieldwidth:60 align:#left offset:[745,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightPhotonTarg" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #photontarget val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightPhotonTarg" + gtoolsLLister.count as string) as name)
		)
		
		if isMRSkyPortal do
		(
			-- Shadow On/Off
			gtoolsLLister.maxLightsRC.addControl #checkbox (("LightShdOn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #castshadows as string)+ \
				" offset:[255,"+ (-22+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("LightShdOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #castshadows state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightShdOn" + gtoolsLLister.count as string) as name)

			-- extend_shadows
			gtoolsLLister.maxLightsRC.addControl #checkbox (("LightExtShad" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #extend_shadows as string)+ \
				" offset:[295,"+ (-21+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("LightExtShad" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #extend_shadows state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightExtShad" + gtoolsLLister.count as string) as name)

			-- shadowSamples
			gtoolsLLister.maxLightsRC.addControl #dropDownList (("LightShdSamples" + gtoolsLLister.count as string) as name) "" filter:on\
				paramStr:(" items:" + gtoolsLLister.MRSkyPortal_ShadowSamples as string + " width:52 offset:[323,"+ (-23+ gtoolsLLister.yOffset) as string + "]" + \
				"selection:" + (((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #shadowSamples) +1) as string))
			gtoolsLLister.maxLightsRC.addHandler (("LightShdSamples" + gtoolsLLister.count as string) as name) #'selected i' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #shadowSamples (i-1)")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightShdSamples" + gtoolsLLister.count as string) as name)

			-- length
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightLength" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,1e8," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #length) as string) + "] type:#worldunits " + \
				"fieldwidth:60 align:#left offset:[376,"+ (-24+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightLength" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #length val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightLength" + gtoolsLLister.count as string) as name)

			-- light_width
			gtoolsLLister.maxLightsRC.addControl #spinner (("LightWidth" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,1e8," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #light_width) as string) + "] type:#worldunits " + \
				"fieldwidth:60 align:#left offset:[451,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("LightWidth" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #light_width val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightWidth" + gtoolsLLister.count as string) as name)

				-- reversed
			gtoolsLLister.maxLightsRC.addControl #checkbox (("LightFlipFlux" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #reversed as string)+ \
				" offset:[535,"+ (-21+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("LightFlipFlux" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #reversed state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightFlipFlux" + gtoolsLLister.count as string) as name)

				-- Area_Visible
			gtoolsLLister.maxLightsRC.addControl #checkbox (("LightAreaVisible" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #Area_Visible as string)+ \
				" offset:[580,"+ (-21+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("LightAreaVisible" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #Area_Visible state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightAreaVisible" + gtoolsLLister.count as string) as name)

			-- transparency
			gtoolsLLister.maxLightsRC.addControl #colorpicker (("LightTransparency" + gtoolsLLister.count as string) as name) "" \
				paramStr:("color:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #transparency) as string + \
				" offset:[625,"+ (-23+ gtoolsLLister.yOffset) as string + "] width:25")
			gtoolsLLister.maxLightsRC.addHandler (("LightTransparency" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #transparency val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightTransparency" + gtoolsLLister.count as string) as name)
				
			-- mode
			gtoolsLLister.maxLightsRC.addControl #dropDownList (("LightSourceMode" + gtoolsLLister.count as string) as name) "" filter:on\
				paramStr:(" items:" + gtoolsLLister.MRSkyPortal_Modes as string + " width:70 offset:[685,"+ (-24+ gtoolsLLister.yOffset) as string + "]" + \
				"selection:" + (((mod ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #mode) + 1) 3) +1 ) as string))
			gtoolsLLister.maxLightsRC.addHandler (("LightSourceMode" + gtoolsLLister.count as string) as name) #'selected i' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #mode (mod (i+1) 3)")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightSourceMode" + gtoolsLLister.count as string) as name)
		)
		
		if isCoronaLightMat do
		(
			-- Texture
			CompleteFileName = (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #texmapSelfIllum)
			NameOnly = ""
			if CompleteFileName == undefined then -- check if there is an ies file
				CompleteFileName = ""
			else (
				if classof CompleteFileName == bitmapTexture then (
				CompleteFileName = CompleteFileName.filename
				NameOnly = filenameFromPath CompleteFileName
				) else (
				NameOnly = CompleteFileName.name
				CompleteFileName = CompleteFileName as string
				
				)
			)
			
			gtoolsLLister.maxLightsRC.addControl #button (("textureFile" + gtoolsLLister.count as string) as name) (NameOnly) \
				paramStr:(" offset:[240,"+ (-26+ gtoolsLLister.yOffset) as string + "] width:130 border:true align:#left tooltip:\" " + CompleteFileName + " \" ")
			

 			gtoolsLLister.maxLightsRC.addHandler (("textureFile" + gtoolsLLister.count as string) as name) #'pressed ' filter:on \
			 				codeStr:(" \n" + \
										"thefile = gt_myTools.loadBitmapTexture() \n" + \ -- load the new file
										"gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #texmap thefile \n" + \ -- update the material
										"NameOnly = filenameFromPath thefile.filename \n" + \	-- get the filename only fot the button text
										"textureFile" + gtoolsLLister.count as string + ".text = NameOnly " + "\n" + \	-- update the button text
										"textureFile" + gtoolsLLister.count as string + ".tooltip = thefile.filename " + "\n" + \		-- update the button tooltip
										" \n")							
						
 			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("textureFile" + gtoolsLLister.count as string) as name)	

			isEmiting = (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #emitLight as string)		
					
			-- Emit Both Sides
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CmSEmitBSides" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #twosidedEmission as string) + \
				" offset:[430,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15 enabled:" + isEmiting as string )				
			gtoolsLLister.maxLightsRC.addHandler (("CmSEmitBSides" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #twosidedEmission state")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CmSEmitBSides" + gtoolsLLister.count as string) as name)			
		
			-- Occlude other lights
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CmOccludeL" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #occludeOther as string) + \
				" offset:[580,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15 enabled:" + isEmiting as string )				
			gtoolsLLister.maxLightsRC.addHandler (("CmOccludeL" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #occludeOther state")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CmOccludeL" + gtoolsLLister.count as string) as name)	
		
			-- Shadow Catcher
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CmShwCatcher" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #shadowcatcherIlluminator as string) + \
				" offset:[650,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15 enabled:" + isEmiting as string )				
			gtoolsLLister.maxLightsRC.addHandler (("CmShwCatcher" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #shadowcatcherIlluminator state")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CmShwCatcher" + gtoolsLLister.count as string) as name)	
			
			--directionality	
			local dirEn = false 
			if lightType == 1 or lightType == 2  then
				dirEn = true
			gtoolsLLister.maxLightsRC.addControl #spinner (("CrDirectionality" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0.0,1.0," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #directionality) as string) + "] type:#float " + \
				"fieldwidth:40 align:#left offset:[480,"+ (-21+ gtoolsLLister.yOffset) as string + "] enabled:" + isEmiting as string )				
			gtoolsLLister.maxLightsRC.addHandler (("CrDirectionality" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].directionality = val" )			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrDirectionality" + gtoolsLLister.count as string) as name)	
			
			
			-- Emit Light
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CmSEmitLight" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #emitLight as string) + \
				" offset:[390,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")				
			gtoolsLLister.maxLightsRC.addHandler (("CmSEmitLight" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("\n" + \
							"gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #emitLight state \n" + \
							"CmSEmitBSides" + gtoolsLLister.count as string + ".enabled = state " + "\n" + \		-- update the state 
							"CrDirectionality" + gtoolsLLister.count as string + ".enabled = state " + "\n" + \		-- update the state 
							"CmOccludeL" + gtoolsLLister.count as string + ".enabled = state " + "\n" + \		-- update the state 
							"CmShwCatcher" + gtoolsLLister.count as string + ".enabled = state " + "\n" + \		-- update the state 
							" \n")						
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CmSEmitLight" + gtoolsLLister.count as string) as name)		
		
			-- Visibly directly
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CmVisDirect" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #visibleDirect as string) + \
				" offset:[710,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15 enabled:" + isEmiting as string )				
			gtoolsLLister.maxLightsRC.addHandler (("CmVisDirect" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #visibleDirect state")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CmVisDirect" + gtoolsLLister.count as string) as name)	
		
			-- Visibly reflections
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CmVisRef" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #visibleRefl as string) + \
				" offset:[765,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15 enabled:" + isEmiting as string )				
			gtoolsLLister.maxLightsRC.addHandler (("CmVisRef" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #visibleRefl state")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CmVisRef" + gtoolsLLister.count as string) as name)	
		
			-- Visibly refractions
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CmVisRefra" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #visibleRefract as string) + \
				" offset:[810,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15 enabled:" + isEmiting as string )				
			gtoolsLLister.maxLightsRC.addHandler (("CmVisRefra" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #visibleRefract state")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CmVisRefra" + gtoolsLLister.count as string) as name)				
			
		)
		
		if isCoronaMat do
		(
			-- Texture
			CompleteFileName = (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #texmapSelfIllum)
			NameOnly = ""
			if CompleteFileName == undefined then -- check if there is an ies file
				CompleteFileName = ""
			else (
				if classof CompleteFileName == bitmapTexture then (
				CompleteFileName = CompleteFileName.filename
				NameOnly = filenameFromPath CompleteFileName
				) else (
				NameOnly = CompleteFileName.name
				CompleteFileName = CompleteFileName as string
				
				)
			)
			
			gtoolsLLister.maxLightsRC.addControl #button (("textureFile" + gtoolsLLister.count as string) as name) (NameOnly) \
				paramStr:(" offset:[240,"+ (-26+ gtoolsLLister.yOffset) as string + "] width:130 border:true align:#left tooltip:\" " + CompleteFileName + " \" ")
			

 			gtoolsLLister.maxLightsRC.addHandler (("textureFile" + gtoolsLLister.count as string) as name) #'pressed ' filter:on \
			 				codeStr:(" \n" + \
										"thefile = gt_myTools.loadBitmapTexture() \n" + \ -- load the new file
										"gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #texmapSelfIllum thefile \n" + \ -- update the material
										"NameOnly = filenameFromPath thefile.filename \n" + \	-- get the filename only fot the button text
										"textureFile" + gtoolsLLister.count as string + ".text = NameOnly " + "\n" + \	-- update the button text
										"textureFile" + gtoolsLLister.count as string + ".tooltip = thefile.filename " + "\n" + \		-- update the button tooltip
										" \n")							
						
 			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("textureFile" + gtoolsLLister.count as string) as name)	
		)
		
		if isCoronaLight do
		(
			gtoolsLLister.maxLightsRC.addControl #checkbox (("DirectInput" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #colorMode) == 1) as string) + \
				" offset:[233,"+ (-22+ gtoolsLLister.yOffset) as string + "] width:15")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("DirectInput" + gtoolsLLister.count as string) as name)
			
			-- Color Temperature
			gtoolsLLister.maxLightsRC.addControl #spinner (("Temperature" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,30000.0," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #blackbodyTemp) as string) + "] type:#float " + \
				"fieldwidth:45 align:#left offset:[247,"+ (-21+ gtoolsLLister.yOffset) as string + "]" + \
				" enabled:" + (gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseobject.colorMode == 1) as string)			
			gtoolsLLister.maxLightsRC.addHandler (("Temperature" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #blackbodyTemp val")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("Temperature" + gtoolsLLister.count as string) as name)
			
			gtoolsLLister.maxLightsRC.addHandler (("DirectInput" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #colorMode (if state then 1 else 0) \n" + \
								"Temperature" + gtoolsLLister.count as string + ".enabled = state \n" + \
								"LightCol" + gtoolsLLister.count as string + ".enabled = not state \n" + \
								"LightCol" + gtoolsLLister.count as string + ".color = gtoolsLLister.getLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #color")
			
			-- Power Units
			gtoolsLLister.maxLightsRC.addControl #dropDownList (("Units" + gtoolsLLister.count as string) as name) "" filter:on\
				paramStr:(" items:#(\"Default: W/(sr.m^2)\", \"Lumen (lm)\", \"Candela (cd)\", \"Lux (lx)\")" + \
								" width:125 offset:[307,"+ (-24+ gtoolsLLister.yOffset) as string + "] selection:" + \
								(((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #intensityUnits)+1) as string) + \
								" enabled:true")
			gtoolsLLister.maxLightsRC.addHandler (("Units" + gtoolsLLister.count as string) as name) #'selected i' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #intensityUnits (i-1) \n" + \
								"LightMult" + gtoolsLLister.count as string + ".value = gtoolsLLister.getLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #intensity")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("Units" + gtoolsLLister.count as string) as name)
			

			-- figure out the type of light, this is used so we can enable/disable controls
			local enableHeight = false
			local lightType = (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #shape)  -- 0 = Sphere = disable , 1 = Rectangle = enable, 2 = Disk = disable, 3 = Cylinder = enable
			--format "lightType:% \n" lightType
			if lightType == 0 or lightType == 2 then 
				enableHeight = false
			else
				enableHeight = true		
		
			-- width/radius
			gtoolsLLister.maxLightsRC.addControl #spinner (("CrWidth" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,100000," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #width) as string) + "] type:#worldunits " + \
				"fieldwidth:55 align:#left offset:[525,"+ (-25+ gtoolsLLister.yOffset) as string + "] "+\
				"tooltip: \"" + ((units.formatValue (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #width)) as string ) + "\" " + \ 
				"enabled:true" )				
			gtoolsLLister.maxLightsRC.addHandler (("CrWidth" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].width = val"+ "\n" +\
							"CrWidth" + gtoolsLLister.count as string + ".tooltip = (units.formatValue val) as string " + "\n" + \		-- update the tooltip
							" \n")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("Width" + gtoolsLLister.count as string) as name)	
			
			--height			
			gtoolsLLister.maxLightsRC.addControl #spinner (("CrHeight" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,100000," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #height) as string) + "] type:#worldunits " + \
				"fieldwidth:55 align:#left offset:[596,"+ (-21+ gtoolsLLister.yOffset) as string + "] " +\
				"tooltip: \"" + ((units.formatValue (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #height)) as string ) + "\" " + \ 
				"enabled:"+ enableHeight as string )
			gtoolsLLister.maxLightsRC.addHandler (("CrHeight" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].height = val" + "\n" +\
							"CrHeight" + gtoolsLLister.count as string + ".tooltip = (units.formatValue val) as string " + "\n" + \		-- update the tooltip
							" \n")					
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrHeight" + gtoolsLLister.count as string) as name)		

			
			--segments		
			local segEn = false 
			if lightType == 0 or lightType == 2 or lightType == 3 then
				segEn = true
			
			gtoolsLLister.maxLightsRC.addControl #spinner (("CrSegments" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,100000," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #segments) as string) + "] type:#integer " + \
				"fieldwidth:45 align:#left offset:[667,"+ (-21+ gtoolsLLister.yOffset) as string + "] enabled:"+ segEn as string )				
			gtoolsLLister.maxLightsRC.addHandler (("CrSegments" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].segments = val" )			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrSegments" + gtoolsLLister.count as string) as name)			

			--directionality	
			local dirEn = false 
			if lightType == 1 or lightType == 2  then
				dirEn = true
			gtoolsLLister.maxLightsRC.addControl #spinner (("CrDirectionality" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0.0,1.0," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #directionality) as string) + "] type:#float " + \
				"fieldwidth:40 align:#left offset:[735,"+ (-21+ gtoolsLLister.yOffset) as string + "] enabled:" + dirEn as string )				
			gtoolsLLister.maxLightsRC.addHandler (("CrDirectionality" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].directionality = val" )			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrDirectionality" + gtoolsLLister.count as string) as name)	
			
			
			-- emit on both sides
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CrEmitBS" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #twosidedEmission as string) + \
				" offset:[810,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")				
			gtoolsLLister.maxLightsRC.addHandler (("CrEmitBS" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #twosidedEmission state")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrEmitBS" + gtoolsLLister.count as string) as name)
			
			
			-- since the shape setting sets enable/disable of some other controls it has to be created after them
			-- Shape			
			gtoolsLLister.maxLightsRC.addControl #dropDownList (("Shape" + gtoolsLLister.count as string) as name) "" filter:on\
				paramStr:(" items:#(\"Sphere\", \"Rectangle\", \"Disk\", \"Cylinder\")" + \
								" width:80 offset:[440,"+ (-24+ gtoolsLLister.yOffset) as string + "] selection:" + \
								(((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #shape)+1) as string) + \
								" enabled:true")
								
			gtoolsLLister.maxLightsRC.addHandler (("Shape" + gtoolsLLister.count as string) as name) #'selected i' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #Shape (i-1) \n" + \
					"tmpBool = " + "gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #Shape (i-1) \n" + "\n" +\
					"setHeight = true  \n" +\		
					"if tmpBool == 0 or tmpBool == 2 then  \n" + \
					"setHeight = false \n" +\
					"setSeg = false  \n" +\	
					"if tmpBool == 0 or tmpBool == 2 or tmpBool == 3 then  \n" + \
					"setSeg = true \n" +\
					"setDir = false  \n" +\	
					"if tmpBool == 1 or tmpBool == 2  then  \n" + \
					"setDir = true \n" +		
					"setEmit = false  \n" +\	
					"if tmpBool == 1 or tmpBool == 2  then  \n" + \
					"setEmit = true \n" +\				
			--		"format \"hola %\n\" " + "tmpBool" +  " \n" +\	
			--		"format \"setHeight: %\n\" " + "setHeight" +  " \n" +\	
					"CrEmitBS" + gtoolsLLister.count as string + ".enabled = setEmit " + "\n" + \		-- set the emit both sides enable/disable	
					"CrDirectionality" + gtoolsLLister.count as string + ".enabled = setDir " + "\n" + \		-- set the Directionality enable/disable	
					"CrSegments" + gtoolsLLister.count as string + ".enabled = setSeg " + "\n" + \		-- set the segments enable/disable		
					"CrHeight" + gtoolsLLister.count as string + ".enabled = setHeight " + "\n")			-- set the height enable/disable	

			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("Shape" + gtoolsLLister.count as string) as name)	

			-- Shadow Catcher
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CrShadowCatcher" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #shadowcatcherIlluminator as string) + \
				" offset:[865,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")				
			gtoolsLLister.maxLightsRC.addHandler (("CrShadowCatcher" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #shadowcatcherIlluminator state")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrShadowCatcher" + gtoolsLLister.count as string) as name)
			
			-- Visible directly
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CrVisibleDirectly" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #visibleDirectly as string) + \
				" offset:[925,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")				
			gtoolsLLister.maxLightsRC.addHandler (("CrVisibleDirectly" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #visibleDirectly state")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrVisibleDirectly" + gtoolsLLister.count as string) as name)
			
			-- Visible in reflections
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CrVisibleReflections" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #visibleRefl as string) + \
				" offset:[975,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")				
			gtoolsLLister.maxLightsRC.addHandler (("CrVisibleReflections" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #visibleRefl state")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrVisibleReflections" + gtoolsLLister.count as string) as name)
			
			-- Visible in refractions
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CrVisibleRefractions" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #visibleRefr as string) + \
				" offset:[1025,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")				
			gtoolsLLister.maxLightsRC.addHandler (("CrVisibleRefractions" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #visibleRefr state")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrVisibleRefractions" + gtoolsLLister.count as string) as name)

			-- Occlude Lights
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CrOccludeL" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #occludeOtherLights as string) + \
				" offset:[1080,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")				
			gtoolsLLister.maxLightsRC.addHandler (("CrOccludeL" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #occludeOtherLights state")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrOccludeL" + gtoolsLLister.count as string) as name)	
			
			-- Gizmo size
			gtoolsLLister.maxLightsRC.addControl #spinner (("CrGizmoSize" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0.0,99.0," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #gizmoDisplaySize) as string) + "] type:#float " + \
				"fieldwidth:40 align:#left offset:[1115,"+ (-21+ gtoolsLLister.yOffset) as string + "] enabled:true"  )				
			gtoolsLLister.maxLightsRC.addHandler (("CrGizmoSize" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].gizmoDisplaySize = val" )			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrGizmoSize" + gtoolsLLister.count as string) as name)				
			
			-- IES On
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CrIesOn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #iesOn as string) + \
				" offset:[1185,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")				
			gtoolsLLister.maxLightsRC.addHandler (("CrIesOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #iesOn state")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrIesOn" + gtoolsLLister.count as string) as name)		
			
	
			-- IES File
			iesCompleteFileName = (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #iesFile)
			if iesCompleteFileName == undefined then -- check if there is an ies file
				iesCompleteFileName = ""
			iesName = filenameFromPath iesCompleteFileName

			
			gtoolsLLister.maxLightsRC.addControl #button (("CrIesFile" + gtoolsLLister.count as string) as name) (iesName) \
				paramStr:(" offset:[1215,"+ (-24+ gtoolsLLister.yOffset) as string + "] width:100 border:true align:#left tooltip:\" " + iesCompleteFileName + " \" ")
			

 			gtoolsLLister.maxLightsRC.addHandler (("CrIesFile" + gtoolsLLister.count as string) as name) #'pressed ' filter:on \
			 				codeStr:(" \n" + \
										"ies = gt_myTools.loadIesFile() \n" + \ -- load the new ies file
										"if ies != undefined then ( \n" + \ 
										"gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #iesFile ies \n" + \ -- update the light
										"iesName = filenameFromPath ies \n" + \	-- get the filename only fot he button text
										"CrIesFile" + gtoolsLLister.count as string + ".text = iesName " + "\n" + \	-- update the button text
										"CrIesFile" + gtoolsLLister.count as string + ".tooltip = ies " + "\n" + \		-- update the button tooltip
										") \n")							
						
			
 			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrIesFile" + gtoolsLLister.count as string) as name)					
		)
		
		if isCoronaSun do
		(
				
			-- Sun Color
			gtoolsLLister.maxLightsRC.addControl #colorpicker (("LightCol" + gtoolsLLister.count as string) as name) "" \
				paramStr:("color:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #colorDirect) as string + \
				" offset:[203,"+ (-23+ gtoolsLLister.yOffset) as string + "] width:25 enabled:true" )
				
			gtoolsLLister.maxLightsRC.addHandler (("LightCol" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #colorDirect val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightCol" + gtoolsLLister.count as string) as name)
			
			-- Size
			gtoolsLLister.maxLightsRC.addControl #spinner (("CrSunSize" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0.0,64.0," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #sizeMultiplier) as string) + "] type:#float " + \
				"fieldwidth:40 align:#left offset:[250,"+ (-21+ gtoolsLLister.yOffset) as string + "] enabled:true"  )				
			gtoolsLLister.maxLightsRC.addHandler (("CrSunSize" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].sizeMultiplier = val" )			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrSunSize" + gtoolsLLister.count as string) as name)	
			
					
			-- Color Temperature
			local tempEn = false 
			local tmpColVal = (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #colorMode)
			if tmpColVal == 1  then
				tempEn = true
			
			gtoolsLLister.maxLightsRC.addControl #spinner (("Temperature" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,30000.0," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #blackbodyTemperature) as string) + "] type:#float " + \
				"fieldwidth:45 align:#left offset:[400,"+ (-25+ gtoolsLLister.yOffset) as string + "]" + \
				" enabled:" + tempEn as string)		
				
			gtoolsLLister.maxLightsRC.addHandler (("Temperature" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #blackbodyTemperature val")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("Temperature" + gtoolsLLister.count as string) as name)

			
			-- Color Mode		
			local tmpMode = (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #colorMode)
			local tmpResult 
			if tmpMode == 0 then tmpResult = 2 
				else if tmpMode == 1 then tmpResult = 0 
				else tmpResult = 1 
					
			gtoolsLLister.maxLightsRC.addControl #dropDownList (("CrColMode" + gtoolsLLister.count as string) as name) "" filter:on\
				paramStr:(" items:#(\"Realistic\", \"Direct Input\", \"Kelvin Temp\")" + \
								" width:80 offset:[310,"+ (-24+ gtoolsLLister.yOffset) as string + "] selection:" + \
								((tmpResult) as string) + \
								" enabled:true")
								
			gtoolsLLister.maxLightsRC.addHandler (("CrColMode" + gtoolsLLister.count as string) as name) #'selected i' filter:on \
				codeStr:("val = i-1 \n" + \
					"result = val \n" + \
					"if val == 0 then result = 2 \n" + \
					"else if val == 1 then result = 0 \n" + \
					"else result = 1 \n" + \
					"gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #colorMode (result) \n" + \
					"tmp = result  \n" +\				
					"setKelEnb = false  \n" +\	
					"if tmp == 1 then  \n" + \
					"setKelEnb = true  \n" +\
					"Temperature" + gtoolsLLister.count as string + ".enabled = setKelEnb " + "\n")			-- set the height enable/disable	
			
  			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrColMode" + gtoolsLLister.count as string) as name)	
			
					
			-- Visible directly
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CrVisibleDirectly" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #visibleDirectly as string) + \
				" offset:[490,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")				
			gtoolsLLister.maxLightsRC.addHandler (("CrVisibleDirectly" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #visibleDirectly state")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrVisibleDirectly" + gtoolsLLister.count as string) as name)
			
			-- Visible in reflections
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CrVisibleReflections" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #visibleRefl as string) + \
				" offset:[560,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")				
			gtoolsLLister.maxLightsRC.addHandler (("CrVisibleReflections" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #visibleRefl state")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrVisibleReflections" + gtoolsLLister.count as string) as name)
			
			-- Visible in refractions
			gtoolsLLister.maxLightsRC.addControl #checkbox (("CrVisibleRefractions" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #visibleRefr as string) + \
				" offset:[630,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")				
			gtoolsLLister.maxLightsRC.addHandler (("CrVisibleRefractions" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #visibleRefr state")			
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CrVisibleRefractions" + gtoolsLLister.count as string) as name)

			
		)
		
		if isVRayLight do 
			(
			-- Temperature/Color switch
			local colorEn = (not gtoolsLLister.LightIndex[gtoolsLLister.count][1].skylightPortal)
			gtoolsLLister.maxLightsRC.addControl #checkbox (("TemperatureEn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #color_mode) == 1) as string) + \
				" offset:[233,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15 enabled:" + colorEn as string)
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("TemperatureEn" + gtoolsLLister.count as string) as name)
			
			-- Color Temperature
			gtoolsLLister.maxLightsRC.addControl #spinner (("Temperature" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,30000.0," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #color_temperature) as string) + "] type:#float " + \
				"fieldwidth:45 align:#left offset:[247,"+ (-20+ gtoolsLLister.yOffset) as string + "]" + \
				" enabled:" + ((gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseobject.color_mode == 1) and colorEn) as string)
			gtoolsLLister.maxLightsRC.addHandler (("Temperature" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #color_temperature val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("Temperature" + gtoolsLLister.count as string) as name)
			gtoolsLLister.maxLightsRC.addHandler (("TemperatureEn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #color_mode (if state then 1 else 0) \n" + \
								"Temperature" + gtoolsLLister.count as string + ".enabled = state \n" + \
								"LightCol" + gtoolsLLister.count as string + ".enabled = not state \n" + \
								"LightCol" + gtoolsLLister.count as string + ".color = gtoolsLLister.getLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #color")
			
			-- Power Units
			local powerUnitsEn = (not gtoolsLLister.LightIndex[gtoolsLLister.count][1].skylightPortal)
			gtoolsLLister.maxLightsRC.addControl #dropDownList (("PowerUnits" + gtoolsLLister.count as string) as name) "" filter:on\
				paramStr:(" items:#(\"Default (image)\", \"Luminous power (lm)\", \"Luminance (lm/m?/sr)\", \"Radiant power (W)\", \"Radiance (W/m?/sr)\")" + \
								" width:100 offset:[307,"+ (-21+ gtoolsLLister.yOffset) as string + "] selection:" + \
								(((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #normalizeColor)+1) as string) + \
								" enabled:" + powerUnitsEn as string)
								
			gtoolsLLister.maxLightsRC.addHandler (("PowerUnits" + gtoolsLLister.count as string) as name) #'selected i' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #normalizeColor (i-1) \n" + \
								"LightMult" + gtoolsLLister.count as string + ".value = gtoolsLLister.getLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #multiplier")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("PowerUnits" + gtoolsLLister.count as string) as name)
			
			-- Shadow On/Off
			local shadowsEn = (not gtoolsLLister.LightIndex[gtoolsLLister.count][1].skylightPortal)
			
			gtoolsLLister.maxLightsRC.addControl #checkbox (("LightShdOn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #castShadows as string)+ \
				" offset:[421,"+ (-23+ gtoolsLLister.yOffset) as string + "] width:15 enabled:" + shadowsEn as string)
			gtoolsLLister.maxLightsRC.addHandler (("LightShdOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #castShadows state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightShdOn" + gtoolsLLister.count as string) as name)
			
			--Subdivs
			gtoolsLLister.maxLightsRC.addControl #spinner (("Subdivs" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[1,1000," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #subdivs) as string) + "] type:#integer " + \
				"fieldwidth:35 align:#left offset:[460,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("Subdivs" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #subdivs val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("Subdivs" + gtoolsLLister.count as string) as name)
			
			--Shadow Bias
			gtoolsLLister.maxLightsRC.addControl #spinner (("ShadowBias" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[-1000000000.0,1000000000.0," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #shadowBias) as string) + "] type:#worldunits " + \
				"fieldwidth:45 align:#left offset:[515,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("ShadowBias" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #shadowBias val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("ShadowBias" + gtoolsLLister.count as string) as name)
			
			-- Invisible
			local invisibleEn = (not gtoolsLLister.LightIndex[gtoolsLLister.count][1].skylightPortal)
			local invisibleTmp = (gtoolsLLister.LightIndex[gtoolsLLister.count][1].invisible)
			gtoolsLLister.maxLightsRC.addControl #checkbox (("InvisibleOn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + invisibleTmp as string + " offset:[595,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15" + \
				" enabled:" + invisibleEn as string)
			gtoolsLLister.maxLightsRC.addHandler (("InvisibleOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #invisible state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("InvisibleOn" + gtoolsLLister.count as string) as name)
			
			-- Skylight mode
			skylightTmp = if (gtoolsLLister.LightIndex[gtoolsLLister.count][1].skylightPortal) then (
									if (gtoolsLLister.LightIndex[gtoolsLLister.count][1].simplePortal) then (3)
									else (2))
								else (1)
			
			gtoolsLLister.maxLightsRC.addControl #dropDownList (("SkylightMode" + gtoolsLLister.count as string) as name) "" filter:on\
				paramStr:(" items:#(\"None\", \"Normal\", \"Simple\") width:60 offset:[630,"+ (-23+ gtoolsLLister.yOffset) as string + "]" + \
				"selection:" + (skylightTmp as string))
			gtoolsLLister.maxLightsRC.addHandler (("SkylightMode" + gtoolsLLister.count as string) as name) #'selected i' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #skylightPortal (i == 2 or i == 3) \n" + \
								"gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #simplePortal (i == 3) \n" + \
								"LightMult" + gtoolsLLister.count as string + ".enabled = (i == 1) \n" + \
								"LightCol" + gtoolsLLister.count as string + ".enabled = ((i == 1) and (not TemperatureEn" + gtoolsLLister.count as string +".checked)) \n" + \
								"TemperatureEn" + gtoolsLLister.count as string + ".enabled = (i == 1) \n" + \
								"Temperature" + gtoolsLLister.count as string + ".enabled = ((i == 1) and (TemperatureEn" + gtoolsLLister.count as string +".checked)) \n" + \
								"PowerUnits" + gtoolsLLister.count as string + ".enabled = (i == 1) \n" + \
								"LightShdOn" + gtoolsLLister.count as string + ".enabled = (i == 1) \n" + \
								"InvisibleOn" + gtoolsLLister.count as string + ".enabled = (i == 1)")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("SkylightMode" + gtoolsLLister.count as string) as name)
			
			-- Affect Diffuse
			gtoolsLLister.maxLightsRC.addControl #checkbox (("AffectDiffOn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #affect_diffuse as string) + \
				" offset:[705,"+ (-25+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("AffectDiffOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #affect_diffuse state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("AffectDiffOn" + gtoolsLLister.count as string) as name)
			
			-- Affect Specular
			gtoolsLLister.maxLightsRC.addControl #checkbox (("AffectSpecOn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #affect_specular as string) + \
				" offset:[740,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("AffectSpecOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #affect_specular state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("AffectSpecOn" + gtoolsLLister.count as string) as name)
			
			-- Affect Reflections
			local affectReflectTmp = (gtoolsLLister.LightIndex[gtoolsLLister.count][1].affect_reflections)
			gtoolsLLister.maxLightsRC.addControl #checkbox (("AffectReflectOn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + affectReflectTmp as string + " offset:[780,"+ (-21+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("AffectReflectOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #affect_reflections state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("AffectReflectOn" + gtoolsLLister.count as string) as name)
		)
		
		if isVRayIESLight do (
			-- Temperature/Color switch
			gtoolsLLister.maxLightsRC.addControl #checkbox (("TemperatureEn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #color_mode) == 1) as string) + \
				" offset:[233,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("TemperatureEn" + gtoolsLLister.count as string) as name)
			
			-- Color Temperature
			gtoolsLLister.maxLightsRC.addControl #spinner (("Temperature" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0,30000.0," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #color_temperature) as string) + "] type:#float " + \
				"fieldwidth:45 align:#left offset:[247,"+ (-20+ gtoolsLLister.yOffset) as string + "]" + \
				" enabled:" + (gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseobject.color_mode == 1) as string)
			gtoolsLLister.maxLightsRC.addHandler (("Temperature" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #color_temperature val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("Temperature" + gtoolsLLister.count as string) as name)
			gtoolsLLister.maxLightsRC.addHandler (("TemperatureEn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #color_mode (if state then 1 else 0) \n" + \
								"Temperature" + gtoolsLLister.count as string + ".enabled = state \n" + \
								"LightCol" + gtoolsLLister.count as string + ".enabled = not state \n" + \
								"LightCol" + gtoolsLLister.count as string + ".color = gtoolsLLister.getLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #color")
			
			-- Shadow On/Off
			gtoolsLLister.maxLightsRC.addControl #checkbox (("LightShdOn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #cast_shadows as string)+ \
				" offset:[321,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("LightShdOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #cast_shadows state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("LightShdOn" + gtoolsLLister.count as string) as name)
			
			--Subdivs
			local subdivsEn = ((gtoolsLLister.LightIndex[gtoolsLLister.count][1].use_light_shape)!=0)
			gtoolsLLister.maxLightsRC.addControl #spinner (("Subdivs" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[1,1000," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #shape_subdivs) as string) + "] type:#integer " + \
				"fieldwidth:45 align:#left offset:[360,"+ (-21+ gtoolsLLister.yOffset) as string + "] enabled:" + subdivsEn as string)
			gtoolsLLister.maxLightsRC.addHandler (("Subdivs" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #shape_subdivs val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("Subdivs" + gtoolsLLister.count as string) as name)
			
			--Shadow Bias
			gtoolsLLister.maxLightsRC.addControl #spinner (("ShadowBias" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[-1000000000.0,1000000000.0," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #shadow_bias) as string) + "] type:#worldunits " + \
				"fieldwidth:45 align:#left offset:[430,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("ShadowBias" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #shadow_bias val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("ShadowBias" + gtoolsLLister.count as string) as name)
			
			-- Affect Diffuse
			gtoolsLLister.maxLightsRC.addControl #checkbox (("AffectDiffOn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #affect_diffuse as string) + \
				" offset:[500,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("AffectDiffOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #affect_diffuse state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("AffectDiffOn" + gtoolsLLister.count as string) as name)
			
			-- Affect Specular
			gtoolsLLister.maxLightsRC.addControl #checkbox (("AffectSpecOn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #affect_specular as string) + \
				" offset:[530,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("AffectSpecOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #affect_specular state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("AffectSpecOn" + gtoolsLLister.count as string) as name)
			
			-- Use Light Shape
			local useLShapeTmp = (gtoolsLLister.LightIndex[gtoolsLLister.count][1].use_light_shape)
			gtoolsLLister.maxLightsRC.addControl #dropDownList (("UseLShape" + gtoolsLLister.count as string) as name) "" filter:on\
				paramStr:(" items:#(\"No\", \"Shadows only\", \"Illumination and shadows\")" + \
								" width:60 offset:[590,"+ (-22+ gtoolsLLister.yOffset) as string + "] selection:" + \
								(((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #use_light_shape)+1) as string))
			gtoolsLLister.maxLightsRC.addHandler (("UseLShape" + gtoolsLLister.count as string) as name) #'selected i' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #use_light_shape (i-1) \n" + \
								"Subdivs" + gtoolsLLister.count as string + ".enabled = " + "gtoolsLLister.getLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #use_light_shape!=0")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("UseLShape" + gtoolsLLister.count as string) as name)
			
			-- Area Speculars
			local areaSpecularsTmp = (gtoolsLLister.LightIndex[gtoolsLLister.count][1].area_speculars)
			gtoolsLLister.maxLightsRC.addControl #checkbox (("AreaSpecularsOn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + areaSpecularsTmp as string + " offset:[675,"+ (-23+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("AreaSpecularsOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #area_speculars state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("AreaSpecularsOn" + gtoolsLLister.count as string) as name)
			
			-- IES File
			iesCompleteFileName = (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #ies_file)
			if iesCompleteFileName == undefined then -- check if there is an ies file
				iesCompleteFileName = ""
			iesName = filenameFromPath iesCompleteFileName

			
			gtoolsLLister.maxLightsRC.addControl #button (("VrayIesFile" + gtoolsLLister.count as string) as name) (iesName) \
				paramStr:(" offset:[900,"+ (-24+ gtoolsLLister.yOffset) as string + "] width:100 border:true align:#left tooltip:\" " + iesCompleteFileName + " \" ")
			

 			gtoolsLLister.maxLightsRC.addHandler (("VrayIesFile" + gtoolsLLister.count as string) as name) #'pressed ' filter:on \
			 				codeStr:(" \n" + \
										"ies = gt_myTools.loadIesFile() \n" + \ -- load the new ies file
										"if ies != undefined then ( \n" + \ 
										"gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #ies_file ies \n" + \ -- update the light
										"iesName = filenameFromPath ies \n" + \	-- get the filename only fot he button text
										"VrayIesFile" + gtoolsLLister.count as string + ".text = iesName " + "\n" + \	-- update the button text
										"VrayIesFile" + gtoolsLLister.count as string + ".tooltip = ies " + "\n" + \		-- update the button tooltip
										") \n")							
						
			
 			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("VrayIesFile" + gtoolsLLister.count as string) as name)		
			
		)
		
		if isVRayAmbientLight do (
		)
		
		if isVRaySun do (
			--Size Multiplier
			gtoolsLLister.maxLightsRC.addControl #spinner (("SizeMultiplier" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0.0,1000000000.0," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #size_multiplier) as string) + "] type:#worldunits " + \
				"fieldwidth:45 align:#left offset:[203,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("SizeMultiplier" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #size_multiplier val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("SizeMultiplier" + gtoolsLLister.count as string) as name)
			
			--Shadow Subdivs
			gtoolsLLister.maxLightsRC.addControl #spinner (("Subdivs" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[1,1000," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #shadow_subdivs) as string) + "] type:#integer " + \
				"fieldwidth:45 align:#left offset:[261,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("Subdivs" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #shadow_subdivs val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("Subdivs" + gtoolsLLister.count as string) as name)
			
			--Shadow Bias
			gtoolsLLister.maxLightsRC.addControl #spinner (("ShadowBias" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[-1000000000.0,1000000000.0," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #shadow_bias) as string) + "] type:#worldunits " + \
				"fieldwidth:45 align:#left offset:[319,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("ShadowBias" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #shadow_bias val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("ShadowBias" + gtoolsLLister.count as string) as name)
			
			-- Invisible
			gtoolsLLister.maxLightsRC.addControl #checkbox (("InvisibleOn" + gtoolsLLister.count as string) as name) "" \
				paramStr:("checked:" + (gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1].baseObject #invisible as string) + \
								" offset:[387,"+ (-20+ gtoolsLLister.yOffset) as string + "] width:15")
			gtoolsLLister.maxLightsRC.addHandler (("InvisibleOn" + gtoolsLLister.count as string) as name) #'changed state' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1].baseobject #invisible state")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("InvisibleOn" + gtoolsLLister.count as string) as name)
			
			--Turbidity
			gtoolsLLister.maxLightsRC.addControl #spinner (("Turbidity" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[2.0,1000000000.0," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #turbidity) as string) + "] type:#float " + \
				"fieldwidth:45 align:#left offset:[417,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("Turbidity" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #turbidity val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("Turbidity" + gtoolsLLister.count as string) as name)
			
			--Ozone
			gtoolsLLister.maxLightsRC.addControl #spinner (("Ozone" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0.0,1.0," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #ozone) as string) + "] type:#float " + \
				"fieldwidth:45 align:#left offset:[475,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("Ozone" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #ozone val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("Ozone" + gtoolsLLister.count as string) as name)
			
			--Photon Emit Radius
			gtoolsLLister.maxLightsRC.addControl #spinner (("PhotonEmitRadius" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0.0,1000000000.0," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #photon_emit_radius) as string) + "] type:#worldunits " + \
				"fieldwidth:45 align:#left offset:[533,"+ (-21+ gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("PhotonEmitRadius" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #photon_emit_radius val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("PhotonEmitRadius" + gtoolsLLister.count as string) as name)
			
			--Sky Model
			gtoolsLLister.maxLightsRC.addControl #dropDownList (("SkyModel" + gtoolsLLister.count as string) as name) "" filter:on\
				paramStr:(" items:#(\"Preetham et al.\", \"CIE Clear\", \"CIE Overcast\") width:100 offset:[593,"+ (-24+ gtoolsLLister.yOffset) as string + "]" + \
								" selection:" + (((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #sky_model) + 1) as string))
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("SkyModel" + gtoolsLLister.count as string) as name)
			
			--Indirect Horizont Illumination
			gtoolsLLister.maxLightsRC.addControl #spinner (("HorizIllumination" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[0.0,50000.0," + ((gtoolsLLister.getLightProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #indirect_horiz_illum) as string) + "] type:#float " + \
				"fieldwidth:45 align:#left offset:[695,"+ (-24+ gtoolsLLister.yOffset) as string + "] enabled:" + (gtoolsLLister.LightIndex[gtoolsLLister.count][1].sky_model > 0) as string)
			gtoolsLLister.maxLightsRC.addHandler (("HorizIllumination" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #indirect_horiz_illum val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("HorizIllumination" + gtoolsLLister.count as string) as name)
			gtoolsLLister.maxLightsRC.addHandler (("SkyModel" + gtoolsLLister.count as string) as name) #'selected i' filter:on \
				codeStr:("gtoolsLLister.setLightProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #sky_model (i - 1) \n" + \
								"horizIllumination" + gtoolsLLister.count as string + ".enabled = (i > 1)")
		)
		
		--Caustics subdivs
		if (hasCausticsSubdivs) then (
			local causticSubdivsTmp=getUserProp gtoolsLLister.LightIndex[gtoolsLLister.count][1] #VRay_Caustics_Subdivs
			local causticSubdivsX=case lightClassName of (
				#VRayLight:			835
				#VRayIES:				720
				#VRayAmbientLight:	231
				#VRaySun:				755
				default:					793
			)
			local causticSubdivsY=case lightClassName of (
				#VRayLight:			-19
				#VRayIES:				-20
				#VRayAmbientLight:	-23
				#VRaySun:				-21
				default:					-21
			)
			if (causticSubdivsTmp==undefined) do (causticSubdivsTmp=1500)
			gtoolsLLister.maxLightsRC.addControl #spinner (("CausticSubdivs" + gtoolsLLister.count as string) as name) "" \
				paramStr:("range:[1,2000000000," + causticSubdivsTmp as string + "] type:#integer " + \
				"fieldwidth:45 align:#left offset:[" + (causticSubdivsX as String) + ","+ (causticSubdivsY + gtoolsLLister.yOffset) as string + "]")
			gtoolsLLister.maxLightsRC.addHandler (("CausticSubdivs" + gtoolsLLister.count as string) as name) #'changed val' filter:on \
				codeStr:("setUserProp gtoolsLLister.LightIndex[" + gtoolsLLister.count as string + "][1] #VRay_Caustics_Subdivs val")
			append gtoolsLLister.UIControlList[2][gtoolsLLister.Count] (("CausticSubdivs" + gtoolsLLister.count as string) as name)
		)
		

		

		if heapFree < 1000000 do heapsize += 1000000 -- AB Jun 20, 2002
	) -- end CreateControls

	local CanAddControls = true
	local LightCountLimit = 150 -- this sets the maximum number of lights displayed

	if gtoolsLLister.CoronaLightMatList.count > 0 and CanAddControls do
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #CoronaLightMatTitle "Corona Light Materials" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.coronaLightMats
		
		-- End Localization
		for x in 1 to gtoolsLLister.CoronaLightMatList.count where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
			append gtoolsLLister.LightIndex gtoolsLLister.CoronaLightMatList[x]

			createControls hasMultiplier:true hasColor:true isCoronaLightMat:true
			gtoolsLLister.count += 1
			gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		) -- end For i in VRay Lights
	)
	
	if gtoolsLLister.CoronaMatList.count > 0 and CanAddControls do
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #CoronaMatTitle "Corona Materials" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.coronaMats
		
		-- End Localization
		for x in 1 to gtoolsLLister.CoronaMatList.count where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
			append gtoolsLLister.LightIndex gtoolsLLister.CoronaMatList[x]

			createControls hasMultiplier:true hasColor:true isCoronaMat:true
			gtoolsLLister.count += 1
			gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		) -- end For i in VRay Lights
	)	
	
	if gtoolsLLister.CoronaLightsList.count > 0 and CanAddControls do
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #CoronaLightTitle "Corona Lights" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.coronaLights
		
		-- End Localization
		for x in 1 to gtoolsLLister.CoronaLightsList.count where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
			append gtoolsLLister.LightIndex gtoolsLLister.CoronaLightsList[x]

			createControls hasMultiplier:true hasColor:true isCoronaLight:true
			gtoolsLLister.count += 1
			gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		) -- end For i in VRay Lights
	)
	

	if gtoolsLLister.CoronaSunSkyLightsList.count > 0 and CanAddControls do
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #CoronaSunSkyLightTitle "Corona Sun/Sky Lights" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.coronaSunSkyLights
		
		-- End Localization
		for x in 1 to gtoolsLLister.CoronaSunSkyLightsList.count where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
			append gtoolsLLister.LightIndex gtoolsLLister.CoronaSunSkyLightsList[x]

			createControls hasMultiplier:true  isCoronaSun:true
			gtoolsLLister.count += 1
			gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		) -- end For i in VRay Lights
	)	
	
	if gtoolsLLister.VRayLightsList.count > 0 and CanAddControls do
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #VRayLightTitle "V-Ray Lights" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.vrayLights
		
		-- End Localization
		for x in 1 to gtoolsLLister.VRayLightsList.count where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
			append gtoolsLLister.LightIndex gtoolsLLister.VRayLightsList[x]
			createControls hasMultiplier:true hasColor:true isVRayLight:true hasCausticsSubdivs:true hasWireColor:true hasIconText:true
			gtoolsLLister.count += 1
			gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		) -- end For i in VRay Lights
	)
	
	if gtoolsLLister.VRayIESLightsList.count > 0 and CanAddControls do
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #VRayIESLightTitle "V-Ray IES Lights" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.vrayIESLights
		
		-- End Localization
		for x in 1 to gtoolsLLister.VRayIESLightsList.count where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
			append gtoolsLLister.LightIndex gtoolsLLister.VRayIESLightsList[x]
			createControls hasMultiplier:true hasColor:true isVRayIESLight:true hasCausticsSubdivs:true hasWireColor:true hasIconText:true
			gtoolsLLister.count += 1
			gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		) -- end For i in VRay Lights
	)
	
	if gtoolsLLister.VRayAmbientLightsList.count > 0 and CanAddControls do
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #VRayAmbientLightTitle "V-Ray Ambient Lights" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.vrayAmbientLights
		
		-- End Localization
		for x in 1 to gtoolsLLister.VRayAmbientLightsList.count where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
			append gtoolsLLister.LightIndex gtoolsLLister.VRayAmbientLightsList[x]
			createControls hasMultiplier:true hasColor:true isVRayAmbientLight:true hasCausticsSubdivs:true hasWireColor:true
			gtoolsLLister.count += 1
			gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		) -- end For i in VRay Lights
	)
	
	if gtoolsLLister.VRaySunSkyLightsList.count > 0 and CanAddControls do
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #VRaySunSkyLightTitle "V-Ray SunSky Lights" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.vraySunSkyLights
		
		-- End Localization
		
		for x in 1 to gtoolsLLister.VRaySunSkyLightsList.count where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
			append gtoolsLLister.LightIndex gtoolsLLister.VRaySunSkyLightsList[x]
			createControls hasMultiplier:true isVRaySun:true hasCausticsSubdivs:true
			gtoolsLLister.count += 1
			gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		) -- end For i in VRay Lights
	)
	
	if gtoolsLLister.maxLightsList.count > 0 do
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #title "Standard Lights" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.maxLights
		
		-- End Localization
		
		for x in 1 to gtoolsLLister.maxLightsList.count where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
		
		append gtoolsLLister.LightIndex gtoolsLLister.maxLightsList[x]
		createControls hasMultiplier:true hasColor:true hasShadow:true hasDecay:true hasCausticsSubdivs:true
		gtoolsLLister.count += 1
		gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		
		) -- end For i in MAXLights
	) -- end MAXLights
	
	if gtoolsLLister.LSLightsList.count > 0 and CanAddControls do -- AB: Jun 20, 2002
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #LStitle "Photometric Lights" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.lsLights
		
		-- End Localization
		
		for x in 1 to gtoolsLLister.LSLightsList.count where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
		append gtoolsLLister.LightIndex gtoolsLLister.LSLightsList[x]
		createControls hasMultiplier:true hasColor:true hasShadow:true hasSize:true
		gtoolsLLister.count += 1
		gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		) -- end For i in LS Lights
	) -- end if LS Lights

	if gtoolsLLister.miLightsList.count > 0 and CanAddControls do -- AB: Jun 20, 2002
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #miLightstitle "mental ray Area Lights" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.miLights
		
		-- End Localization
		
		for x in 1 to gtoolsLLister.miLightsList.count  where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
		append gtoolsLLister.LightIndex gtoolsLLister.miLightsList[x]
		createControls hasMultiplier:true hasColor:true hasShadow:true hasDecay:true
		gtoolsLLister.count += 1
		gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		) -- end For i in miLightsList
	) -- end miLightsList

	if gtoolsLLister.LuminairesList.count > 0 and CanAddControls do -- AB: Jun 20, 2002
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #Luminairetitle "Luminaires" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.luminaires
		
		-- End Localization
		
		for x in 1 to gtoolsLLister.LuminairesList.count  where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
		append gtoolsLLister.LightIndex gtoolsLLister.LuminairesList[x]
		createControls hasMultiplier:true hasColor:true isLuminaire:true
		gtoolsLLister.count += 1
		gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		) -- end For i in LS Lights
	) -- end Luminaires

	if gtoolsLLister.SunLightsList.count > 0 and CanAddControls do
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #Suntitle "Sun Lights" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.sunLights
		
		-- End Localization
		
		for x in 1 to gtoolsLLister.SunLightsList.count where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
		append gtoolsLLister.LightIndex gtoolsLLister.SunLightsList[x]
		createControls hasMultiplier:true hasColor:true hasShadow:true
		gtoolsLLister.count += 1
		gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		) -- end For i in Sun Lights
	)


	if gtoolsLLister.SkyLightsList.count > 0 and CanAddControls do
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #Skytitle "Sky Lights" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.skyLights
		
		-- End Localization
		
		for x in 1 to gtoolsLLister.SkyLightsList.count where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
		append gtoolsLLister.LightIndex gtoolsLLister.SkyLightsList[x]
		createControls hasMultiplier:true hasColor:true
		gtoolsLLister.count += 1
		gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		) -- end For i in Sky Lights
	)
	
	if gtoolsLLister.mrSkyLightsList.count > 0 and CanAddControls do
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #mrSkytitle "mr Sky Lights" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.mrSkyLights
		
		-- End Localization
		
		for x in 1 to gtoolsLLister.mrSkyLightsList.count where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
		append gtoolsLLister.LightIndex gtoolsLLister.mrSkyLightsList[x]
		createControls hasMultiplier:true isMRSky:true
		gtoolsLLister.count += 1
		gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		) -- end For i in mr Sky Lights
	)
	
	if gtoolsLLister.mrSunLightsList.count > 0 and CanAddControls do
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #mrSuntitle "mr Sun Lights" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.mrSunLights
		
		-- End Localization
		
		for x in 1 to gtoolsLLister.mrSunLightsList.count where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
		append gtoolsLLister.LightIndex gtoolsLLister.mrSunLightsList[x]
		createControls hasMultiplier:true isMRSun:true
		gtoolsLLister.count += 1
		gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		) -- end For i in mr Sun Lights
	)
	
	if gtoolsLLister.mrSkyPortalLightsList.count > 0 and CanAddControls do
	(
		-- Start Localization
		
		gtoolsLLister.maxLightsRC.addControl #label #mrSkyPortaltitle "mr Sky Portal Lights" paramStr:" align:#left"
		
		WriteTitle labels:titleTemplates.mrSkyPortalLights
		
		-- End Localization
		
		for x in 1 to gtoolsLLister.mrSkyPortalLightsList.count where (CanAddControls = gtoolsLLister.count < LightCountLimit) do
		(
		append gtoolsLLister.LightIndex gtoolsLLister.mrSkyPortalLightsList[x]
		createControls hasMultiplier:true hasColor:true isMRSkyPortal:true
		gtoolsLLister.count += 1
		gtoolsLLister.LightInspectorSetup.pbar.value = gtoolsLLister.count*100/gtoolsLLister.totalLightCount
		) -- end For i in mr_Sky_Portal Lights
	)
	
	-- Callback Handlers
	
	gtoolsLLister.maxLightsRC.addHandler "maxLightsRollout" #'open' filter:off \
		codeStr:("gtoolsLLister.DeleteCallback = when gtoolsLLister.UIControlList[1] deleted obj do" + \
		"\n(\nlocal foundMe = findItem gtoolsLLister.UIControlList[1] obj\n" + \
		"if foundMe > 0 do\n(\n" + \
		"try(gtoolsLLister.disableUIElements gtoolsLLister.UIControlList[2][foundMe])catch()\n)\n)")

	gtoolsLLister.maxLightsRC.addHandler "maxLightsRollout" #'close' filter:off \
		codeStr:"DeleteChangeHandler gtoolsLLister.DeleteCallback"
		
	-- Removing the Refresh/ProgressBar
	
	gtoolsLLister.LightInspectorSetup.pbar.value = 0
	gtoolsLLister.LightInspectorSetup.pbar.visible = false
	
	-- AB: Jun 20, 2002
	-- Add a new control that tells users to use the selection mode if they had too many lights in the list
	
	if not CanAddControls and gtoolsLLister.maxLightsRC.str != "" do 
		gtoolsLLister.maxLightsRC.addControl #label #lbLimitControls "The maximum number of Lights has been reached, please select fewer lights and use the Selected Lights option" \
			paramStr:" align:#center offset:[0,10]"
	
	if gtoolsLLister.maxLightsRC.str != "" then gtoolsLLister.maxLightsRC.end() else undefined
)

gtoolsLLister.CreateLightRollout = CreateLightRollout

gtoolsLLister.GlobalLightParameters =
(local GlobalLightParameters
rollout GlobalLightParameters "General Settings"
(
	
	-- Start Localization
	
	radioButtons rbtoggle labels:#("Selected Lights","All Lights")
	
	local lblOffset = -18 + (if gtoolsLListeryOffset == undefined then 0 else gtoolsLListerYOffset)
	
	label lb01 "On" align:#left offset:[-6,-3]
	label lb03 "Multiplier"  align:#left offset:[12,lblOffset]
	label lb03a "Multiplier (%)" align:#left offset:[81,lblOffset] visible:(ProductAppID == #vizR)
	label lb04 "Color"  align:#left offset:[67,lblOffset]
	label lb05 "Shadows"  align:#left offset:[96,lblOffset]
	label lb06 "Map Size"  align:#left offset:[229,lblOffset] visible:(ProductAppID != #VIZR)
	label lb07 "Bias"  align:#left offset:[286,lblOffset] visible:(ProductAppID != #VIZR)
	label lb08 "Sm.Range"  align:#left offset:[337,lblOffset] visible:(ProductAppID != #VIZR)
	label lb09 "Trans."  align:#left offset:[390,lblOffset] visible:(ProductAppID != #VIZR)
	label lb10 "Int."  align:#left offset:[424,lblOffset] visible:(ProductAppID != #VIZR)
	label lb11 "Qual."  align:#left offset:[461,lblOffset] visible:(ProductAppID != #VIZR)
	label lb12 "Decay"  align:#left offset:[505,lblOffset] visible:(ProductAppID != #VIZR)
	label lb13 "Start"  align:#left offset:[586,lblOffset] visible:(ProductAppID != #VIZR)
	label lb14 "Length"  align:#left offset:[643,lblOffset] visible:(ProductAppID != #VIZR)
	label lb15 "Width/Radius"  align:#left offset:[699,lblOffset] visible:(ProductAppID != #VIZR)
	-- End Localization

	checkBox lightOn "" width:15 checked:true offset:[-4,0]
	spinner lightInten "" fieldWidth:45 type:#float range:[-10000,10000,1500] align:#left offset:[10,-20+ gtoolsLListeryOffset] visible:(ProductAppID == #vizR)
	checkBox lightMultOn "" width:15 checked:false offset:[81,-20+ gtoolsLListeryOffset] visible:(ProductAppID == #vizR)
	spinner lightMult "" fieldWidth:45 type:#float range:[-10000,10000,1.0] align:#left offset:[10,-20+ gtoolsLListeryOffset]
	colorPicker lightCol "" width:25 color:white offset:[66,-23+ gtoolsLListeryOffset]
	checkBox shadowOn "" width:15 checked:true offset:[96,-22+ gtoolsLListeryOffset]
	dropDownList shadowType width:115 items:gtoolsLLister.ShadowPluginsName offset:[113,-23+ gtoolsLListeryOffset] visible:(ProductAppID != #vizR)
	spinner ShadowMapSize "" fieldWidth:45 type:#integer range:[0,10000,512] align:#left offset:[227,-24+ gtoolsLListeryOffset] visible:(ProductAppID != #VIZR)
	spinner ShadowBias "" fieldWidth:45 type:#float range:[0,10000,0.5] align:#left offset:[284,-21+ gtoolsLListeryOffset] visible:(ProductAppID != #VIZR)
	spinner ShadowSmpRange "" fieldWidth:45 type:#float range:[0,50,4.0] align:#left offset:[341,-21+ gtoolsLListeryOffset] visible:(ProductAppID != #VIZR)
	checkBox shadowTrans "" width:15 offset:[401,-20+ gtoolsLListeryOffset] visible:(ProductAppID != #VIZR)
	spinner ShadowInteg "" fieldWidth:30 type:#integer range:[0,15,1] align:#left offset:[415,-21+ gtoolsLListeryOffset] visible:(ProductAppID != #VIZR)
	spinner ShadowQual 	"" fieldWidth:30 type:#Integer range:[0,15,2] align:#left offset:[459,-21+ gtoolsLListeryOffset] visible:(ProductAppID != #VIZR)
	dropDownList lightDecay width:80 items:gtoolsLLister.decayStrings offset:[504,-23+ gtoolsLListeryOffset] visible:(ProductAppID != #VIZR)
	spinner lightDecaySt "" fieldWidth:45 type:#float range:[0,10000,40] align:#left offset:[584,-24+ gtoolsLListeryOffset] visible:(ProductAppID != #VIZR)
	spinner lightLength "" fieldWidth:45 type:#float range:[0,10000,40] align:#left offset:[641,-21+ gtoolsLListeryOffset] visible:(ProductAppID != #VIZR)
	spinner lightWidth "" fieldWidth:45 type:#float range:[0,10000,40] align:#left offset:[697,-21+ gtoolsLListeryOffset] visible:(ProductAppID != #VIZR)

	group ""
	(
	
	-- Start Localization
	
	colorpicker gTint "Global Tint:" color:lightTintColor visible:(ProductAppID != #VIZR) offset:[180,0]
	spinner gLevel "Global Level:" range:[0,10000,lightLevel]  fieldWidth:45 align:#left offset:[290,-22+ gtoolsLListeryOffset] visible:(ProductAppID != #VIZR)
	colorPicker cpAmbient "Ambient Color" color:ambientColor offset:[420,-24+ gtoolsLListeryOffset] visible:(ProductAppID != #VIZR)
	
	-- End Localization
	
	)
	
	on GlobalLightParameters open do
	(
		if ProductAppID == #vizR do
		(
			lightmult.range = [0,1000000.0,100.0]
			lb03.text = "Intensity (cd)"
			lb04.pos = lb04.pos + [88,0]
			lb05.pos = lb05.pos + [87,0]
			lightmult.pos = lightMult.pos + [87,0]
			lightCol.pos = lightCol.pos + [88,0]
			shadowOn.pos = shadowOn.pos + [88,0]
		)
		dialogUp = true
	)
	
	on GlobalLightParameters close do
	(
		dialogUp = false
		updateToolbarButtons()
	)
	
	on gtint changed val do lightTintColor = val
	on glevel changed val do lightLevel = val
	on cpAmbient changed val do ambientColor = val
	
	fn setCollectionProperty prop val CreateUndo:true =
	(
		if createUndo then
		(
			undo "GtoolsLightLister" on 
			(
				local myCollection = if rbToggle.state == 1 then Selection else Lights
				for i in myCollection do 
				(
					setLightProp i.baseobject prop val
					setShdProp i.baseObject prop val
				)
			)
		)
		else
		(
			local myCollection = if rbToggle.state == 1 then Selection else Lights
			for i in myCollection do
			(
				setLightProp i.baseobject prop val
				setShdProp i.baseObject prop val
			)
		)
	)
	
	on lightOn changed state do setCollectionProperty #enabled state
	on lightCol changed val do 
	(
		setCollectionProperty #color val CreateUndo:false
		setCollectionProperty #filter_Color val CreateUndo:false
		setCollectionProperty #FilterColor val CreateUndo:false
	)
	on shadowOn changed state do setCollectionProperty #castShadows state
	on shadowTrans changed state do setCollectionProperty #shadow_transparent state
	on shadowInteg changed val do setCollectionProperty #pass1 val CreateUndo:false
	on shadowQual changed val do setCollectionProperty #pass2 val CreateUndo:false
	on lightWidth changed val do 
	(
		setCollectionProperty #light_Width val CreateUndo:false
		setCollectionProperty #light_Radius val CreateUndo:false
	)
	on lightLength changed val do setCollectionProperty #length val CreateUndo:false
	on lightMult changed val do 
	(
		setCollectionProperty #multiplier val CreateUndo:false
		setCollectionProperty #intensity val CreateUndo:false
		setCollectionProperty #dimmer val CreateUndo:false
		setCollectionProperty #power val CreateUndo:false
		setCollectionProperty #intensity_multiplier val CreateUndo:false
	)
	on lightMultOn changed state do setCollectionProperty #useMultiplier state
	on lightInten changed val do setCollectionProperty #intensity val CreateUndo:false
	on ShadowMapSize changed val do (
		setCollectionProperty #mapSize val CreateUndo:false
		setCollectionProperty #resolution val CreateUndo:false
	)
	on ShadowSmpRange changed val do setCollectionProperty #sampleRange val CreateUndo:false
	on lightDecaySt changed val do setCollectionProperty #decayRadius val CreateUndo:false
	on lightDecay selected d do setCollectionProperty #attenDecay d
	on shadowBias changed val do
	(
		setCollectionProperty #mapBias val CreateUndo:false
		setCollectionProperty #ray_Bias val CreateUndo:false
		setCollectionProperty #raytraceBias val CreateUndo:false
		setCollectionProperty #bias val CreateUndo:false
		setCollectionProperty #shadowBias val CreateUndo:false
		setCollectionProperty #shadow_bias val CreateUndo:false
	)
	
	on shadowType selected j do
	(
		local myCollection = if rbToggle.state == 1 then Selection else Lights
		for i in myCollection do setLightProp i.baseobject #shadowGenerator (gtoolsLLister.ShadowPlugins[j]())
	)

) -- end Rollout
) -- end structDef
local lblSelector = #("General Settings","All Lights","Selected Lights","Selection Set:") -- localize

gtoolsLLister.LightInspectorSetup =
(local LightInspectorSetup
rollout LightInspectorSetup "Configuration" -- Localize
(
	radiobuttons rolloutSelector labels:lblSelector across:2 align:#center offset:[300,0]
	dropdownlist setSelector width:100 align:#center offset:[-28,-2]
	checkbutton btnReload "Refresh" align:#right offset:[0,-24] height:16 highlightColor:(color ((colorman.getcolor #activecommand).x *255) ((colorman.getcolor #activecommand).y *255)((colorman.getcolor #activecommand).z *255)) checked:false -- Localize
	progressBar pbar width:134 pos:(btnReload.pos - [80,-1])
	
	local setChanged = false
	
	fn setSelectorUpdate selected:undefined = (
		local setNames = #()
		for i = 1 to selectionSets.count do (
			append setNames selectionSets[i].name
		)
		sort setNames
		setSelector.items = setNames
		
		if (selected != undefined) do (
			local lastIndex = findItem setSelector.items selected
			if (lastIndex == 0) then (lastIndex = 1)
			setSelector.selection = lastIndex
		)
	)
	
	on rolloutSelector changed state do
	(
		rolloutSelector.state = state
		case rolloutSelector.state of
		(
			1:	(
				try(RemoveRollout gtoolsLLister.GlobalLightParameters gtoolsLLister.LightInspectorFloater) catch()
				try(RemoveRollout gtoolsLLister.LightInspectorListRollout gtoolsLLister.LightInspectorFloater) catch()
				addRollout gtoolsLLister.GlobalLightParameters gtoolsLLister.LightInspectorFloater
				btnReload.visible = false
			)
			2:	(
				btnReload.visible = false
				try(RemoveRollout gtoolsLLister.GlobalLightParameters gtoolsLLister.LightInspectorFloater) catch()
				try(RemoveRollout gtoolsLLister.LightInspectorListRollout gtoolsLLister.LightInspectorFloater) catch()
				gtoolsLLister.LightInspectorListRollout = gtoolsLLister.CreateLightRollout (Lights as array + helpers as array)
				if gtoolsLLister.LightInspectorListRollout != undefined do
					addRollout gtoolsLLister.LightInspectorListRollout gtoolsLLister.LightInspectorFloater
				gtoolsLLister.maxLightsRC = undefined
				gc light:true
				btnReload.visible = true
			)
			3:	(
				btnReload.visible = false
				try(RemoveRollout gtoolsLLister.GlobalLightParameters gtoolsLLister.LightInspectorFloater) catch()
				try(RemoveRollout gtoolsLLister.LightInspectorListRollout gtoolsLLister.LightInspectorFloater) catch()
				gtoolsLLister.LightInspectorListRollout = gtoolsLLister.CreateLightRollout Selection
				if gtoolsLLister.LightInspectorListRollout != undefined do
					addRollout gtoolsLLister.LightInspectorListRollout gtoolsLLister.LightInspectorFloater
				gtoolsLLister.maxLightsRC = undefined
				gc light:true
				btnReload.visible = true
			)
			4: (
				btnReload.visible = false
				try(RemoveRollout gtoolsLLister.GlobalLightParameters gtoolsLLister.LightInspectorFloater) catch()
				try(RemoveRollout gtoolsLLister.LightInspectorListRollout gtoolsLLister.LightInspectorFloater) catch()
				if (setSelector.selection != 0) do (
					gtoolsLLister.LightInspectorListRollout = gtoolsLLister.CreateLightRollout (selectionSets[setSelector.selected])
					if gtoolsLLister.LightInspectorListRollout != undefined do
						addRollout gtoolsLLister.LightInspectorListRollout gtoolsLLister.LightInspectorFloater
				)
				gtoolsLLister.maxLightsRC = undefined
				gc light:true
				btnReload.visible = true
			)
		)
	)
	
	on btnReload changed state do
	(
		if (selectionSets.count > 0) do (
			local lastSetName = if (setSelector.selection> 0) then (setSelector.items[setSelector.selection]) else (undefined)
			setSelectorUpdate selected:lastSetName
		)
		
		rolloutSelector.changed rolloutSelector.state
		btnReload.checked = false
	)
	
	on setSelector selected i do (
		setChanged = true
		if (rolloutSelector.state == 4) do (
			rolloutSelector.changed rolloutSelector.state
		)
	)
	
	on LightInspectorSetup close do
	(
		callBacks.RemoveScripts id:#gtoolsLListerRollout
		setIniSetting "$plugCfg/gtoolsLLister.cfg" "General" "DialogPos" (gtoolsLLister.LightInspectorFloater.Pos as string) -- do not localize
		setIniSetting "$plugCfg/gtoolsLLister.cfg" "General" "DialogSize" (gtoolsLLister.LightInspectorFloater.Size as string) -- do not localize
		setIniSetting "$plugCfg/gtoolsLLister.cfg" "General" "LastState" (rolloutSelector.state as string) -- do not localize
		if (setChanged) do setIniSetting "$plugCfg/gtoolsLLister.cfg" "General" "LastSet" (setSelector.selected as string) -- do not localize
		
		dialogUp = false
		updateToolbarButtons()
	)
	
	on LightInspectorSetup open do
	(
		if (selectionSets.count > 0) do (
			local lastSetName = (getIniSetting "$plugCfg/gtoolsLLister.cfg" "General" "LastSet") as string
			setSelectorUpdate selected:lastSetName
		)
		
		pbar.visible = false
		local lastState = (getIniSetting "$plugCfg/gtoolsLLister.cfg" "General" "LastState") as integer  -- do not localize
		if lastState == 0 do lastState = 2
		if lastState < 5 do
			rolloutSelector.changed lastState
		
		-- Callbacks to remove Floater
		callBacks.AddScript #systemPreReset "CloseRolloutFloater gtoolsLLister.LightInspectorFloater" id:#gtoolsLListerRollout  -- do not localize
		callBacks.AddScript #systemPreNew "CloseRolloutFloater gtoolsLLister.LightInspectorFloater" id:#gtoolsLListerRollout -- do not localize
		callBacks.AddScript #filePreOpen "CloseRolloutFloater gtoolsLLister.LightInspectorFloater" id:#gtoolsLListerRollout -- do not localize
		
		dialogUp = true
		updateToolbarButtons()
	)
	
) -- end Rollout
) -- end StructDef

local enabled = true
fn checkRenderer = (
	productRenderer = renderers.production

	if(productRenderer != undefined) then (
		enabled = ((productRenderer.classid[1] == 1941615238) and (productRenderer.classid[2] == 2012806412) or (productRenderer.classid[1] == 1770671000) and (productRenderer.classid[2] == 1323107829))
	) else (
		enabled = false
	)
	
)
	
on execute do
	(

	-- Loading rollout size and position, if available
		
	local dialogPos, dialogSize
	
	dialogPos = execute (getIniSetting "$plugCfg/gtoolsLLister.cfg" "General" "DialogPos") -- Do not localize
	dialogSize = execute (getIniSetting "$plugCfg/gtoolsLLister.cfg" "General" "DialogSize") -- Do not localize
	
	if classof DialogPos != Point2 do dialogPos = [200,300]
	if classof DialogSize != Point2 do dialogSize = [1400,600]
	
	dialogSize.x = 1350 --940
	
	try(closeRolloutFloater gtoolsLLister.LightInspectorFloater) catch()
	theTitile = "Gtools Light Lister  " + ver
	gtoolsLLister.LightInspectorFloater = newRolloutFloater theTitile dialogSize.x dialogSize.y dialogPos.x dialogPos.y
	
	addRollout gtoolsLLister.LightInspectorSetup gtoolsLLister.LightInspectorFloater
	dialogUp = true
	)

-- on isEnabled do
-- 	(
-- 		checkRenderer()
-- 		enabled
-- 	)
-- on isVisible do
-- 	(
-- 		checkRenderer()
-- 		enabled
-- 	)
	
on closeDialogs do
	(
		try(closeRolloutFloater gtoolsLListergtoolsLLister.LightInspectorFloater) catch( print "Error in LightLister" )	
		dialogUp = false	
	)
on isChecked return
	(
		dialogUp
	)
)
